# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: ErrorMessage.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'ErrorMessage.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x45rrorMessage.proto\x12\x08protobuf\"\xd8\x01\n\x0c\x45rrorMessage\x12\x0f\n\x02id\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12\x16\n\terrorTime\x18\x02 \x01(\x03H\x01\x88\x01\x01\x12\x16\n\terrorCode\x18\x03 \x01(\x05H\x02\x88\x01\x01\x12\x15\n\x08\x65rrorMsg\x18\x04 \x01(\tH\x03\x88\x01\x01\x12$\n\x17\x61\x64vancedOrderRejectJson\x18\x05 \x01(\tH\x04\x88\x01\x01\x42\x05\n\x03_idB\x0c\n\n_errorTimeB\x0c\n\n_errorCodeB\x0b\n\t_errorMsgB\x1a\n\x18_advancedOrderRejectJsonB<\n\x16\x63om.ib.client.protobufB\x11\x45rrorMessageProto\xaa\x02\x0eIBApi.protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ErrorMessage_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.ib.client.protobufB\021ErrorMessageProto\252\002\016IBApi.protobuf'
  _globals['_ERRORMESSAGE']._serialized_start=33
  _globals['_ERRORMESSAGE']._serialized_end=249
# @@protoc_insertion_point(module_scope)
