// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: OrderAllocation.proto
// Protobuf Java Version: 4.29.3

package com.ib.client.protobuf;

public final class OrderAllocationProto {
  private OrderAllocationProto() {}
  static {
    com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
      com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
      /* major= */ 4,
      /* minor= */ 29,
      /* patch= */ 3,
      /* suffix= */ "",
      OrderAllocationProto.class.getName());
  }
  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistryLite registry) {
  }

  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistry registry) {
    registerAllExtensions(
        (com.google.protobuf.ExtensionRegistryLite) registry);
  }
  public interface OrderAllocationOrBuilder extends
      // @@protoc_insertion_point(interface_extends:protobuf.OrderAllocation)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>optional string account = 1;</code>
     * @return Whether the account field is set.
     */
    boolean hasAccount();
    /**
     * <code>optional string account = 1;</code>
     * @return The account.
     */
    java.lang.String getAccount();
    /**
     * <code>optional string account = 1;</code>
     * @return The bytes for account.
     */
    com.google.protobuf.ByteString
        getAccountBytes();

    /**
     * <code>optional string position = 2;</code>
     * @return Whether the position field is set.
     */
    boolean hasPosition();
    /**
     * <code>optional string position = 2;</code>
     * @return The position.
     */
    java.lang.String getPosition();
    /**
     * <code>optional string position = 2;</code>
     * @return The bytes for position.
     */
    com.google.protobuf.ByteString
        getPositionBytes();

    /**
     * <code>optional string positionDesired = 3;</code>
     * @return Whether the positionDesired field is set.
     */
    boolean hasPositionDesired();
    /**
     * <code>optional string positionDesired = 3;</code>
     * @return The positionDesired.
     */
    java.lang.String getPositionDesired();
    /**
     * <code>optional string positionDesired = 3;</code>
     * @return The bytes for positionDesired.
     */
    com.google.protobuf.ByteString
        getPositionDesiredBytes();

    /**
     * <code>optional string positionAfter = 4;</code>
     * @return Whether the positionAfter field is set.
     */
    boolean hasPositionAfter();
    /**
     * <code>optional string positionAfter = 4;</code>
     * @return The positionAfter.
     */
    java.lang.String getPositionAfter();
    /**
     * <code>optional string positionAfter = 4;</code>
     * @return The bytes for positionAfter.
     */
    com.google.protobuf.ByteString
        getPositionAfterBytes();

    /**
     * <code>optional string desiredAllocQty = 5;</code>
     * @return Whether the desiredAllocQty field is set.
     */
    boolean hasDesiredAllocQty();
    /**
     * <code>optional string desiredAllocQty = 5;</code>
     * @return The desiredAllocQty.
     */
    java.lang.String getDesiredAllocQty();
    /**
     * <code>optional string desiredAllocQty = 5;</code>
     * @return The bytes for desiredAllocQty.
     */
    com.google.protobuf.ByteString
        getDesiredAllocQtyBytes();

    /**
     * <code>optional string allowedAllocQty = 6;</code>
     * @return Whether the allowedAllocQty field is set.
     */
    boolean hasAllowedAllocQty();
    /**
     * <code>optional string allowedAllocQty = 6;</code>
     * @return The allowedAllocQty.
     */
    java.lang.String getAllowedAllocQty();
    /**
     * <code>optional string allowedAllocQty = 6;</code>
     * @return The bytes for allowedAllocQty.
     */
    com.google.protobuf.ByteString
        getAllowedAllocQtyBytes();

    /**
     * <code>optional bool isMonetary = 7;</code>
     * @return Whether the isMonetary field is set.
     */
    boolean hasIsMonetary();
    /**
     * <code>optional bool isMonetary = 7;</code>
     * @return The isMonetary.
     */
    boolean getIsMonetary();
  }
  /**
   * Protobuf type {@code protobuf.OrderAllocation}
   */
  public static final class OrderAllocation extends
      com.google.protobuf.GeneratedMessage implements
      // @@protoc_insertion_point(message_implements:protobuf.OrderAllocation)
      OrderAllocationOrBuilder {
  private static final long serialVersionUID = 0L;
    static {
      com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
        com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
        /* major= */ 4,
        /* minor= */ 29,
        /* patch= */ 3,
        /* suffix= */ "",
        OrderAllocation.class.getName());
    }
    // Use OrderAllocation.newBuilder() to construct.
    private OrderAllocation(com.google.protobuf.GeneratedMessage.Builder<?> builder) {
      super(builder);
    }
    private OrderAllocation() {
      account_ = "";
      position_ = "";
      positionDesired_ = "";
      positionAfter_ = "";
      desiredAllocQty_ = "";
      allowedAllocQty_ = "";
    }

    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return com.ib.client.protobuf.OrderAllocationProto.internal_static_protobuf_OrderAllocation_descriptor;
    }

    @java.lang.Override
    protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return com.ib.client.protobuf.OrderAllocationProto.internal_static_protobuf_OrderAllocation_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              com.ib.client.protobuf.OrderAllocationProto.OrderAllocation.class, com.ib.client.protobuf.OrderAllocationProto.OrderAllocation.Builder.class);
    }

    private int bitField0_;
    public static final int ACCOUNT_FIELD_NUMBER = 1;
    @SuppressWarnings("serial")
    private volatile java.lang.Object account_ = "";
    /**
     * <code>optional string account = 1;</code>
     * @return Whether the account field is set.
     */
    @java.lang.Override
    public boolean hasAccount() {
      return ((bitField0_ & 0x00000001) != 0);
    }
    /**
     * <code>optional string account = 1;</code>
     * @return The account.
     */
    @java.lang.Override
    public java.lang.String getAccount() {
      java.lang.Object ref = account_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        account_ = s;
        return s;
      }
    }
    /**
     * <code>optional string account = 1;</code>
     * @return The bytes for account.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getAccountBytes() {
      java.lang.Object ref = account_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        account_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int POSITION_FIELD_NUMBER = 2;
    @SuppressWarnings("serial")
    private volatile java.lang.Object position_ = "";
    /**
     * <code>optional string position = 2;</code>
     * @return Whether the position field is set.
     */
    @java.lang.Override
    public boolean hasPosition() {
      return ((bitField0_ & 0x00000002) != 0);
    }
    /**
     * <code>optional string position = 2;</code>
     * @return The position.
     */
    @java.lang.Override
    public java.lang.String getPosition() {
      java.lang.Object ref = position_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        position_ = s;
        return s;
      }
    }
    /**
     * <code>optional string position = 2;</code>
     * @return The bytes for position.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getPositionBytes() {
      java.lang.Object ref = position_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        position_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int POSITIONDESIRED_FIELD_NUMBER = 3;
    @SuppressWarnings("serial")
    private volatile java.lang.Object positionDesired_ = "";
    /**
     * <code>optional string positionDesired = 3;</code>
     * @return Whether the positionDesired field is set.
     */
    @java.lang.Override
    public boolean hasPositionDesired() {
      return ((bitField0_ & 0x00000004) != 0);
    }
    /**
     * <code>optional string positionDesired = 3;</code>
     * @return The positionDesired.
     */
    @java.lang.Override
    public java.lang.String getPositionDesired() {
      java.lang.Object ref = positionDesired_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        positionDesired_ = s;
        return s;
      }
    }
    /**
     * <code>optional string positionDesired = 3;</code>
     * @return The bytes for positionDesired.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getPositionDesiredBytes() {
      java.lang.Object ref = positionDesired_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        positionDesired_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int POSITIONAFTER_FIELD_NUMBER = 4;
    @SuppressWarnings("serial")
    private volatile java.lang.Object positionAfter_ = "";
    /**
     * <code>optional string positionAfter = 4;</code>
     * @return Whether the positionAfter field is set.
     */
    @java.lang.Override
    public boolean hasPositionAfter() {
      return ((bitField0_ & 0x00000008) != 0);
    }
    /**
     * <code>optional string positionAfter = 4;</code>
     * @return The positionAfter.
     */
    @java.lang.Override
    public java.lang.String getPositionAfter() {
      java.lang.Object ref = positionAfter_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        positionAfter_ = s;
        return s;
      }
    }
    /**
     * <code>optional string positionAfter = 4;</code>
     * @return The bytes for positionAfter.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getPositionAfterBytes() {
      java.lang.Object ref = positionAfter_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        positionAfter_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int DESIREDALLOCQTY_FIELD_NUMBER = 5;
    @SuppressWarnings("serial")
    private volatile java.lang.Object desiredAllocQty_ = "";
    /**
     * <code>optional string desiredAllocQty = 5;</code>
     * @return Whether the desiredAllocQty field is set.
     */
    @java.lang.Override
    public boolean hasDesiredAllocQty() {
      return ((bitField0_ & 0x00000010) != 0);
    }
    /**
     * <code>optional string desiredAllocQty = 5;</code>
     * @return The desiredAllocQty.
     */
    @java.lang.Override
    public java.lang.String getDesiredAllocQty() {
      java.lang.Object ref = desiredAllocQty_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        desiredAllocQty_ = s;
        return s;
      }
    }
    /**
     * <code>optional string desiredAllocQty = 5;</code>
     * @return The bytes for desiredAllocQty.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getDesiredAllocQtyBytes() {
      java.lang.Object ref = desiredAllocQty_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        desiredAllocQty_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int ALLOWEDALLOCQTY_FIELD_NUMBER = 6;
    @SuppressWarnings("serial")
    private volatile java.lang.Object allowedAllocQty_ = "";
    /**
     * <code>optional string allowedAllocQty = 6;</code>
     * @return Whether the allowedAllocQty field is set.
     */
    @java.lang.Override
    public boolean hasAllowedAllocQty() {
      return ((bitField0_ & 0x00000020) != 0);
    }
    /**
     * <code>optional string allowedAllocQty = 6;</code>
     * @return The allowedAllocQty.
     */
    @java.lang.Override
    public java.lang.String getAllowedAllocQty() {
      java.lang.Object ref = allowedAllocQty_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        allowedAllocQty_ = s;
        return s;
      }
    }
    /**
     * <code>optional string allowedAllocQty = 6;</code>
     * @return The bytes for allowedAllocQty.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getAllowedAllocQtyBytes() {
      java.lang.Object ref = allowedAllocQty_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        allowedAllocQty_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int ISMONETARY_FIELD_NUMBER = 7;
    private boolean isMonetary_ = false;
    /**
     * <code>optional bool isMonetary = 7;</code>
     * @return Whether the isMonetary field is set.
     */
    @java.lang.Override
    public boolean hasIsMonetary() {
      return ((bitField0_ & 0x00000040) != 0);
    }
    /**
     * <code>optional bool isMonetary = 7;</code>
     * @return The isMonetary.
     */
    @java.lang.Override
    public boolean getIsMonetary() {
      return isMonetary_;
    }

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      if (((bitField0_ & 0x00000001) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 1, account_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 2, position_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 3, positionDesired_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 4, positionAfter_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 5, desiredAllocQty_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 6, allowedAllocQty_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        output.writeBool(7, isMonetary_);
      }
      getUnknownFields().writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      if (((bitField0_ & 0x00000001) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(1, account_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(2, position_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(3, positionDesired_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(4, positionAfter_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(5, desiredAllocQty_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(6, allowedAllocQty_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeBoolSize(7, isMonetary_);
      }
      size += getUnknownFields().getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof com.ib.client.protobuf.OrderAllocationProto.OrderAllocation)) {
        return super.equals(obj);
      }
      com.ib.client.protobuf.OrderAllocationProto.OrderAllocation other = (com.ib.client.protobuf.OrderAllocationProto.OrderAllocation) obj;

      if (hasAccount() != other.hasAccount()) return false;
      if (hasAccount()) {
        if (!getAccount()
            .equals(other.getAccount())) return false;
      }
      if (hasPosition() != other.hasPosition()) return false;
      if (hasPosition()) {
        if (!getPosition()
            .equals(other.getPosition())) return false;
      }
      if (hasPositionDesired() != other.hasPositionDesired()) return false;
      if (hasPositionDesired()) {
        if (!getPositionDesired()
            .equals(other.getPositionDesired())) return false;
      }
      if (hasPositionAfter() != other.hasPositionAfter()) return false;
      if (hasPositionAfter()) {
        if (!getPositionAfter()
            .equals(other.getPositionAfter())) return false;
      }
      if (hasDesiredAllocQty() != other.hasDesiredAllocQty()) return false;
      if (hasDesiredAllocQty()) {
        if (!getDesiredAllocQty()
            .equals(other.getDesiredAllocQty())) return false;
      }
      if (hasAllowedAllocQty() != other.hasAllowedAllocQty()) return false;
      if (hasAllowedAllocQty()) {
        if (!getAllowedAllocQty()
            .equals(other.getAllowedAllocQty())) return false;
      }
      if (hasIsMonetary() != other.hasIsMonetary()) return false;
      if (hasIsMonetary()) {
        if (getIsMonetary()
            != other.getIsMonetary()) return false;
      }
      if (!getUnknownFields().equals(other.getUnknownFields())) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      if (hasAccount()) {
        hash = (37 * hash) + ACCOUNT_FIELD_NUMBER;
        hash = (53 * hash) + getAccount().hashCode();
      }
      if (hasPosition()) {
        hash = (37 * hash) + POSITION_FIELD_NUMBER;
        hash = (53 * hash) + getPosition().hashCode();
      }
      if (hasPositionDesired()) {
        hash = (37 * hash) + POSITIONDESIRED_FIELD_NUMBER;
        hash = (53 * hash) + getPositionDesired().hashCode();
      }
      if (hasPositionAfter()) {
        hash = (37 * hash) + POSITIONAFTER_FIELD_NUMBER;
        hash = (53 * hash) + getPositionAfter().hashCode();
      }
      if (hasDesiredAllocQty()) {
        hash = (37 * hash) + DESIREDALLOCQTY_FIELD_NUMBER;
        hash = (53 * hash) + getDesiredAllocQty().hashCode();
      }
      if (hasAllowedAllocQty()) {
        hash = (37 * hash) + ALLOWEDALLOCQTY_FIELD_NUMBER;
        hash = (53 * hash) + getAllowedAllocQty().hashCode();
      }
      if (hasIsMonetary()) {
        hash = (37 * hash) + ISMONETARY_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashBoolean(
            getIsMonetary());
      }
      hash = (29 * hash) + getUnknownFields().hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input);
    }

    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(com.ib.client.protobuf.OrderAllocationProto.OrderAllocation prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessage.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code protobuf.OrderAllocation}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessage.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:protobuf.OrderAllocation)
        com.ib.client.protobuf.OrderAllocationProto.OrderAllocationOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return com.ib.client.protobuf.OrderAllocationProto.internal_static_protobuf_OrderAllocation_descriptor;
      }

      @java.lang.Override
      protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return com.ib.client.protobuf.OrderAllocationProto.internal_static_protobuf_OrderAllocation_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                com.ib.client.protobuf.OrderAllocationProto.OrderAllocation.class, com.ib.client.protobuf.OrderAllocationProto.OrderAllocation.Builder.class);
      }

      // Construct using com.ib.client.protobuf.OrderAllocationProto.OrderAllocation.newBuilder()
      private Builder() {

      }

      private Builder(
          com.google.protobuf.GeneratedMessage.BuilderParent parent) {
        super(parent);

      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        bitField0_ = 0;
        account_ = "";
        position_ = "";
        positionDesired_ = "";
        positionAfter_ = "";
        desiredAllocQty_ = "";
        allowedAllocQty_ = "";
        isMonetary_ = false;
        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return com.ib.client.protobuf.OrderAllocationProto.internal_static_protobuf_OrderAllocation_descriptor;
      }

      @java.lang.Override
      public com.ib.client.protobuf.OrderAllocationProto.OrderAllocation getDefaultInstanceForType() {
        return com.ib.client.protobuf.OrderAllocationProto.OrderAllocation.getDefaultInstance();
      }

      @java.lang.Override
      public com.ib.client.protobuf.OrderAllocationProto.OrderAllocation build() {
        com.ib.client.protobuf.OrderAllocationProto.OrderAllocation result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public com.ib.client.protobuf.OrderAllocationProto.OrderAllocation buildPartial() {
        com.ib.client.protobuf.OrderAllocationProto.OrderAllocation result = new com.ib.client.protobuf.OrderAllocationProto.OrderAllocation(this);
        if (bitField0_ != 0) { buildPartial0(result); }
        onBuilt();
        return result;
      }

      private void buildPartial0(com.ib.client.protobuf.OrderAllocationProto.OrderAllocation result) {
        int from_bitField0_ = bitField0_;
        int to_bitField0_ = 0;
        if (((from_bitField0_ & 0x00000001) != 0)) {
          result.account_ = account_;
          to_bitField0_ |= 0x00000001;
        }
        if (((from_bitField0_ & 0x00000002) != 0)) {
          result.position_ = position_;
          to_bitField0_ |= 0x00000002;
        }
        if (((from_bitField0_ & 0x00000004) != 0)) {
          result.positionDesired_ = positionDesired_;
          to_bitField0_ |= 0x00000004;
        }
        if (((from_bitField0_ & 0x00000008) != 0)) {
          result.positionAfter_ = positionAfter_;
          to_bitField0_ |= 0x00000008;
        }
        if (((from_bitField0_ & 0x00000010) != 0)) {
          result.desiredAllocQty_ = desiredAllocQty_;
          to_bitField0_ |= 0x00000010;
        }
        if (((from_bitField0_ & 0x00000020) != 0)) {
          result.allowedAllocQty_ = allowedAllocQty_;
          to_bitField0_ |= 0x00000020;
        }
        if (((from_bitField0_ & 0x00000040) != 0)) {
          result.isMonetary_ = isMonetary_;
          to_bitField0_ |= 0x00000040;
        }
        result.bitField0_ |= to_bitField0_;
      }

      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof com.ib.client.protobuf.OrderAllocationProto.OrderAllocation) {
          return mergeFrom((com.ib.client.protobuf.OrderAllocationProto.OrderAllocation)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(com.ib.client.protobuf.OrderAllocationProto.OrderAllocation other) {
        if (other == com.ib.client.protobuf.OrderAllocationProto.OrderAllocation.getDefaultInstance()) return this;
        if (other.hasAccount()) {
          account_ = other.account_;
          bitField0_ |= 0x00000001;
          onChanged();
        }
        if (other.hasPosition()) {
          position_ = other.position_;
          bitField0_ |= 0x00000002;
          onChanged();
        }
        if (other.hasPositionDesired()) {
          positionDesired_ = other.positionDesired_;
          bitField0_ |= 0x00000004;
          onChanged();
        }
        if (other.hasPositionAfter()) {
          positionAfter_ = other.positionAfter_;
          bitField0_ |= 0x00000008;
          onChanged();
        }
        if (other.hasDesiredAllocQty()) {
          desiredAllocQty_ = other.desiredAllocQty_;
          bitField0_ |= 0x00000010;
          onChanged();
        }
        if (other.hasAllowedAllocQty()) {
          allowedAllocQty_ = other.allowedAllocQty_;
          bitField0_ |= 0x00000020;
          onChanged();
        }
        if (other.hasIsMonetary()) {
          setIsMonetary(other.getIsMonetary());
        }
        this.mergeUnknownFields(other.getUnknownFields());
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        if (extensionRegistry == null) {
          throw new java.lang.NullPointerException();
        }
        try {
          boolean done = false;
          while (!done) {
            int tag = input.readTag();
            switch (tag) {
              case 0:
                done = true;
                break;
              case 10: {
                account_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000001;
                break;
              } // case 10
              case 18: {
                position_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000002;
                break;
              } // case 18
              case 26: {
                positionDesired_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000004;
                break;
              } // case 26
              case 34: {
                positionAfter_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000008;
                break;
              } // case 34
              case 42: {
                desiredAllocQty_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000010;
                break;
              } // case 42
              case 50: {
                allowedAllocQty_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000020;
                break;
              } // case 50
              case 56: {
                isMonetary_ = input.readBool();
                bitField0_ |= 0x00000040;
                break;
              } // case 56
              default: {
                if (!super.parseUnknownField(input, extensionRegistry, tag)) {
                  done = true; // was an endgroup tag
                }
                break;
              } // default:
            } // switch (tag)
          } // while (!done)
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.unwrapIOException();
        } finally {
          onChanged();
        } // finally
        return this;
      }
      private int bitField0_;

      private java.lang.Object account_ = "";
      /**
       * <code>optional string account = 1;</code>
       * @return Whether the account field is set.
       */
      public boolean hasAccount() {
        return ((bitField0_ & 0x00000001) != 0);
      }
      /**
       * <code>optional string account = 1;</code>
       * @return The account.
       */
      public java.lang.String getAccount() {
        java.lang.Object ref = account_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          account_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string account = 1;</code>
       * @return The bytes for account.
       */
      public com.google.protobuf.ByteString
          getAccountBytes() {
        java.lang.Object ref = account_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          account_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string account = 1;</code>
       * @param value The account to set.
       * @return This builder for chaining.
       */
      public Builder setAccount(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        account_ = value;
        bitField0_ |= 0x00000001;
        onChanged();
        return this;
      }
      /**
       * <code>optional string account = 1;</code>
       * @return This builder for chaining.
       */
      public Builder clearAccount() {
        account_ = getDefaultInstance().getAccount();
        bitField0_ = (bitField0_ & ~0x00000001);
        onChanged();
        return this;
      }
      /**
       * <code>optional string account = 1;</code>
       * @param value The bytes for account to set.
       * @return This builder for chaining.
       */
      public Builder setAccountBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        account_ = value;
        bitField0_ |= 0x00000001;
        onChanged();
        return this;
      }

      private java.lang.Object position_ = "";
      /**
       * <code>optional string position = 2;</code>
       * @return Whether the position field is set.
       */
      public boolean hasPosition() {
        return ((bitField0_ & 0x00000002) != 0);
      }
      /**
       * <code>optional string position = 2;</code>
       * @return The position.
       */
      public java.lang.String getPosition() {
        java.lang.Object ref = position_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          position_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string position = 2;</code>
       * @return The bytes for position.
       */
      public com.google.protobuf.ByteString
          getPositionBytes() {
        java.lang.Object ref = position_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          position_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string position = 2;</code>
       * @param value The position to set.
       * @return This builder for chaining.
       */
      public Builder setPosition(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        position_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }
      /**
       * <code>optional string position = 2;</code>
       * @return This builder for chaining.
       */
      public Builder clearPosition() {
        position_ = getDefaultInstance().getPosition();
        bitField0_ = (bitField0_ & ~0x00000002);
        onChanged();
        return this;
      }
      /**
       * <code>optional string position = 2;</code>
       * @param value The bytes for position to set.
       * @return This builder for chaining.
       */
      public Builder setPositionBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        position_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }

      private java.lang.Object positionDesired_ = "";
      /**
       * <code>optional string positionDesired = 3;</code>
       * @return Whether the positionDesired field is set.
       */
      public boolean hasPositionDesired() {
        return ((bitField0_ & 0x00000004) != 0);
      }
      /**
       * <code>optional string positionDesired = 3;</code>
       * @return The positionDesired.
       */
      public java.lang.String getPositionDesired() {
        java.lang.Object ref = positionDesired_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          positionDesired_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string positionDesired = 3;</code>
       * @return The bytes for positionDesired.
       */
      public com.google.protobuf.ByteString
          getPositionDesiredBytes() {
        java.lang.Object ref = positionDesired_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          positionDesired_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string positionDesired = 3;</code>
       * @param value The positionDesired to set.
       * @return This builder for chaining.
       */
      public Builder setPositionDesired(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        positionDesired_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }
      /**
       * <code>optional string positionDesired = 3;</code>
       * @return This builder for chaining.
       */
      public Builder clearPositionDesired() {
        positionDesired_ = getDefaultInstance().getPositionDesired();
        bitField0_ = (bitField0_ & ~0x00000004);
        onChanged();
        return this;
      }
      /**
       * <code>optional string positionDesired = 3;</code>
       * @param value The bytes for positionDesired to set.
       * @return This builder for chaining.
       */
      public Builder setPositionDesiredBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        positionDesired_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }

      private java.lang.Object positionAfter_ = "";
      /**
       * <code>optional string positionAfter = 4;</code>
       * @return Whether the positionAfter field is set.
       */
      public boolean hasPositionAfter() {
        return ((bitField0_ & 0x00000008) != 0);
      }
      /**
       * <code>optional string positionAfter = 4;</code>
       * @return The positionAfter.
       */
      public java.lang.String getPositionAfter() {
        java.lang.Object ref = positionAfter_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          positionAfter_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string positionAfter = 4;</code>
       * @return The bytes for positionAfter.
       */
      public com.google.protobuf.ByteString
          getPositionAfterBytes() {
        java.lang.Object ref = positionAfter_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          positionAfter_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string positionAfter = 4;</code>
       * @param value The positionAfter to set.
       * @return This builder for chaining.
       */
      public Builder setPositionAfter(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        positionAfter_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }
      /**
       * <code>optional string positionAfter = 4;</code>
       * @return This builder for chaining.
       */
      public Builder clearPositionAfter() {
        positionAfter_ = getDefaultInstance().getPositionAfter();
        bitField0_ = (bitField0_ & ~0x00000008);
        onChanged();
        return this;
      }
      /**
       * <code>optional string positionAfter = 4;</code>
       * @param value The bytes for positionAfter to set.
       * @return This builder for chaining.
       */
      public Builder setPositionAfterBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        positionAfter_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }

      private java.lang.Object desiredAllocQty_ = "";
      /**
       * <code>optional string desiredAllocQty = 5;</code>
       * @return Whether the desiredAllocQty field is set.
       */
      public boolean hasDesiredAllocQty() {
        return ((bitField0_ & 0x00000010) != 0);
      }
      /**
       * <code>optional string desiredAllocQty = 5;</code>
       * @return The desiredAllocQty.
       */
      public java.lang.String getDesiredAllocQty() {
        java.lang.Object ref = desiredAllocQty_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          desiredAllocQty_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string desiredAllocQty = 5;</code>
       * @return The bytes for desiredAllocQty.
       */
      public com.google.protobuf.ByteString
          getDesiredAllocQtyBytes() {
        java.lang.Object ref = desiredAllocQty_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          desiredAllocQty_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string desiredAllocQty = 5;</code>
       * @param value The desiredAllocQty to set.
       * @return This builder for chaining.
       */
      public Builder setDesiredAllocQty(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        desiredAllocQty_ = value;
        bitField0_ |= 0x00000010;
        onChanged();
        return this;
      }
      /**
       * <code>optional string desiredAllocQty = 5;</code>
       * @return This builder for chaining.
       */
      public Builder clearDesiredAllocQty() {
        desiredAllocQty_ = getDefaultInstance().getDesiredAllocQty();
        bitField0_ = (bitField0_ & ~0x00000010);
        onChanged();
        return this;
      }
      /**
       * <code>optional string desiredAllocQty = 5;</code>
       * @param value The bytes for desiredAllocQty to set.
       * @return This builder for chaining.
       */
      public Builder setDesiredAllocQtyBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        desiredAllocQty_ = value;
        bitField0_ |= 0x00000010;
        onChanged();
        return this;
      }

      private java.lang.Object allowedAllocQty_ = "";
      /**
       * <code>optional string allowedAllocQty = 6;</code>
       * @return Whether the allowedAllocQty field is set.
       */
      public boolean hasAllowedAllocQty() {
        return ((bitField0_ & 0x00000020) != 0);
      }
      /**
       * <code>optional string allowedAllocQty = 6;</code>
       * @return The allowedAllocQty.
       */
      public java.lang.String getAllowedAllocQty() {
        java.lang.Object ref = allowedAllocQty_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          allowedAllocQty_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string allowedAllocQty = 6;</code>
       * @return The bytes for allowedAllocQty.
       */
      public com.google.protobuf.ByteString
          getAllowedAllocQtyBytes() {
        java.lang.Object ref = allowedAllocQty_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          allowedAllocQty_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string allowedAllocQty = 6;</code>
       * @param value The allowedAllocQty to set.
       * @return This builder for chaining.
       */
      public Builder setAllowedAllocQty(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        allowedAllocQty_ = value;
        bitField0_ |= 0x00000020;
        onChanged();
        return this;
      }
      /**
       * <code>optional string allowedAllocQty = 6;</code>
       * @return This builder for chaining.
       */
      public Builder clearAllowedAllocQty() {
        allowedAllocQty_ = getDefaultInstance().getAllowedAllocQty();
        bitField0_ = (bitField0_ & ~0x00000020);
        onChanged();
        return this;
      }
      /**
       * <code>optional string allowedAllocQty = 6;</code>
       * @param value The bytes for allowedAllocQty to set.
       * @return This builder for chaining.
       */
      public Builder setAllowedAllocQtyBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        allowedAllocQty_ = value;
        bitField0_ |= 0x00000020;
        onChanged();
        return this;
      }

      private boolean isMonetary_ ;
      /**
       * <code>optional bool isMonetary = 7;</code>
       * @return Whether the isMonetary field is set.
       */
      @java.lang.Override
      public boolean hasIsMonetary() {
        return ((bitField0_ & 0x00000040) != 0);
      }
      /**
       * <code>optional bool isMonetary = 7;</code>
       * @return The isMonetary.
       */
      @java.lang.Override
      public boolean getIsMonetary() {
        return isMonetary_;
      }
      /**
       * <code>optional bool isMonetary = 7;</code>
       * @param value The isMonetary to set.
       * @return This builder for chaining.
       */
      public Builder setIsMonetary(boolean value) {

        isMonetary_ = value;
        bitField0_ |= 0x00000040;
        onChanged();
        return this;
      }
      /**
       * <code>optional bool isMonetary = 7;</code>
       * @return This builder for chaining.
       */
      public Builder clearIsMonetary() {
        bitField0_ = (bitField0_ & ~0x00000040);
        isMonetary_ = false;
        onChanged();
        return this;
      }

      // @@protoc_insertion_point(builder_scope:protobuf.OrderAllocation)
    }

    // @@protoc_insertion_point(class_scope:protobuf.OrderAllocation)
    private static final com.ib.client.protobuf.OrderAllocationProto.OrderAllocation DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new com.ib.client.protobuf.OrderAllocationProto.OrderAllocation();
    }

    public static com.ib.client.protobuf.OrderAllocationProto.OrderAllocation getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<OrderAllocation>
        PARSER = new com.google.protobuf.AbstractParser<OrderAllocation>() {
      @java.lang.Override
      public OrderAllocation parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        Builder builder = newBuilder();
        try {
          builder.mergeFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.setUnfinishedMessage(builder.buildPartial());
        } catch (com.google.protobuf.UninitializedMessageException e) {
          throw e.asInvalidProtocolBufferException().setUnfinishedMessage(builder.buildPartial());
        } catch (java.io.IOException e) {
          throw new com.google.protobuf.InvalidProtocolBufferException(e)
              .setUnfinishedMessage(builder.buildPartial());
        }
        return builder.buildPartial();
      }
    };

    public static com.google.protobuf.Parser<OrderAllocation> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<OrderAllocation> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public com.ib.client.protobuf.OrderAllocationProto.OrderAllocation getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_protobuf_OrderAllocation_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessage.FieldAccessorTable
      internal_static_protobuf_OrderAllocation_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor
      getDescriptor() {
    return descriptor;
  }
  private static  com.google.protobuf.Descriptors.FileDescriptor
      descriptor;
  static {
    java.lang.String[] descriptorData = {
      "\n\025OrderAllocation.proto\022\010protobuf\"\303\002\n\017Or" +
      "derAllocation\022\024\n\007account\030\001 \001(\tH\000\210\001\001\022\025\n\010p" +
      "osition\030\002 \001(\tH\001\210\001\001\022\034\n\017positionDesired\030\003 " +
      "\001(\tH\002\210\001\001\022\032\n\rpositionAfter\030\004 \001(\tH\003\210\001\001\022\034\n\017" +
      "desiredAllocQty\030\005 \001(\tH\004\210\001\001\022\034\n\017allowedAll" +
      "ocQty\030\006 \001(\tH\005\210\001\001\022\027\n\nisMonetary\030\007 \001(\010H\006\210\001" +
      "\001B\n\n\010_accountB\013\n\t_positionB\022\n\020_positionD" +
      "esiredB\020\n\016_positionAfterB\022\n\020_desiredAllo" +
      "cQtyB\022\n\020_allowedAllocQtyB\r\n\013_isMonetaryB" +
      "?\n\026com.ib.client.protobufB\024OrderAllocati" +
      "onProto\252\002\016IBApi.protobufb\006proto3"
    };
    descriptor = com.google.protobuf.Descriptors.FileDescriptor
      .internalBuildGeneratedFileFrom(descriptorData,
        new com.google.protobuf.Descriptors.FileDescriptor[] {
        });
    internal_static_protobuf_OrderAllocation_descriptor =
      getDescriptor().getMessageTypes().get(0);
    internal_static_protobuf_OrderAllocation_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessage.FieldAccessorTable(
        internal_static_protobuf_OrderAllocation_descriptor,
        new java.lang.String[] { "Account", "Position", "PositionDesired", "PositionAfter", "DesiredAllocQty", "AllowedAllocQty", "IsMonetary", });
    descriptor.resolveAllFeaturesImmutable();
  }

  // @@protoc_insertion_point(outer_class_scope)
}
