/* Copyright (C) 2025 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 * and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable. */

syntax = "proto3";

package protobuf;

import "ComboLeg.proto";
import "DeltaNeutralContract.proto";

option java_outer_classname = "ContractProto";
option java_package = "com.ib.client.protobuf";
option csharp_namespace = "IBApi.protobuf";

message Contract {
    optional int32 conId = 1;
    optional string symbol = 2;
    optional string secType = 3;
    optional string lastTradeDateOrContractMonth = 4;
    optional double strike = 5;
    optional string right = 6;
    optional double multiplier = 7;
    optional string exchange = 8;
    optional string primaryExch = 9;
    optional string currency = 10;
    optional string localSymbol = 11;
    optional string tradingClass = 12;
    optional string secIdType = 13;
    optional string secId = 14;
    optional string description = 15;
    optional string issuerId = 16;
    optional DeltaNeutralContract deltaNeutralContract = 17;
    optional bool includeExpired = 18;
    optional string comboLegsDescrip = 19;
    repeated ComboLeg comboLegs = 20;
}
