// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: OrderCondition.proto
// Protobuf Java Version: 4.29.3

package com.ib.client.protobuf;

public final class OrderConditionProto {
  private OrderConditionProto() {}
  static {
    com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
      com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
      /* major= */ 4,
      /* minor= */ 29,
      /* patch= */ 3,
      /* suffix= */ "",
      OrderConditionProto.class.getName());
  }
  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistryLite registry) {
  }

  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistry registry) {
    registerAllExtensions(
        (com.google.protobuf.ExtensionRegistryLite) registry);
  }
  public interface OrderConditionOrBuilder extends
      // @@protoc_insertion_point(interface_extends:protobuf.OrderCondition)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>optional int32 type = 1;</code>
     * @return Whether the type field is set.
     */
    boolean hasType();
    /**
     * <code>optional int32 type = 1;</code>
     * @return The type.
     */
    int getType();

    /**
     * <code>optional bool isConjunctionConnection = 2;</code>
     * @return Whether the isConjunctionConnection field is set.
     */
    boolean hasIsConjunctionConnection();
    /**
     * <code>optional bool isConjunctionConnection = 2;</code>
     * @return The isConjunctionConnection.
     */
    boolean getIsConjunctionConnection();

    /**
     * <code>optional bool isMore = 3;</code>
     * @return Whether the isMore field is set.
     */
    boolean hasIsMore();
    /**
     * <code>optional bool isMore = 3;</code>
     * @return The isMore.
     */
    boolean getIsMore();

    /**
     * <code>optional int32 conId = 4;</code>
     * @return Whether the conId field is set.
     */
    boolean hasConId();
    /**
     * <code>optional int32 conId = 4;</code>
     * @return The conId.
     */
    int getConId();

    /**
     * <code>optional string exchange = 5;</code>
     * @return Whether the exchange field is set.
     */
    boolean hasExchange();
    /**
     * <code>optional string exchange = 5;</code>
     * @return The exchange.
     */
    java.lang.String getExchange();
    /**
     * <code>optional string exchange = 5;</code>
     * @return The bytes for exchange.
     */
    com.google.protobuf.ByteString
        getExchangeBytes();

    /**
     * <code>optional string symbol = 6;</code>
     * @return Whether the symbol field is set.
     */
    boolean hasSymbol();
    /**
     * <code>optional string symbol = 6;</code>
     * @return The symbol.
     */
    java.lang.String getSymbol();
    /**
     * <code>optional string symbol = 6;</code>
     * @return The bytes for symbol.
     */
    com.google.protobuf.ByteString
        getSymbolBytes();

    /**
     * <code>optional string secType = 7;</code>
     * @return Whether the secType field is set.
     */
    boolean hasSecType();
    /**
     * <code>optional string secType = 7;</code>
     * @return The secType.
     */
    java.lang.String getSecType();
    /**
     * <code>optional string secType = 7;</code>
     * @return The bytes for secType.
     */
    com.google.protobuf.ByteString
        getSecTypeBytes();

    /**
     * <code>optional int32 percent = 8;</code>
     * @return Whether the percent field is set.
     */
    boolean hasPercent();
    /**
     * <code>optional int32 percent = 8;</code>
     * @return The percent.
     */
    int getPercent();

    /**
     * <code>optional double changePercent = 9;</code>
     * @return Whether the changePercent field is set.
     */
    boolean hasChangePercent();
    /**
     * <code>optional double changePercent = 9;</code>
     * @return The changePercent.
     */
    double getChangePercent();

    /**
     * <code>optional double price = 10;</code>
     * @return Whether the price field is set.
     */
    boolean hasPrice();
    /**
     * <code>optional double price = 10;</code>
     * @return The price.
     */
    double getPrice();

    /**
     * <code>optional int32 triggerMethod = 11;</code>
     * @return Whether the triggerMethod field is set.
     */
    boolean hasTriggerMethod();
    /**
     * <code>optional int32 triggerMethod = 11;</code>
     * @return The triggerMethod.
     */
    int getTriggerMethod();

    /**
     * <code>optional string time = 12;</code>
     * @return Whether the time field is set.
     */
    boolean hasTime();
    /**
     * <code>optional string time = 12;</code>
     * @return The time.
     */
    java.lang.String getTime();
    /**
     * <code>optional string time = 12;</code>
     * @return The bytes for time.
     */
    com.google.protobuf.ByteString
        getTimeBytes();

    /**
     * <code>optional int32 volume = 13;</code>
     * @return Whether the volume field is set.
     */
    boolean hasVolume();
    /**
     * <code>optional int32 volume = 13;</code>
     * @return The volume.
     */
    int getVolume();
  }
  /**
   * Protobuf type {@code protobuf.OrderCondition}
   */
  public static final class OrderCondition extends
      com.google.protobuf.GeneratedMessage implements
      // @@protoc_insertion_point(message_implements:protobuf.OrderCondition)
      OrderConditionOrBuilder {
  private static final long serialVersionUID = 0L;
    static {
      com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
        com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
        /* major= */ 4,
        /* minor= */ 29,
        /* patch= */ 3,
        /* suffix= */ "",
        OrderCondition.class.getName());
    }
    // Use OrderCondition.newBuilder() to construct.
    private OrderCondition(com.google.protobuf.GeneratedMessage.Builder<?> builder) {
      super(builder);
    }
    private OrderCondition() {
      exchange_ = "";
      symbol_ = "";
      secType_ = "";
      time_ = "";
    }

    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return com.ib.client.protobuf.OrderConditionProto.internal_static_protobuf_OrderCondition_descriptor;
    }

    @java.lang.Override
    protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return com.ib.client.protobuf.OrderConditionProto.internal_static_protobuf_OrderCondition_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              com.ib.client.protobuf.OrderConditionProto.OrderCondition.class, com.ib.client.protobuf.OrderConditionProto.OrderCondition.Builder.class);
    }

    private int bitField0_;
    public static final int TYPE_FIELD_NUMBER = 1;
    private int type_ = 0;
    /**
     * <code>optional int32 type = 1;</code>
     * @return Whether the type field is set.
     */
    @java.lang.Override
    public boolean hasType() {
      return ((bitField0_ & 0x00000001) != 0);
    }
    /**
     * <code>optional int32 type = 1;</code>
     * @return The type.
     */
    @java.lang.Override
    public int getType() {
      return type_;
    }

    public static final int ISCONJUNCTIONCONNECTION_FIELD_NUMBER = 2;
    private boolean isConjunctionConnection_ = false;
    /**
     * <code>optional bool isConjunctionConnection = 2;</code>
     * @return Whether the isConjunctionConnection field is set.
     */
    @java.lang.Override
    public boolean hasIsConjunctionConnection() {
      return ((bitField0_ & 0x00000002) != 0);
    }
    /**
     * <code>optional bool isConjunctionConnection = 2;</code>
     * @return The isConjunctionConnection.
     */
    @java.lang.Override
    public boolean getIsConjunctionConnection() {
      return isConjunctionConnection_;
    }

    public static final int ISMORE_FIELD_NUMBER = 3;
    private boolean isMore_ = false;
    /**
     * <code>optional bool isMore = 3;</code>
     * @return Whether the isMore field is set.
     */
    @java.lang.Override
    public boolean hasIsMore() {
      return ((bitField0_ & 0x00000004) != 0);
    }
    /**
     * <code>optional bool isMore = 3;</code>
     * @return The isMore.
     */
    @java.lang.Override
    public boolean getIsMore() {
      return isMore_;
    }

    public static final int CONID_FIELD_NUMBER = 4;
    private int conId_ = 0;
    /**
     * <code>optional int32 conId = 4;</code>
     * @return Whether the conId field is set.
     */
    @java.lang.Override
    public boolean hasConId() {
      return ((bitField0_ & 0x00000008) != 0);
    }
    /**
     * <code>optional int32 conId = 4;</code>
     * @return The conId.
     */
    @java.lang.Override
    public int getConId() {
      return conId_;
    }

    public static final int EXCHANGE_FIELD_NUMBER = 5;
    @SuppressWarnings("serial")
    private volatile java.lang.Object exchange_ = "";
    /**
     * <code>optional string exchange = 5;</code>
     * @return Whether the exchange field is set.
     */
    @java.lang.Override
    public boolean hasExchange() {
      return ((bitField0_ & 0x00000010) != 0);
    }
    /**
     * <code>optional string exchange = 5;</code>
     * @return The exchange.
     */
    @java.lang.Override
    public java.lang.String getExchange() {
      java.lang.Object ref = exchange_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        exchange_ = s;
        return s;
      }
    }
    /**
     * <code>optional string exchange = 5;</code>
     * @return The bytes for exchange.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getExchangeBytes() {
      java.lang.Object ref = exchange_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        exchange_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int SYMBOL_FIELD_NUMBER = 6;
    @SuppressWarnings("serial")
    private volatile java.lang.Object symbol_ = "";
    /**
     * <code>optional string symbol = 6;</code>
     * @return Whether the symbol field is set.
     */
    @java.lang.Override
    public boolean hasSymbol() {
      return ((bitField0_ & 0x00000020) != 0);
    }
    /**
     * <code>optional string symbol = 6;</code>
     * @return The symbol.
     */
    @java.lang.Override
    public java.lang.String getSymbol() {
      java.lang.Object ref = symbol_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        symbol_ = s;
        return s;
      }
    }
    /**
     * <code>optional string symbol = 6;</code>
     * @return The bytes for symbol.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getSymbolBytes() {
      java.lang.Object ref = symbol_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        symbol_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int SECTYPE_FIELD_NUMBER = 7;
    @SuppressWarnings("serial")
    private volatile java.lang.Object secType_ = "";
    /**
     * <code>optional string secType = 7;</code>
     * @return Whether the secType field is set.
     */
    @java.lang.Override
    public boolean hasSecType() {
      return ((bitField0_ & 0x00000040) != 0);
    }
    /**
     * <code>optional string secType = 7;</code>
     * @return The secType.
     */
    @java.lang.Override
    public java.lang.String getSecType() {
      java.lang.Object ref = secType_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        secType_ = s;
        return s;
      }
    }
    /**
     * <code>optional string secType = 7;</code>
     * @return The bytes for secType.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getSecTypeBytes() {
      java.lang.Object ref = secType_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        secType_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int PERCENT_FIELD_NUMBER = 8;
    private int percent_ = 0;
    /**
     * <code>optional int32 percent = 8;</code>
     * @return Whether the percent field is set.
     */
    @java.lang.Override
    public boolean hasPercent() {
      return ((bitField0_ & 0x00000080) != 0);
    }
    /**
     * <code>optional int32 percent = 8;</code>
     * @return The percent.
     */
    @java.lang.Override
    public int getPercent() {
      return percent_;
    }

    public static final int CHANGEPERCENT_FIELD_NUMBER = 9;
    private double changePercent_ = 0D;
    /**
     * <code>optional double changePercent = 9;</code>
     * @return Whether the changePercent field is set.
     */
    @java.lang.Override
    public boolean hasChangePercent() {
      return ((bitField0_ & 0x00000100) != 0);
    }
    /**
     * <code>optional double changePercent = 9;</code>
     * @return The changePercent.
     */
    @java.lang.Override
    public double getChangePercent() {
      return changePercent_;
    }

    public static final int PRICE_FIELD_NUMBER = 10;
    private double price_ = 0D;
    /**
     * <code>optional double price = 10;</code>
     * @return Whether the price field is set.
     */
    @java.lang.Override
    public boolean hasPrice() {
      return ((bitField0_ & 0x00000200) != 0);
    }
    /**
     * <code>optional double price = 10;</code>
     * @return The price.
     */
    @java.lang.Override
    public double getPrice() {
      return price_;
    }

    public static final int TRIGGERMETHOD_FIELD_NUMBER = 11;
    private int triggerMethod_ = 0;
    /**
     * <code>optional int32 triggerMethod = 11;</code>
     * @return Whether the triggerMethod field is set.
     */
    @java.lang.Override
    public boolean hasTriggerMethod() {
      return ((bitField0_ & 0x00000400) != 0);
    }
    /**
     * <code>optional int32 triggerMethod = 11;</code>
     * @return The triggerMethod.
     */
    @java.lang.Override
    public int getTriggerMethod() {
      return triggerMethod_;
    }

    public static final int TIME_FIELD_NUMBER = 12;
    @SuppressWarnings("serial")
    private volatile java.lang.Object time_ = "";
    /**
     * <code>optional string time = 12;</code>
     * @return Whether the time field is set.
     */
    @java.lang.Override
    public boolean hasTime() {
      return ((bitField0_ & 0x00000800) != 0);
    }
    /**
     * <code>optional string time = 12;</code>
     * @return The time.
     */
    @java.lang.Override
    public java.lang.String getTime() {
      java.lang.Object ref = time_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        time_ = s;
        return s;
      }
    }
    /**
     * <code>optional string time = 12;</code>
     * @return The bytes for time.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getTimeBytes() {
      java.lang.Object ref = time_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        time_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int VOLUME_FIELD_NUMBER = 13;
    private int volume_ = 0;
    /**
     * <code>optional int32 volume = 13;</code>
     * @return Whether the volume field is set.
     */
    @java.lang.Override
    public boolean hasVolume() {
      return ((bitField0_ & 0x00001000) != 0);
    }
    /**
     * <code>optional int32 volume = 13;</code>
     * @return The volume.
     */
    @java.lang.Override
    public int getVolume() {
      return volume_;
    }

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      if (((bitField0_ & 0x00000001) != 0)) {
        output.writeInt32(1, type_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        output.writeBool(2, isConjunctionConnection_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        output.writeBool(3, isMore_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        output.writeInt32(4, conId_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 5, exchange_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 6, symbol_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 7, secType_);
      }
      if (((bitField0_ & 0x00000080) != 0)) {
        output.writeInt32(8, percent_);
      }
      if (((bitField0_ & 0x00000100) != 0)) {
        output.writeDouble(9, changePercent_);
      }
      if (((bitField0_ & 0x00000200) != 0)) {
        output.writeDouble(10, price_);
      }
      if (((bitField0_ & 0x00000400) != 0)) {
        output.writeInt32(11, triggerMethod_);
      }
      if (((bitField0_ & 0x00000800) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 12, time_);
      }
      if (((bitField0_ & 0x00001000) != 0)) {
        output.writeInt32(13, volume_);
      }
      getUnknownFields().writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      if (((bitField0_ & 0x00000001) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(1, type_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeBoolSize(2, isConjunctionConnection_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeBoolSize(3, isMore_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(4, conId_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(5, exchange_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(6, symbol_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(7, secType_);
      }
      if (((bitField0_ & 0x00000080) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(8, percent_);
      }
      if (((bitField0_ & 0x00000100) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeDoubleSize(9, changePercent_);
      }
      if (((bitField0_ & 0x00000200) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeDoubleSize(10, price_);
      }
      if (((bitField0_ & 0x00000400) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(11, triggerMethod_);
      }
      if (((bitField0_ & 0x00000800) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(12, time_);
      }
      if (((bitField0_ & 0x00001000) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(13, volume_);
      }
      size += getUnknownFields().getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof com.ib.client.protobuf.OrderConditionProto.OrderCondition)) {
        return super.equals(obj);
      }
      com.ib.client.protobuf.OrderConditionProto.OrderCondition other = (com.ib.client.protobuf.OrderConditionProto.OrderCondition) obj;

      if (hasType() != other.hasType()) return false;
      if (hasType()) {
        if (getType()
            != other.getType()) return false;
      }
      if (hasIsConjunctionConnection() != other.hasIsConjunctionConnection()) return false;
      if (hasIsConjunctionConnection()) {
        if (getIsConjunctionConnection()
            != other.getIsConjunctionConnection()) return false;
      }
      if (hasIsMore() != other.hasIsMore()) return false;
      if (hasIsMore()) {
        if (getIsMore()
            != other.getIsMore()) return false;
      }
      if (hasConId() != other.hasConId()) return false;
      if (hasConId()) {
        if (getConId()
            != other.getConId()) return false;
      }
      if (hasExchange() != other.hasExchange()) return false;
      if (hasExchange()) {
        if (!getExchange()
            .equals(other.getExchange())) return false;
      }
      if (hasSymbol() != other.hasSymbol()) return false;
      if (hasSymbol()) {
        if (!getSymbol()
            .equals(other.getSymbol())) return false;
      }
      if (hasSecType() != other.hasSecType()) return false;
      if (hasSecType()) {
        if (!getSecType()
            .equals(other.getSecType())) return false;
      }
      if (hasPercent() != other.hasPercent()) return false;
      if (hasPercent()) {
        if (getPercent()
            != other.getPercent()) return false;
      }
      if (hasChangePercent() != other.hasChangePercent()) return false;
      if (hasChangePercent()) {
        if (java.lang.Double.doubleToLongBits(getChangePercent())
            != java.lang.Double.doubleToLongBits(
                other.getChangePercent())) return false;
      }
      if (hasPrice() != other.hasPrice()) return false;
      if (hasPrice()) {
        if (java.lang.Double.doubleToLongBits(getPrice())
            != java.lang.Double.doubleToLongBits(
                other.getPrice())) return false;
      }
      if (hasTriggerMethod() != other.hasTriggerMethod()) return false;
      if (hasTriggerMethod()) {
        if (getTriggerMethod()
            != other.getTriggerMethod()) return false;
      }
      if (hasTime() != other.hasTime()) return false;
      if (hasTime()) {
        if (!getTime()
            .equals(other.getTime())) return false;
      }
      if (hasVolume() != other.hasVolume()) return false;
      if (hasVolume()) {
        if (getVolume()
            != other.getVolume()) return false;
      }
      if (!getUnknownFields().equals(other.getUnknownFields())) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      if (hasType()) {
        hash = (37 * hash) + TYPE_FIELD_NUMBER;
        hash = (53 * hash) + getType();
      }
      if (hasIsConjunctionConnection()) {
        hash = (37 * hash) + ISCONJUNCTIONCONNECTION_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashBoolean(
            getIsConjunctionConnection());
      }
      if (hasIsMore()) {
        hash = (37 * hash) + ISMORE_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashBoolean(
            getIsMore());
      }
      if (hasConId()) {
        hash = (37 * hash) + CONID_FIELD_NUMBER;
        hash = (53 * hash) + getConId();
      }
      if (hasExchange()) {
        hash = (37 * hash) + EXCHANGE_FIELD_NUMBER;
        hash = (53 * hash) + getExchange().hashCode();
      }
      if (hasSymbol()) {
        hash = (37 * hash) + SYMBOL_FIELD_NUMBER;
        hash = (53 * hash) + getSymbol().hashCode();
      }
      if (hasSecType()) {
        hash = (37 * hash) + SECTYPE_FIELD_NUMBER;
        hash = (53 * hash) + getSecType().hashCode();
      }
      if (hasPercent()) {
        hash = (37 * hash) + PERCENT_FIELD_NUMBER;
        hash = (53 * hash) + getPercent();
      }
      if (hasChangePercent()) {
        hash = (37 * hash) + CHANGEPERCENT_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            java.lang.Double.doubleToLongBits(getChangePercent()));
      }
      if (hasPrice()) {
        hash = (37 * hash) + PRICE_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            java.lang.Double.doubleToLongBits(getPrice()));
      }
      if (hasTriggerMethod()) {
        hash = (37 * hash) + TRIGGERMETHOD_FIELD_NUMBER;
        hash = (53 * hash) + getTriggerMethod();
      }
      if (hasTime()) {
        hash = (37 * hash) + TIME_FIELD_NUMBER;
        hash = (53 * hash) + getTime().hashCode();
      }
      if (hasVolume()) {
        hash = (37 * hash) + VOLUME_FIELD_NUMBER;
        hash = (53 * hash) + getVolume();
      }
      hash = (29 * hash) + getUnknownFields().hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input);
    }

    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(com.ib.client.protobuf.OrderConditionProto.OrderCondition prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessage.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code protobuf.OrderCondition}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessage.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:protobuf.OrderCondition)
        com.ib.client.protobuf.OrderConditionProto.OrderConditionOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return com.ib.client.protobuf.OrderConditionProto.internal_static_protobuf_OrderCondition_descriptor;
      }

      @java.lang.Override
      protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return com.ib.client.protobuf.OrderConditionProto.internal_static_protobuf_OrderCondition_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                com.ib.client.protobuf.OrderConditionProto.OrderCondition.class, com.ib.client.protobuf.OrderConditionProto.OrderCondition.Builder.class);
      }

      // Construct using com.ib.client.protobuf.OrderConditionProto.OrderCondition.newBuilder()
      private Builder() {

      }

      private Builder(
          com.google.protobuf.GeneratedMessage.BuilderParent parent) {
        super(parent);

      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        bitField0_ = 0;
        type_ = 0;
        isConjunctionConnection_ = false;
        isMore_ = false;
        conId_ = 0;
        exchange_ = "";
        symbol_ = "";
        secType_ = "";
        percent_ = 0;
        changePercent_ = 0D;
        price_ = 0D;
        triggerMethod_ = 0;
        time_ = "";
        volume_ = 0;
        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return com.ib.client.protobuf.OrderConditionProto.internal_static_protobuf_OrderCondition_descriptor;
      }

      @java.lang.Override
      public com.ib.client.protobuf.OrderConditionProto.OrderCondition getDefaultInstanceForType() {
        return com.ib.client.protobuf.OrderConditionProto.OrderCondition.getDefaultInstance();
      }

      @java.lang.Override
      public com.ib.client.protobuf.OrderConditionProto.OrderCondition build() {
        com.ib.client.protobuf.OrderConditionProto.OrderCondition result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public com.ib.client.protobuf.OrderConditionProto.OrderCondition buildPartial() {
        com.ib.client.protobuf.OrderConditionProto.OrderCondition result = new com.ib.client.protobuf.OrderConditionProto.OrderCondition(this);
        if (bitField0_ != 0) { buildPartial0(result); }
        onBuilt();
        return result;
      }

      private void buildPartial0(com.ib.client.protobuf.OrderConditionProto.OrderCondition result) {
        int from_bitField0_ = bitField0_;
        int to_bitField0_ = 0;
        if (((from_bitField0_ & 0x00000001) != 0)) {
          result.type_ = type_;
          to_bitField0_ |= 0x00000001;
        }
        if (((from_bitField0_ & 0x00000002) != 0)) {
          result.isConjunctionConnection_ = isConjunctionConnection_;
          to_bitField0_ |= 0x00000002;
        }
        if (((from_bitField0_ & 0x00000004) != 0)) {
          result.isMore_ = isMore_;
          to_bitField0_ |= 0x00000004;
        }
        if (((from_bitField0_ & 0x00000008) != 0)) {
          result.conId_ = conId_;
          to_bitField0_ |= 0x00000008;
        }
        if (((from_bitField0_ & 0x00000010) != 0)) {
          result.exchange_ = exchange_;
          to_bitField0_ |= 0x00000010;
        }
        if (((from_bitField0_ & 0x00000020) != 0)) {
          result.symbol_ = symbol_;
          to_bitField0_ |= 0x00000020;
        }
        if (((from_bitField0_ & 0x00000040) != 0)) {
          result.secType_ = secType_;
          to_bitField0_ |= 0x00000040;
        }
        if (((from_bitField0_ & 0x00000080) != 0)) {
          result.percent_ = percent_;
          to_bitField0_ |= 0x00000080;
        }
        if (((from_bitField0_ & 0x00000100) != 0)) {
          result.changePercent_ = changePercent_;
          to_bitField0_ |= 0x00000100;
        }
        if (((from_bitField0_ & 0x00000200) != 0)) {
          result.price_ = price_;
          to_bitField0_ |= 0x00000200;
        }
        if (((from_bitField0_ & 0x00000400) != 0)) {
          result.triggerMethod_ = triggerMethod_;
          to_bitField0_ |= 0x00000400;
        }
        if (((from_bitField0_ & 0x00000800) != 0)) {
          result.time_ = time_;
          to_bitField0_ |= 0x00000800;
        }
        if (((from_bitField0_ & 0x00001000) != 0)) {
          result.volume_ = volume_;
          to_bitField0_ |= 0x00001000;
        }
        result.bitField0_ |= to_bitField0_;
      }

      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof com.ib.client.protobuf.OrderConditionProto.OrderCondition) {
          return mergeFrom((com.ib.client.protobuf.OrderConditionProto.OrderCondition)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(com.ib.client.protobuf.OrderConditionProto.OrderCondition other) {
        if (other == com.ib.client.protobuf.OrderConditionProto.OrderCondition.getDefaultInstance()) return this;
        if (other.hasType()) {
          setType(other.getType());
        }
        if (other.hasIsConjunctionConnection()) {
          setIsConjunctionConnection(other.getIsConjunctionConnection());
        }
        if (other.hasIsMore()) {
          setIsMore(other.getIsMore());
        }
        if (other.hasConId()) {
          setConId(other.getConId());
        }
        if (other.hasExchange()) {
          exchange_ = other.exchange_;
          bitField0_ |= 0x00000010;
          onChanged();
        }
        if (other.hasSymbol()) {
          symbol_ = other.symbol_;
          bitField0_ |= 0x00000020;
          onChanged();
        }
        if (other.hasSecType()) {
          secType_ = other.secType_;
          bitField0_ |= 0x00000040;
          onChanged();
        }
        if (other.hasPercent()) {
          setPercent(other.getPercent());
        }
        if (other.hasChangePercent()) {
          setChangePercent(other.getChangePercent());
        }
        if (other.hasPrice()) {
          setPrice(other.getPrice());
        }
        if (other.hasTriggerMethod()) {
          setTriggerMethod(other.getTriggerMethod());
        }
        if (other.hasTime()) {
          time_ = other.time_;
          bitField0_ |= 0x00000800;
          onChanged();
        }
        if (other.hasVolume()) {
          setVolume(other.getVolume());
        }
        this.mergeUnknownFields(other.getUnknownFields());
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        if (extensionRegistry == null) {
          throw new java.lang.NullPointerException();
        }
        try {
          boolean done = false;
          while (!done) {
            int tag = input.readTag();
            switch (tag) {
              case 0:
                done = true;
                break;
              case 8: {
                type_ = input.readInt32();
                bitField0_ |= 0x00000001;
                break;
              } // case 8
              case 16: {
                isConjunctionConnection_ = input.readBool();
                bitField0_ |= 0x00000002;
                break;
              } // case 16
              case 24: {
                isMore_ = input.readBool();
                bitField0_ |= 0x00000004;
                break;
              } // case 24
              case 32: {
                conId_ = input.readInt32();
                bitField0_ |= 0x00000008;
                break;
              } // case 32
              case 42: {
                exchange_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000010;
                break;
              } // case 42
              case 50: {
                symbol_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000020;
                break;
              } // case 50
              case 58: {
                secType_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000040;
                break;
              } // case 58
              case 64: {
                percent_ = input.readInt32();
                bitField0_ |= 0x00000080;
                break;
              } // case 64
              case 73: {
                changePercent_ = input.readDouble();
                bitField0_ |= 0x00000100;
                break;
              } // case 73
              case 81: {
                price_ = input.readDouble();
                bitField0_ |= 0x00000200;
                break;
              } // case 81
              case 88: {
                triggerMethod_ = input.readInt32();
                bitField0_ |= 0x00000400;
                break;
              } // case 88
              case 98: {
                time_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000800;
                break;
              } // case 98
              case 104: {
                volume_ = input.readInt32();
                bitField0_ |= 0x00001000;
                break;
              } // case 104
              default: {
                if (!super.parseUnknownField(input, extensionRegistry, tag)) {
                  done = true; // was an endgroup tag
                }
                break;
              } // default:
            } // switch (tag)
          } // while (!done)
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.unwrapIOException();
        } finally {
          onChanged();
        } // finally
        return this;
      }
      private int bitField0_;

      private int type_ ;
      /**
       * <code>optional int32 type = 1;</code>
       * @return Whether the type field is set.
       */
      @java.lang.Override
      public boolean hasType() {
        return ((bitField0_ & 0x00000001) != 0);
      }
      /**
       * <code>optional int32 type = 1;</code>
       * @return The type.
       */
      @java.lang.Override
      public int getType() {
        return type_;
      }
      /**
       * <code>optional int32 type = 1;</code>
       * @param value The type to set.
       * @return This builder for chaining.
       */
      public Builder setType(int value) {

        type_ = value;
        bitField0_ |= 0x00000001;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 type = 1;</code>
       * @return This builder for chaining.
       */
      public Builder clearType() {
        bitField0_ = (bitField0_ & ~0x00000001);
        type_ = 0;
        onChanged();
        return this;
      }

      private boolean isConjunctionConnection_ ;
      /**
       * <code>optional bool isConjunctionConnection = 2;</code>
       * @return Whether the isConjunctionConnection field is set.
       */
      @java.lang.Override
      public boolean hasIsConjunctionConnection() {
        return ((bitField0_ & 0x00000002) != 0);
      }
      /**
       * <code>optional bool isConjunctionConnection = 2;</code>
       * @return The isConjunctionConnection.
       */
      @java.lang.Override
      public boolean getIsConjunctionConnection() {
        return isConjunctionConnection_;
      }
      /**
       * <code>optional bool isConjunctionConnection = 2;</code>
       * @param value The isConjunctionConnection to set.
       * @return This builder for chaining.
       */
      public Builder setIsConjunctionConnection(boolean value) {

        isConjunctionConnection_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }
      /**
       * <code>optional bool isConjunctionConnection = 2;</code>
       * @return This builder for chaining.
       */
      public Builder clearIsConjunctionConnection() {
        bitField0_ = (bitField0_ & ~0x00000002);
        isConjunctionConnection_ = false;
        onChanged();
        return this;
      }

      private boolean isMore_ ;
      /**
       * <code>optional bool isMore = 3;</code>
       * @return Whether the isMore field is set.
       */
      @java.lang.Override
      public boolean hasIsMore() {
        return ((bitField0_ & 0x00000004) != 0);
      }
      /**
       * <code>optional bool isMore = 3;</code>
       * @return The isMore.
       */
      @java.lang.Override
      public boolean getIsMore() {
        return isMore_;
      }
      /**
       * <code>optional bool isMore = 3;</code>
       * @param value The isMore to set.
       * @return This builder for chaining.
       */
      public Builder setIsMore(boolean value) {

        isMore_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }
      /**
       * <code>optional bool isMore = 3;</code>
       * @return This builder for chaining.
       */
      public Builder clearIsMore() {
        bitField0_ = (bitField0_ & ~0x00000004);
        isMore_ = false;
        onChanged();
        return this;
      }

      private int conId_ ;
      /**
       * <code>optional int32 conId = 4;</code>
       * @return Whether the conId field is set.
       */
      @java.lang.Override
      public boolean hasConId() {
        return ((bitField0_ & 0x00000008) != 0);
      }
      /**
       * <code>optional int32 conId = 4;</code>
       * @return The conId.
       */
      @java.lang.Override
      public int getConId() {
        return conId_;
      }
      /**
       * <code>optional int32 conId = 4;</code>
       * @param value The conId to set.
       * @return This builder for chaining.
       */
      public Builder setConId(int value) {

        conId_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 conId = 4;</code>
       * @return This builder for chaining.
       */
      public Builder clearConId() {
        bitField0_ = (bitField0_ & ~0x00000008);
        conId_ = 0;
        onChanged();
        return this;
      }

      private java.lang.Object exchange_ = "";
      /**
       * <code>optional string exchange = 5;</code>
       * @return Whether the exchange field is set.
       */
      public boolean hasExchange() {
        return ((bitField0_ & 0x00000010) != 0);
      }
      /**
       * <code>optional string exchange = 5;</code>
       * @return The exchange.
       */
      public java.lang.String getExchange() {
        java.lang.Object ref = exchange_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          exchange_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string exchange = 5;</code>
       * @return The bytes for exchange.
       */
      public com.google.protobuf.ByteString
          getExchangeBytes() {
        java.lang.Object ref = exchange_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          exchange_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string exchange = 5;</code>
       * @param value The exchange to set.
       * @return This builder for chaining.
       */
      public Builder setExchange(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        exchange_ = value;
        bitField0_ |= 0x00000010;
        onChanged();
        return this;
      }
      /**
       * <code>optional string exchange = 5;</code>
       * @return This builder for chaining.
       */
      public Builder clearExchange() {
        exchange_ = getDefaultInstance().getExchange();
        bitField0_ = (bitField0_ & ~0x00000010);
        onChanged();
        return this;
      }
      /**
       * <code>optional string exchange = 5;</code>
       * @param value The bytes for exchange to set.
       * @return This builder for chaining.
       */
      public Builder setExchangeBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        exchange_ = value;
        bitField0_ |= 0x00000010;
        onChanged();
        return this;
      }

      private java.lang.Object symbol_ = "";
      /**
       * <code>optional string symbol = 6;</code>
       * @return Whether the symbol field is set.
       */
      public boolean hasSymbol() {
        return ((bitField0_ & 0x00000020) != 0);
      }
      /**
       * <code>optional string symbol = 6;</code>
       * @return The symbol.
       */
      public java.lang.String getSymbol() {
        java.lang.Object ref = symbol_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          symbol_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string symbol = 6;</code>
       * @return The bytes for symbol.
       */
      public com.google.protobuf.ByteString
          getSymbolBytes() {
        java.lang.Object ref = symbol_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          symbol_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string symbol = 6;</code>
       * @param value The symbol to set.
       * @return This builder for chaining.
       */
      public Builder setSymbol(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        symbol_ = value;
        bitField0_ |= 0x00000020;
        onChanged();
        return this;
      }
      /**
       * <code>optional string symbol = 6;</code>
       * @return This builder for chaining.
       */
      public Builder clearSymbol() {
        symbol_ = getDefaultInstance().getSymbol();
        bitField0_ = (bitField0_ & ~0x00000020);
        onChanged();
        return this;
      }
      /**
       * <code>optional string symbol = 6;</code>
       * @param value The bytes for symbol to set.
       * @return This builder for chaining.
       */
      public Builder setSymbolBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        symbol_ = value;
        bitField0_ |= 0x00000020;
        onChanged();
        return this;
      }

      private java.lang.Object secType_ = "";
      /**
       * <code>optional string secType = 7;</code>
       * @return Whether the secType field is set.
       */
      public boolean hasSecType() {
        return ((bitField0_ & 0x00000040) != 0);
      }
      /**
       * <code>optional string secType = 7;</code>
       * @return The secType.
       */
      public java.lang.String getSecType() {
        java.lang.Object ref = secType_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          secType_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string secType = 7;</code>
       * @return The bytes for secType.
       */
      public com.google.protobuf.ByteString
          getSecTypeBytes() {
        java.lang.Object ref = secType_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          secType_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string secType = 7;</code>
       * @param value The secType to set.
       * @return This builder for chaining.
       */
      public Builder setSecType(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        secType_ = value;
        bitField0_ |= 0x00000040;
        onChanged();
        return this;
      }
      /**
       * <code>optional string secType = 7;</code>
       * @return This builder for chaining.
       */
      public Builder clearSecType() {
        secType_ = getDefaultInstance().getSecType();
        bitField0_ = (bitField0_ & ~0x00000040);
        onChanged();
        return this;
      }
      /**
       * <code>optional string secType = 7;</code>
       * @param value The bytes for secType to set.
       * @return This builder for chaining.
       */
      public Builder setSecTypeBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        secType_ = value;
        bitField0_ |= 0x00000040;
        onChanged();
        return this;
      }

      private int percent_ ;
      /**
       * <code>optional int32 percent = 8;</code>
       * @return Whether the percent field is set.
       */
      @java.lang.Override
      public boolean hasPercent() {
        return ((bitField0_ & 0x00000080) != 0);
      }
      /**
       * <code>optional int32 percent = 8;</code>
       * @return The percent.
       */
      @java.lang.Override
      public int getPercent() {
        return percent_;
      }
      /**
       * <code>optional int32 percent = 8;</code>
       * @param value The percent to set.
       * @return This builder for chaining.
       */
      public Builder setPercent(int value) {

        percent_ = value;
        bitField0_ |= 0x00000080;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 percent = 8;</code>
       * @return This builder for chaining.
       */
      public Builder clearPercent() {
        bitField0_ = (bitField0_ & ~0x00000080);
        percent_ = 0;
        onChanged();
        return this;
      }

      private double changePercent_ ;
      /**
       * <code>optional double changePercent = 9;</code>
       * @return Whether the changePercent field is set.
       */
      @java.lang.Override
      public boolean hasChangePercent() {
        return ((bitField0_ & 0x00000100) != 0);
      }
      /**
       * <code>optional double changePercent = 9;</code>
       * @return The changePercent.
       */
      @java.lang.Override
      public double getChangePercent() {
        return changePercent_;
      }
      /**
       * <code>optional double changePercent = 9;</code>
       * @param value The changePercent to set.
       * @return This builder for chaining.
       */
      public Builder setChangePercent(double value) {

        changePercent_ = value;
        bitField0_ |= 0x00000100;
        onChanged();
        return this;
      }
      /**
       * <code>optional double changePercent = 9;</code>
       * @return This builder for chaining.
       */
      public Builder clearChangePercent() {
        bitField0_ = (bitField0_ & ~0x00000100);
        changePercent_ = 0D;
        onChanged();
        return this;
      }

      private double price_ ;
      /**
       * <code>optional double price = 10;</code>
       * @return Whether the price field is set.
       */
      @java.lang.Override
      public boolean hasPrice() {
        return ((bitField0_ & 0x00000200) != 0);
      }
      /**
       * <code>optional double price = 10;</code>
       * @return The price.
       */
      @java.lang.Override
      public double getPrice() {
        return price_;
      }
      /**
       * <code>optional double price = 10;</code>
       * @param value The price to set.
       * @return This builder for chaining.
       */
      public Builder setPrice(double value) {

        price_ = value;
        bitField0_ |= 0x00000200;
        onChanged();
        return this;
      }
      /**
       * <code>optional double price = 10;</code>
       * @return This builder for chaining.
       */
      public Builder clearPrice() {
        bitField0_ = (bitField0_ & ~0x00000200);
        price_ = 0D;
        onChanged();
        return this;
      }

      private int triggerMethod_ ;
      /**
       * <code>optional int32 triggerMethod = 11;</code>
       * @return Whether the triggerMethod field is set.
       */
      @java.lang.Override
      public boolean hasTriggerMethod() {
        return ((bitField0_ & 0x00000400) != 0);
      }
      /**
       * <code>optional int32 triggerMethod = 11;</code>
       * @return The triggerMethod.
       */
      @java.lang.Override
      public int getTriggerMethod() {
        return triggerMethod_;
      }
      /**
       * <code>optional int32 triggerMethod = 11;</code>
       * @param value The triggerMethod to set.
       * @return This builder for chaining.
       */
      public Builder setTriggerMethod(int value) {

        triggerMethod_ = value;
        bitField0_ |= 0x00000400;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 triggerMethod = 11;</code>
       * @return This builder for chaining.
       */
      public Builder clearTriggerMethod() {
        bitField0_ = (bitField0_ & ~0x00000400);
        triggerMethod_ = 0;
        onChanged();
        return this;
      }

      private java.lang.Object time_ = "";
      /**
       * <code>optional string time = 12;</code>
       * @return Whether the time field is set.
       */
      public boolean hasTime() {
        return ((bitField0_ & 0x00000800) != 0);
      }
      /**
       * <code>optional string time = 12;</code>
       * @return The time.
       */
      public java.lang.String getTime() {
        java.lang.Object ref = time_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          time_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string time = 12;</code>
       * @return The bytes for time.
       */
      public com.google.protobuf.ByteString
          getTimeBytes() {
        java.lang.Object ref = time_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          time_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string time = 12;</code>
       * @param value The time to set.
       * @return This builder for chaining.
       */
      public Builder setTime(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        time_ = value;
        bitField0_ |= 0x00000800;
        onChanged();
        return this;
      }
      /**
       * <code>optional string time = 12;</code>
       * @return This builder for chaining.
       */
      public Builder clearTime() {
        time_ = getDefaultInstance().getTime();
        bitField0_ = (bitField0_ & ~0x00000800);
        onChanged();
        return this;
      }
      /**
       * <code>optional string time = 12;</code>
       * @param value The bytes for time to set.
       * @return This builder for chaining.
       */
      public Builder setTimeBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        time_ = value;
        bitField0_ |= 0x00000800;
        onChanged();
        return this;
      }

      private int volume_ ;
      /**
       * <code>optional int32 volume = 13;</code>
       * @return Whether the volume field is set.
       */
      @java.lang.Override
      public boolean hasVolume() {
        return ((bitField0_ & 0x00001000) != 0);
      }
      /**
       * <code>optional int32 volume = 13;</code>
       * @return The volume.
       */
      @java.lang.Override
      public int getVolume() {
        return volume_;
      }
      /**
       * <code>optional int32 volume = 13;</code>
       * @param value The volume to set.
       * @return This builder for chaining.
       */
      public Builder setVolume(int value) {

        volume_ = value;
        bitField0_ |= 0x00001000;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 volume = 13;</code>
       * @return This builder for chaining.
       */
      public Builder clearVolume() {
        bitField0_ = (bitField0_ & ~0x00001000);
        volume_ = 0;
        onChanged();
        return this;
      }

      // @@protoc_insertion_point(builder_scope:protobuf.OrderCondition)
    }

    // @@protoc_insertion_point(class_scope:protobuf.OrderCondition)
    private static final com.ib.client.protobuf.OrderConditionProto.OrderCondition DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new com.ib.client.protobuf.OrderConditionProto.OrderCondition();
    }

    public static com.ib.client.protobuf.OrderConditionProto.OrderCondition getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<OrderCondition>
        PARSER = new com.google.protobuf.AbstractParser<OrderCondition>() {
      @java.lang.Override
      public OrderCondition parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        Builder builder = newBuilder();
        try {
          builder.mergeFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.setUnfinishedMessage(builder.buildPartial());
        } catch (com.google.protobuf.UninitializedMessageException e) {
          throw e.asInvalidProtocolBufferException().setUnfinishedMessage(builder.buildPartial());
        } catch (java.io.IOException e) {
          throw new com.google.protobuf.InvalidProtocolBufferException(e)
              .setUnfinishedMessage(builder.buildPartial());
        }
        return builder.buildPartial();
      }
    };

    public static com.google.protobuf.Parser<OrderCondition> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<OrderCondition> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public com.ib.client.protobuf.OrderConditionProto.OrderCondition getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_protobuf_OrderCondition_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessage.FieldAccessorTable
      internal_static_protobuf_OrderCondition_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor
      getDescriptor() {
    return descriptor;
  }
  private static  com.google.protobuf.Descriptors.FileDescriptor
      descriptor;
  static {
    java.lang.String[] descriptorData = {
      "\n\024OrderCondition.proto\022\010protobuf\"\352\003\n\016Ord" +
      "erCondition\022\021\n\004type\030\001 \001(\005H\000\210\001\001\022$\n\027isConj" +
      "unctionConnection\030\002 \001(\010H\001\210\001\001\022\023\n\006isMore\030\003" +
      " \001(\010H\002\210\001\001\022\022\n\005conId\030\004 \001(\005H\003\210\001\001\022\025\n\010exchang" +
      "e\030\005 \001(\tH\004\210\001\001\022\023\n\006symbol\030\006 \001(\tH\005\210\001\001\022\024\n\007sec" +
      "Type\030\007 \001(\tH\006\210\001\001\022\024\n\007percent\030\010 \001(\005H\007\210\001\001\022\032\n" +
      "\rchangePercent\030\t \001(\001H\010\210\001\001\022\022\n\005price\030\n \001(\001" +
      "H\t\210\001\001\022\032\n\rtriggerMethod\030\013 \001(\005H\n\210\001\001\022\021\n\004tim" +
      "e\030\014 \001(\tH\013\210\001\001\022\023\n\006volume\030\r \001(\005H\014\210\001\001B\007\n\005_ty" +
      "peB\032\n\030_isConjunctionConnectionB\t\n\007_isMor" +
      "eB\010\n\006_conIdB\013\n\t_exchangeB\t\n\007_symbolB\n\n\010_" +
      "secTypeB\n\n\010_percentB\020\n\016_changePercentB\010\n" +
      "\006_priceB\020\n\016_triggerMethodB\007\n\005_timeB\t\n\007_v" +
      "olumeB>\n\026com.ib.client.protobufB\023OrderCo" +
      "nditionProto\252\002\016IBApi.protobufb\006proto3"
    };
    descriptor = com.google.protobuf.Descriptors.FileDescriptor
      .internalBuildGeneratedFileFrom(descriptorData,
        new com.google.protobuf.Descriptors.FileDescriptor[] {
        });
    internal_static_protobuf_OrderCondition_descriptor =
      getDescriptor().getMessageTypes().get(0);
    internal_static_protobuf_OrderCondition_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessage.FieldAccessorTable(
        internal_static_protobuf_OrderCondition_descriptor,
        new java.lang.String[] { "Type", "IsConjunctionConnection", "IsMore", "ConId", "Exchange", "Symbol", "SecType", "Percent", "ChangePercent", "Price", "TriggerMethod", "Time", "Volume", });
    descriptor.resolveAllFeaturesImmutable();
  }

  // @@protoc_insertion_point(outer_class_scope)
}
