// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: ComboLeg.proto
// Protobuf Java Version: 4.29.3

package com.ib.client.protobuf;

public final class ComboLegProto {
  private ComboLegProto() {}
  static {
    com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
      com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
      /* major= */ 4,
      /* minor= */ 29,
      /* patch= */ 3,
      /* suffix= */ "",
      ComboLegProto.class.getName());
  }
  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistryLite registry) {
  }

  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistry registry) {
    registerAllExtensions(
        (com.google.protobuf.ExtensionRegistryLite) registry);
  }
  public interface ComboLegOrBuilder extends
      // @@protoc_insertion_point(interface_extends:protobuf.ComboLeg)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>optional int32 conId = 1;</code>
     * @return Whether the conId field is set.
     */
    boolean hasConId();
    /**
     * <code>optional int32 conId = 1;</code>
     * @return The conId.
     */
    int getConId();

    /**
     * <code>optional int32 ratio = 2;</code>
     * @return Whether the ratio field is set.
     */
    boolean hasRatio();
    /**
     * <code>optional int32 ratio = 2;</code>
     * @return The ratio.
     */
    int getRatio();

    /**
     * <code>optional string action = 3;</code>
     * @return Whether the action field is set.
     */
    boolean hasAction();
    /**
     * <code>optional string action = 3;</code>
     * @return The action.
     */
    java.lang.String getAction();
    /**
     * <code>optional string action = 3;</code>
     * @return The bytes for action.
     */
    com.google.protobuf.ByteString
        getActionBytes();

    /**
     * <code>optional string exchange = 4;</code>
     * @return Whether the exchange field is set.
     */
    boolean hasExchange();
    /**
     * <code>optional string exchange = 4;</code>
     * @return The exchange.
     */
    java.lang.String getExchange();
    /**
     * <code>optional string exchange = 4;</code>
     * @return The bytes for exchange.
     */
    com.google.protobuf.ByteString
        getExchangeBytes();

    /**
     * <code>optional int32 openClose = 5;</code>
     * @return Whether the openClose field is set.
     */
    boolean hasOpenClose();
    /**
     * <code>optional int32 openClose = 5;</code>
     * @return The openClose.
     */
    int getOpenClose();

    /**
     * <code>optional int32 shortSalesSlot = 6;</code>
     * @return Whether the shortSalesSlot field is set.
     */
    boolean hasShortSalesSlot();
    /**
     * <code>optional int32 shortSalesSlot = 6;</code>
     * @return The shortSalesSlot.
     */
    int getShortSalesSlot();

    /**
     * <code>optional string designatedLocation = 7;</code>
     * @return Whether the designatedLocation field is set.
     */
    boolean hasDesignatedLocation();
    /**
     * <code>optional string designatedLocation = 7;</code>
     * @return The designatedLocation.
     */
    java.lang.String getDesignatedLocation();
    /**
     * <code>optional string designatedLocation = 7;</code>
     * @return The bytes for designatedLocation.
     */
    com.google.protobuf.ByteString
        getDesignatedLocationBytes();

    /**
     * <code>optional int32 exemptCode = 8;</code>
     * @return Whether the exemptCode field is set.
     */
    boolean hasExemptCode();
    /**
     * <code>optional int32 exemptCode = 8;</code>
     * @return The exemptCode.
     */
    int getExemptCode();

    /**
     * <code>optional double perLegPrice = 9;</code>
     * @return Whether the perLegPrice field is set.
     */
    boolean hasPerLegPrice();
    /**
     * <code>optional double perLegPrice = 9;</code>
     * @return The perLegPrice.
     */
    double getPerLegPrice();
  }
  /**
   * Protobuf type {@code protobuf.ComboLeg}
   */
  public static final class ComboLeg extends
      com.google.protobuf.GeneratedMessage implements
      // @@protoc_insertion_point(message_implements:protobuf.ComboLeg)
      ComboLegOrBuilder {
  private static final long serialVersionUID = 0L;
    static {
      com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
        com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
        /* major= */ 4,
        /* minor= */ 29,
        /* patch= */ 3,
        /* suffix= */ "",
        ComboLeg.class.getName());
    }
    // Use ComboLeg.newBuilder() to construct.
    private ComboLeg(com.google.protobuf.GeneratedMessage.Builder<?> builder) {
      super(builder);
    }
    private ComboLeg() {
      action_ = "";
      exchange_ = "";
      designatedLocation_ = "";
    }

    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return com.ib.client.protobuf.ComboLegProto.internal_static_protobuf_ComboLeg_descriptor;
    }

    @java.lang.Override
    protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return com.ib.client.protobuf.ComboLegProto.internal_static_protobuf_ComboLeg_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              com.ib.client.protobuf.ComboLegProto.ComboLeg.class, com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder.class);
    }

    private int bitField0_;
    public static final int CONID_FIELD_NUMBER = 1;
    private int conId_ = 0;
    /**
     * <code>optional int32 conId = 1;</code>
     * @return Whether the conId field is set.
     */
    @java.lang.Override
    public boolean hasConId() {
      return ((bitField0_ & 0x00000001) != 0);
    }
    /**
     * <code>optional int32 conId = 1;</code>
     * @return The conId.
     */
    @java.lang.Override
    public int getConId() {
      return conId_;
    }

    public static final int RATIO_FIELD_NUMBER = 2;
    private int ratio_ = 0;
    /**
     * <code>optional int32 ratio = 2;</code>
     * @return Whether the ratio field is set.
     */
    @java.lang.Override
    public boolean hasRatio() {
      return ((bitField0_ & 0x00000002) != 0);
    }
    /**
     * <code>optional int32 ratio = 2;</code>
     * @return The ratio.
     */
    @java.lang.Override
    public int getRatio() {
      return ratio_;
    }

    public static final int ACTION_FIELD_NUMBER = 3;
    @SuppressWarnings("serial")
    private volatile java.lang.Object action_ = "";
    /**
     * <code>optional string action = 3;</code>
     * @return Whether the action field is set.
     */
    @java.lang.Override
    public boolean hasAction() {
      return ((bitField0_ & 0x00000004) != 0);
    }
    /**
     * <code>optional string action = 3;</code>
     * @return The action.
     */
    @java.lang.Override
    public java.lang.String getAction() {
      java.lang.Object ref = action_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        action_ = s;
        return s;
      }
    }
    /**
     * <code>optional string action = 3;</code>
     * @return The bytes for action.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getActionBytes() {
      java.lang.Object ref = action_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        action_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int EXCHANGE_FIELD_NUMBER = 4;
    @SuppressWarnings("serial")
    private volatile java.lang.Object exchange_ = "";
    /**
     * <code>optional string exchange = 4;</code>
     * @return Whether the exchange field is set.
     */
    @java.lang.Override
    public boolean hasExchange() {
      return ((bitField0_ & 0x00000008) != 0);
    }
    /**
     * <code>optional string exchange = 4;</code>
     * @return The exchange.
     */
    @java.lang.Override
    public java.lang.String getExchange() {
      java.lang.Object ref = exchange_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        exchange_ = s;
        return s;
      }
    }
    /**
     * <code>optional string exchange = 4;</code>
     * @return The bytes for exchange.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getExchangeBytes() {
      java.lang.Object ref = exchange_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        exchange_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int OPENCLOSE_FIELD_NUMBER = 5;
    private int openClose_ = 0;
    /**
     * <code>optional int32 openClose = 5;</code>
     * @return Whether the openClose field is set.
     */
    @java.lang.Override
    public boolean hasOpenClose() {
      return ((bitField0_ & 0x00000010) != 0);
    }
    /**
     * <code>optional int32 openClose = 5;</code>
     * @return The openClose.
     */
    @java.lang.Override
    public int getOpenClose() {
      return openClose_;
    }

    public static final int SHORTSALESSLOT_FIELD_NUMBER = 6;
    private int shortSalesSlot_ = 0;
    /**
     * <code>optional int32 shortSalesSlot = 6;</code>
     * @return Whether the shortSalesSlot field is set.
     */
    @java.lang.Override
    public boolean hasShortSalesSlot() {
      return ((bitField0_ & 0x00000020) != 0);
    }
    /**
     * <code>optional int32 shortSalesSlot = 6;</code>
     * @return The shortSalesSlot.
     */
    @java.lang.Override
    public int getShortSalesSlot() {
      return shortSalesSlot_;
    }

    public static final int DESIGNATEDLOCATION_FIELD_NUMBER = 7;
    @SuppressWarnings("serial")
    private volatile java.lang.Object designatedLocation_ = "";
    /**
     * <code>optional string designatedLocation = 7;</code>
     * @return Whether the designatedLocation field is set.
     */
    @java.lang.Override
    public boolean hasDesignatedLocation() {
      return ((bitField0_ & 0x00000040) != 0);
    }
    /**
     * <code>optional string designatedLocation = 7;</code>
     * @return The designatedLocation.
     */
    @java.lang.Override
    public java.lang.String getDesignatedLocation() {
      java.lang.Object ref = designatedLocation_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        designatedLocation_ = s;
        return s;
      }
    }
    /**
     * <code>optional string designatedLocation = 7;</code>
     * @return The bytes for designatedLocation.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getDesignatedLocationBytes() {
      java.lang.Object ref = designatedLocation_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        designatedLocation_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int EXEMPTCODE_FIELD_NUMBER = 8;
    private int exemptCode_ = 0;
    /**
     * <code>optional int32 exemptCode = 8;</code>
     * @return Whether the exemptCode field is set.
     */
    @java.lang.Override
    public boolean hasExemptCode() {
      return ((bitField0_ & 0x00000080) != 0);
    }
    /**
     * <code>optional int32 exemptCode = 8;</code>
     * @return The exemptCode.
     */
    @java.lang.Override
    public int getExemptCode() {
      return exemptCode_;
    }

    public static final int PERLEGPRICE_FIELD_NUMBER = 9;
    private double perLegPrice_ = 0D;
    /**
     * <code>optional double perLegPrice = 9;</code>
     * @return Whether the perLegPrice field is set.
     */
    @java.lang.Override
    public boolean hasPerLegPrice() {
      return ((bitField0_ & 0x00000100) != 0);
    }
    /**
     * <code>optional double perLegPrice = 9;</code>
     * @return The perLegPrice.
     */
    @java.lang.Override
    public double getPerLegPrice() {
      return perLegPrice_;
    }

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      if (((bitField0_ & 0x00000001) != 0)) {
        output.writeInt32(1, conId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        output.writeInt32(2, ratio_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 3, action_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 4, exchange_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        output.writeInt32(5, openClose_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        output.writeInt32(6, shortSalesSlot_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 7, designatedLocation_);
      }
      if (((bitField0_ & 0x00000080) != 0)) {
        output.writeInt32(8, exemptCode_);
      }
      if (((bitField0_ & 0x00000100) != 0)) {
        output.writeDouble(9, perLegPrice_);
      }
      getUnknownFields().writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      if (((bitField0_ & 0x00000001) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(1, conId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(2, ratio_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(3, action_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(4, exchange_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(5, openClose_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(6, shortSalesSlot_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(7, designatedLocation_);
      }
      if (((bitField0_ & 0x00000080) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(8, exemptCode_);
      }
      if (((bitField0_ & 0x00000100) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeDoubleSize(9, perLegPrice_);
      }
      size += getUnknownFields().getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof com.ib.client.protobuf.ComboLegProto.ComboLeg)) {
        return super.equals(obj);
      }
      com.ib.client.protobuf.ComboLegProto.ComboLeg other = (com.ib.client.protobuf.ComboLegProto.ComboLeg) obj;

      if (hasConId() != other.hasConId()) return false;
      if (hasConId()) {
        if (getConId()
            != other.getConId()) return false;
      }
      if (hasRatio() != other.hasRatio()) return false;
      if (hasRatio()) {
        if (getRatio()
            != other.getRatio()) return false;
      }
      if (hasAction() != other.hasAction()) return false;
      if (hasAction()) {
        if (!getAction()
            .equals(other.getAction())) return false;
      }
      if (hasExchange() != other.hasExchange()) return false;
      if (hasExchange()) {
        if (!getExchange()
            .equals(other.getExchange())) return false;
      }
      if (hasOpenClose() != other.hasOpenClose()) return false;
      if (hasOpenClose()) {
        if (getOpenClose()
            != other.getOpenClose()) return false;
      }
      if (hasShortSalesSlot() != other.hasShortSalesSlot()) return false;
      if (hasShortSalesSlot()) {
        if (getShortSalesSlot()
            != other.getShortSalesSlot()) return false;
      }
      if (hasDesignatedLocation() != other.hasDesignatedLocation()) return false;
      if (hasDesignatedLocation()) {
        if (!getDesignatedLocation()
            .equals(other.getDesignatedLocation())) return false;
      }
      if (hasExemptCode() != other.hasExemptCode()) return false;
      if (hasExemptCode()) {
        if (getExemptCode()
            != other.getExemptCode()) return false;
      }
      if (hasPerLegPrice() != other.hasPerLegPrice()) return false;
      if (hasPerLegPrice()) {
        if (java.lang.Double.doubleToLongBits(getPerLegPrice())
            != java.lang.Double.doubleToLongBits(
                other.getPerLegPrice())) return false;
      }
      if (!getUnknownFields().equals(other.getUnknownFields())) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      if (hasConId()) {
        hash = (37 * hash) + CONID_FIELD_NUMBER;
        hash = (53 * hash) + getConId();
      }
      if (hasRatio()) {
        hash = (37 * hash) + RATIO_FIELD_NUMBER;
        hash = (53 * hash) + getRatio();
      }
      if (hasAction()) {
        hash = (37 * hash) + ACTION_FIELD_NUMBER;
        hash = (53 * hash) + getAction().hashCode();
      }
      if (hasExchange()) {
        hash = (37 * hash) + EXCHANGE_FIELD_NUMBER;
        hash = (53 * hash) + getExchange().hashCode();
      }
      if (hasOpenClose()) {
        hash = (37 * hash) + OPENCLOSE_FIELD_NUMBER;
        hash = (53 * hash) + getOpenClose();
      }
      if (hasShortSalesSlot()) {
        hash = (37 * hash) + SHORTSALESSLOT_FIELD_NUMBER;
        hash = (53 * hash) + getShortSalesSlot();
      }
      if (hasDesignatedLocation()) {
        hash = (37 * hash) + DESIGNATEDLOCATION_FIELD_NUMBER;
        hash = (53 * hash) + getDesignatedLocation().hashCode();
      }
      if (hasExemptCode()) {
        hash = (37 * hash) + EXEMPTCODE_FIELD_NUMBER;
        hash = (53 * hash) + getExemptCode();
      }
      if (hasPerLegPrice()) {
        hash = (37 * hash) + PERLEGPRICE_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            java.lang.Double.doubleToLongBits(getPerLegPrice()));
      }
      hash = (29 * hash) + getUnknownFields().hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input);
    }

    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.ComboLegProto.ComboLeg parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(com.ib.client.protobuf.ComboLegProto.ComboLeg prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessage.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code protobuf.ComboLeg}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessage.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:protobuf.ComboLeg)
        com.ib.client.protobuf.ComboLegProto.ComboLegOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return com.ib.client.protobuf.ComboLegProto.internal_static_protobuf_ComboLeg_descriptor;
      }

      @java.lang.Override
      protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return com.ib.client.protobuf.ComboLegProto.internal_static_protobuf_ComboLeg_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                com.ib.client.protobuf.ComboLegProto.ComboLeg.class, com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder.class);
      }

      // Construct using com.ib.client.protobuf.ComboLegProto.ComboLeg.newBuilder()
      private Builder() {

      }

      private Builder(
          com.google.protobuf.GeneratedMessage.BuilderParent parent) {
        super(parent);

      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        bitField0_ = 0;
        conId_ = 0;
        ratio_ = 0;
        action_ = "";
        exchange_ = "";
        openClose_ = 0;
        shortSalesSlot_ = 0;
        designatedLocation_ = "";
        exemptCode_ = 0;
        perLegPrice_ = 0D;
        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return com.ib.client.protobuf.ComboLegProto.internal_static_protobuf_ComboLeg_descriptor;
      }

      @java.lang.Override
      public com.ib.client.protobuf.ComboLegProto.ComboLeg getDefaultInstanceForType() {
        return com.ib.client.protobuf.ComboLegProto.ComboLeg.getDefaultInstance();
      }

      @java.lang.Override
      public com.ib.client.protobuf.ComboLegProto.ComboLeg build() {
        com.ib.client.protobuf.ComboLegProto.ComboLeg result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public com.ib.client.protobuf.ComboLegProto.ComboLeg buildPartial() {
        com.ib.client.protobuf.ComboLegProto.ComboLeg result = new com.ib.client.protobuf.ComboLegProto.ComboLeg(this);
        if (bitField0_ != 0) { buildPartial0(result); }
        onBuilt();
        return result;
      }

      private void buildPartial0(com.ib.client.protobuf.ComboLegProto.ComboLeg result) {
        int from_bitField0_ = bitField0_;
        int to_bitField0_ = 0;
        if (((from_bitField0_ & 0x00000001) != 0)) {
          result.conId_ = conId_;
          to_bitField0_ |= 0x00000001;
        }
        if (((from_bitField0_ & 0x00000002) != 0)) {
          result.ratio_ = ratio_;
          to_bitField0_ |= 0x00000002;
        }
        if (((from_bitField0_ & 0x00000004) != 0)) {
          result.action_ = action_;
          to_bitField0_ |= 0x00000004;
        }
        if (((from_bitField0_ & 0x00000008) != 0)) {
          result.exchange_ = exchange_;
          to_bitField0_ |= 0x00000008;
        }
        if (((from_bitField0_ & 0x00000010) != 0)) {
          result.openClose_ = openClose_;
          to_bitField0_ |= 0x00000010;
        }
        if (((from_bitField0_ & 0x00000020) != 0)) {
          result.shortSalesSlot_ = shortSalesSlot_;
          to_bitField0_ |= 0x00000020;
        }
        if (((from_bitField0_ & 0x00000040) != 0)) {
          result.designatedLocation_ = designatedLocation_;
          to_bitField0_ |= 0x00000040;
        }
        if (((from_bitField0_ & 0x00000080) != 0)) {
          result.exemptCode_ = exemptCode_;
          to_bitField0_ |= 0x00000080;
        }
        if (((from_bitField0_ & 0x00000100) != 0)) {
          result.perLegPrice_ = perLegPrice_;
          to_bitField0_ |= 0x00000100;
        }
        result.bitField0_ |= to_bitField0_;
      }

      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof com.ib.client.protobuf.ComboLegProto.ComboLeg) {
          return mergeFrom((com.ib.client.protobuf.ComboLegProto.ComboLeg)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(com.ib.client.protobuf.ComboLegProto.ComboLeg other) {
        if (other == com.ib.client.protobuf.ComboLegProto.ComboLeg.getDefaultInstance()) return this;
        if (other.hasConId()) {
          setConId(other.getConId());
        }
        if (other.hasRatio()) {
          setRatio(other.getRatio());
        }
        if (other.hasAction()) {
          action_ = other.action_;
          bitField0_ |= 0x00000004;
          onChanged();
        }
        if (other.hasExchange()) {
          exchange_ = other.exchange_;
          bitField0_ |= 0x00000008;
          onChanged();
        }
        if (other.hasOpenClose()) {
          setOpenClose(other.getOpenClose());
        }
        if (other.hasShortSalesSlot()) {
          setShortSalesSlot(other.getShortSalesSlot());
        }
        if (other.hasDesignatedLocation()) {
          designatedLocation_ = other.designatedLocation_;
          bitField0_ |= 0x00000040;
          onChanged();
        }
        if (other.hasExemptCode()) {
          setExemptCode(other.getExemptCode());
        }
        if (other.hasPerLegPrice()) {
          setPerLegPrice(other.getPerLegPrice());
        }
        this.mergeUnknownFields(other.getUnknownFields());
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        if (extensionRegistry == null) {
          throw new java.lang.NullPointerException();
        }
        try {
          boolean done = false;
          while (!done) {
            int tag = input.readTag();
            switch (tag) {
              case 0:
                done = true;
                break;
              case 8: {
                conId_ = input.readInt32();
                bitField0_ |= 0x00000001;
                break;
              } // case 8
              case 16: {
                ratio_ = input.readInt32();
                bitField0_ |= 0x00000002;
                break;
              } // case 16
              case 26: {
                action_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000004;
                break;
              } // case 26
              case 34: {
                exchange_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000008;
                break;
              } // case 34
              case 40: {
                openClose_ = input.readInt32();
                bitField0_ |= 0x00000010;
                break;
              } // case 40
              case 48: {
                shortSalesSlot_ = input.readInt32();
                bitField0_ |= 0x00000020;
                break;
              } // case 48
              case 58: {
                designatedLocation_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000040;
                break;
              } // case 58
              case 64: {
                exemptCode_ = input.readInt32();
                bitField0_ |= 0x00000080;
                break;
              } // case 64
              case 73: {
                perLegPrice_ = input.readDouble();
                bitField0_ |= 0x00000100;
                break;
              } // case 73
              default: {
                if (!super.parseUnknownField(input, extensionRegistry, tag)) {
                  done = true; // was an endgroup tag
                }
                break;
              } // default:
            } // switch (tag)
          } // while (!done)
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.unwrapIOException();
        } finally {
          onChanged();
        } // finally
        return this;
      }
      private int bitField0_;

      private int conId_ ;
      /**
       * <code>optional int32 conId = 1;</code>
       * @return Whether the conId field is set.
       */
      @java.lang.Override
      public boolean hasConId() {
        return ((bitField0_ & 0x00000001) != 0);
      }
      /**
       * <code>optional int32 conId = 1;</code>
       * @return The conId.
       */
      @java.lang.Override
      public int getConId() {
        return conId_;
      }
      /**
       * <code>optional int32 conId = 1;</code>
       * @param value The conId to set.
       * @return This builder for chaining.
       */
      public Builder setConId(int value) {

        conId_ = value;
        bitField0_ |= 0x00000001;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 conId = 1;</code>
       * @return This builder for chaining.
       */
      public Builder clearConId() {
        bitField0_ = (bitField0_ & ~0x00000001);
        conId_ = 0;
        onChanged();
        return this;
      }

      private int ratio_ ;
      /**
       * <code>optional int32 ratio = 2;</code>
       * @return Whether the ratio field is set.
       */
      @java.lang.Override
      public boolean hasRatio() {
        return ((bitField0_ & 0x00000002) != 0);
      }
      /**
       * <code>optional int32 ratio = 2;</code>
       * @return The ratio.
       */
      @java.lang.Override
      public int getRatio() {
        return ratio_;
      }
      /**
       * <code>optional int32 ratio = 2;</code>
       * @param value The ratio to set.
       * @return This builder for chaining.
       */
      public Builder setRatio(int value) {

        ratio_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 ratio = 2;</code>
       * @return This builder for chaining.
       */
      public Builder clearRatio() {
        bitField0_ = (bitField0_ & ~0x00000002);
        ratio_ = 0;
        onChanged();
        return this;
      }

      private java.lang.Object action_ = "";
      /**
       * <code>optional string action = 3;</code>
       * @return Whether the action field is set.
       */
      public boolean hasAction() {
        return ((bitField0_ & 0x00000004) != 0);
      }
      /**
       * <code>optional string action = 3;</code>
       * @return The action.
       */
      public java.lang.String getAction() {
        java.lang.Object ref = action_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          action_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string action = 3;</code>
       * @return The bytes for action.
       */
      public com.google.protobuf.ByteString
          getActionBytes() {
        java.lang.Object ref = action_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          action_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string action = 3;</code>
       * @param value The action to set.
       * @return This builder for chaining.
       */
      public Builder setAction(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        action_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }
      /**
       * <code>optional string action = 3;</code>
       * @return This builder for chaining.
       */
      public Builder clearAction() {
        action_ = getDefaultInstance().getAction();
        bitField0_ = (bitField0_ & ~0x00000004);
        onChanged();
        return this;
      }
      /**
       * <code>optional string action = 3;</code>
       * @param value The bytes for action to set.
       * @return This builder for chaining.
       */
      public Builder setActionBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        action_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }

      private java.lang.Object exchange_ = "";
      /**
       * <code>optional string exchange = 4;</code>
       * @return Whether the exchange field is set.
       */
      public boolean hasExchange() {
        return ((bitField0_ & 0x00000008) != 0);
      }
      /**
       * <code>optional string exchange = 4;</code>
       * @return The exchange.
       */
      public java.lang.String getExchange() {
        java.lang.Object ref = exchange_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          exchange_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string exchange = 4;</code>
       * @return The bytes for exchange.
       */
      public com.google.protobuf.ByteString
          getExchangeBytes() {
        java.lang.Object ref = exchange_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          exchange_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string exchange = 4;</code>
       * @param value The exchange to set.
       * @return This builder for chaining.
       */
      public Builder setExchange(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        exchange_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }
      /**
       * <code>optional string exchange = 4;</code>
       * @return This builder for chaining.
       */
      public Builder clearExchange() {
        exchange_ = getDefaultInstance().getExchange();
        bitField0_ = (bitField0_ & ~0x00000008);
        onChanged();
        return this;
      }
      /**
       * <code>optional string exchange = 4;</code>
       * @param value The bytes for exchange to set.
       * @return This builder for chaining.
       */
      public Builder setExchangeBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        exchange_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }

      private int openClose_ ;
      /**
       * <code>optional int32 openClose = 5;</code>
       * @return Whether the openClose field is set.
       */
      @java.lang.Override
      public boolean hasOpenClose() {
        return ((bitField0_ & 0x00000010) != 0);
      }
      /**
       * <code>optional int32 openClose = 5;</code>
       * @return The openClose.
       */
      @java.lang.Override
      public int getOpenClose() {
        return openClose_;
      }
      /**
       * <code>optional int32 openClose = 5;</code>
       * @param value The openClose to set.
       * @return This builder for chaining.
       */
      public Builder setOpenClose(int value) {

        openClose_ = value;
        bitField0_ |= 0x00000010;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 openClose = 5;</code>
       * @return This builder for chaining.
       */
      public Builder clearOpenClose() {
        bitField0_ = (bitField0_ & ~0x00000010);
        openClose_ = 0;
        onChanged();
        return this;
      }

      private int shortSalesSlot_ ;
      /**
       * <code>optional int32 shortSalesSlot = 6;</code>
       * @return Whether the shortSalesSlot field is set.
       */
      @java.lang.Override
      public boolean hasShortSalesSlot() {
        return ((bitField0_ & 0x00000020) != 0);
      }
      /**
       * <code>optional int32 shortSalesSlot = 6;</code>
       * @return The shortSalesSlot.
       */
      @java.lang.Override
      public int getShortSalesSlot() {
        return shortSalesSlot_;
      }
      /**
       * <code>optional int32 shortSalesSlot = 6;</code>
       * @param value The shortSalesSlot to set.
       * @return This builder for chaining.
       */
      public Builder setShortSalesSlot(int value) {

        shortSalesSlot_ = value;
        bitField0_ |= 0x00000020;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 shortSalesSlot = 6;</code>
       * @return This builder for chaining.
       */
      public Builder clearShortSalesSlot() {
        bitField0_ = (bitField0_ & ~0x00000020);
        shortSalesSlot_ = 0;
        onChanged();
        return this;
      }

      private java.lang.Object designatedLocation_ = "";
      /**
       * <code>optional string designatedLocation = 7;</code>
       * @return Whether the designatedLocation field is set.
       */
      public boolean hasDesignatedLocation() {
        return ((bitField0_ & 0x00000040) != 0);
      }
      /**
       * <code>optional string designatedLocation = 7;</code>
       * @return The designatedLocation.
       */
      public java.lang.String getDesignatedLocation() {
        java.lang.Object ref = designatedLocation_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          designatedLocation_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string designatedLocation = 7;</code>
       * @return The bytes for designatedLocation.
       */
      public com.google.protobuf.ByteString
          getDesignatedLocationBytes() {
        java.lang.Object ref = designatedLocation_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          designatedLocation_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string designatedLocation = 7;</code>
       * @param value The designatedLocation to set.
       * @return This builder for chaining.
       */
      public Builder setDesignatedLocation(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        designatedLocation_ = value;
        bitField0_ |= 0x00000040;
        onChanged();
        return this;
      }
      /**
       * <code>optional string designatedLocation = 7;</code>
       * @return This builder for chaining.
       */
      public Builder clearDesignatedLocation() {
        designatedLocation_ = getDefaultInstance().getDesignatedLocation();
        bitField0_ = (bitField0_ & ~0x00000040);
        onChanged();
        return this;
      }
      /**
       * <code>optional string designatedLocation = 7;</code>
       * @param value The bytes for designatedLocation to set.
       * @return This builder for chaining.
       */
      public Builder setDesignatedLocationBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        designatedLocation_ = value;
        bitField0_ |= 0x00000040;
        onChanged();
        return this;
      }

      private int exemptCode_ ;
      /**
       * <code>optional int32 exemptCode = 8;</code>
       * @return Whether the exemptCode field is set.
       */
      @java.lang.Override
      public boolean hasExemptCode() {
        return ((bitField0_ & 0x00000080) != 0);
      }
      /**
       * <code>optional int32 exemptCode = 8;</code>
       * @return The exemptCode.
       */
      @java.lang.Override
      public int getExemptCode() {
        return exemptCode_;
      }
      /**
       * <code>optional int32 exemptCode = 8;</code>
       * @param value The exemptCode to set.
       * @return This builder for chaining.
       */
      public Builder setExemptCode(int value) {

        exemptCode_ = value;
        bitField0_ |= 0x00000080;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 exemptCode = 8;</code>
       * @return This builder for chaining.
       */
      public Builder clearExemptCode() {
        bitField0_ = (bitField0_ & ~0x00000080);
        exemptCode_ = 0;
        onChanged();
        return this;
      }

      private double perLegPrice_ ;
      /**
       * <code>optional double perLegPrice = 9;</code>
       * @return Whether the perLegPrice field is set.
       */
      @java.lang.Override
      public boolean hasPerLegPrice() {
        return ((bitField0_ & 0x00000100) != 0);
      }
      /**
       * <code>optional double perLegPrice = 9;</code>
       * @return The perLegPrice.
       */
      @java.lang.Override
      public double getPerLegPrice() {
        return perLegPrice_;
      }
      /**
       * <code>optional double perLegPrice = 9;</code>
       * @param value The perLegPrice to set.
       * @return This builder for chaining.
       */
      public Builder setPerLegPrice(double value) {

        perLegPrice_ = value;
        bitField0_ |= 0x00000100;
        onChanged();
        return this;
      }
      /**
       * <code>optional double perLegPrice = 9;</code>
       * @return This builder for chaining.
       */
      public Builder clearPerLegPrice() {
        bitField0_ = (bitField0_ & ~0x00000100);
        perLegPrice_ = 0D;
        onChanged();
        return this;
      }

      // @@protoc_insertion_point(builder_scope:protobuf.ComboLeg)
    }

    // @@protoc_insertion_point(class_scope:protobuf.ComboLeg)
    private static final com.ib.client.protobuf.ComboLegProto.ComboLeg DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new com.ib.client.protobuf.ComboLegProto.ComboLeg();
    }

    public static com.ib.client.protobuf.ComboLegProto.ComboLeg getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<ComboLeg>
        PARSER = new com.google.protobuf.AbstractParser<ComboLeg>() {
      @java.lang.Override
      public ComboLeg parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        Builder builder = newBuilder();
        try {
          builder.mergeFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.setUnfinishedMessage(builder.buildPartial());
        } catch (com.google.protobuf.UninitializedMessageException e) {
          throw e.asInvalidProtocolBufferException().setUnfinishedMessage(builder.buildPartial());
        } catch (java.io.IOException e) {
          throw new com.google.protobuf.InvalidProtocolBufferException(e)
              .setUnfinishedMessage(builder.buildPartial());
        }
        return builder.buildPartial();
      }
    };

    public static com.google.protobuf.Parser<ComboLeg> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<ComboLeg> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public com.ib.client.protobuf.ComboLegProto.ComboLeg getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_protobuf_ComboLeg_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessage.FieldAccessorTable
      internal_static_protobuf_ComboLeg_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor
      getDescriptor() {
    return descriptor;
  }
  private static  com.google.protobuf.Descriptors.FileDescriptor
      descriptor;
  static {
    java.lang.String[] descriptorData = {
      "\n\016ComboLeg.proto\022\010protobuf\"\352\002\n\010ComboLeg\022" +
      "\022\n\005conId\030\001 \001(\005H\000\210\001\001\022\022\n\005ratio\030\002 \001(\005H\001\210\001\001\022" +
      "\023\n\006action\030\003 \001(\tH\002\210\001\001\022\025\n\010exchange\030\004 \001(\tH\003" +
      "\210\001\001\022\026\n\topenClose\030\005 \001(\005H\004\210\001\001\022\033\n\016shortSale" +
      "sSlot\030\006 \001(\005H\005\210\001\001\022\037\n\022designatedLocation\030\007" +
      " \001(\tH\006\210\001\001\022\027\n\nexemptCode\030\010 \001(\005H\007\210\001\001\022\030\n\013pe" +
      "rLegPrice\030\t \001(\001H\010\210\001\001B\010\n\006_conIdB\010\n\006_ratio" +
      "B\t\n\007_actionB\013\n\t_exchangeB\014\n\n_openCloseB\021" +
      "\n\017_shortSalesSlotB\025\n\023_designatedLocation" +
      "B\r\n\013_exemptCodeB\016\n\014_perLegPriceB8\n\026com.i" +
      "b.client.protobufB\rComboLegProto\252\002\016IBApi" +
      ".protobufb\006proto3"
    };
    descriptor = com.google.protobuf.Descriptors.FileDescriptor
      .internalBuildGeneratedFileFrom(descriptorData,
        new com.google.protobuf.Descriptors.FileDescriptor[] {
        });
    internal_static_protobuf_ComboLeg_descriptor =
      getDescriptor().getMessageTypes().get(0);
    internal_static_protobuf_ComboLeg_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessage.FieldAccessorTable(
        internal_static_protobuf_ComboLeg_descriptor,
        new java.lang.String[] { "ConId", "Ratio", "Action", "Exchange", "OpenClose", "ShortSalesSlot", "DesignatedLocation", "ExemptCode", "PerLegPrice", });
    descriptor.resolveAllFeaturesImmutable();
  }

  // @@protoc_insertion_point(outer_class_scope)
}
