// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: Contract.proto
// Protobuf Java Version: 4.29.3

package com.ib.client.protobuf;

public final class ContractProto {
  private ContractProto() {}
  static {
    com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
      com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
      /* major= */ 4,
      /* minor= */ 29,
      /* patch= */ 3,
      /* suffix= */ "",
      ContractProto.class.getName());
  }
  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistryLite registry) {
  }

  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistry registry) {
    registerAllExtensions(
        (com.google.protobuf.ExtensionRegistryLite) registry);
  }
  public interface ContractOrBuilder extends
      // @@protoc_insertion_point(interface_extends:protobuf.Contract)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>optional int32 conId = 1;</code>
     * @return Whether the conId field is set.
     */
    boolean hasConId();
    /**
     * <code>optional int32 conId = 1;</code>
     * @return The conId.
     */
    int getConId();

    /**
     * <code>optional string symbol = 2;</code>
     * @return Whether the symbol field is set.
     */
    boolean hasSymbol();
    /**
     * <code>optional string symbol = 2;</code>
     * @return The symbol.
     */
    java.lang.String getSymbol();
    /**
     * <code>optional string symbol = 2;</code>
     * @return The bytes for symbol.
     */
    com.google.protobuf.ByteString
        getSymbolBytes();

    /**
     * <code>optional string secType = 3;</code>
     * @return Whether the secType field is set.
     */
    boolean hasSecType();
    /**
     * <code>optional string secType = 3;</code>
     * @return The secType.
     */
    java.lang.String getSecType();
    /**
     * <code>optional string secType = 3;</code>
     * @return The bytes for secType.
     */
    com.google.protobuf.ByteString
        getSecTypeBytes();

    /**
     * <code>optional string lastTradeDateOrContractMonth = 4;</code>
     * @return Whether the lastTradeDateOrContractMonth field is set.
     */
    boolean hasLastTradeDateOrContractMonth();
    /**
     * <code>optional string lastTradeDateOrContractMonth = 4;</code>
     * @return The lastTradeDateOrContractMonth.
     */
    java.lang.String getLastTradeDateOrContractMonth();
    /**
     * <code>optional string lastTradeDateOrContractMonth = 4;</code>
     * @return The bytes for lastTradeDateOrContractMonth.
     */
    com.google.protobuf.ByteString
        getLastTradeDateOrContractMonthBytes();

    /**
     * <code>optional double strike = 5;</code>
     * @return Whether the strike field is set.
     */
    boolean hasStrike();
    /**
     * <code>optional double strike = 5;</code>
     * @return The strike.
     */
    double getStrike();

    /**
     * <code>optional string right = 6;</code>
     * @return Whether the right field is set.
     */
    boolean hasRight();
    /**
     * <code>optional string right = 6;</code>
     * @return The right.
     */
    java.lang.String getRight();
    /**
     * <code>optional string right = 6;</code>
     * @return The bytes for right.
     */
    com.google.protobuf.ByteString
        getRightBytes();

    /**
     * <code>optional double multiplier = 7;</code>
     * @return Whether the multiplier field is set.
     */
    boolean hasMultiplier();
    /**
     * <code>optional double multiplier = 7;</code>
     * @return The multiplier.
     */
    double getMultiplier();

    /**
     * <code>optional string exchange = 8;</code>
     * @return Whether the exchange field is set.
     */
    boolean hasExchange();
    /**
     * <code>optional string exchange = 8;</code>
     * @return The exchange.
     */
    java.lang.String getExchange();
    /**
     * <code>optional string exchange = 8;</code>
     * @return The bytes for exchange.
     */
    com.google.protobuf.ByteString
        getExchangeBytes();

    /**
     * <code>optional string primaryExch = 9;</code>
     * @return Whether the primaryExch field is set.
     */
    boolean hasPrimaryExch();
    /**
     * <code>optional string primaryExch = 9;</code>
     * @return The primaryExch.
     */
    java.lang.String getPrimaryExch();
    /**
     * <code>optional string primaryExch = 9;</code>
     * @return The bytes for primaryExch.
     */
    com.google.protobuf.ByteString
        getPrimaryExchBytes();

    /**
     * <code>optional string currency = 10;</code>
     * @return Whether the currency field is set.
     */
    boolean hasCurrency();
    /**
     * <code>optional string currency = 10;</code>
     * @return The currency.
     */
    java.lang.String getCurrency();
    /**
     * <code>optional string currency = 10;</code>
     * @return The bytes for currency.
     */
    com.google.protobuf.ByteString
        getCurrencyBytes();

    /**
     * <code>optional string localSymbol = 11;</code>
     * @return Whether the localSymbol field is set.
     */
    boolean hasLocalSymbol();
    /**
     * <code>optional string localSymbol = 11;</code>
     * @return The localSymbol.
     */
    java.lang.String getLocalSymbol();
    /**
     * <code>optional string localSymbol = 11;</code>
     * @return The bytes for localSymbol.
     */
    com.google.protobuf.ByteString
        getLocalSymbolBytes();

    /**
     * <code>optional string tradingClass = 12;</code>
     * @return Whether the tradingClass field is set.
     */
    boolean hasTradingClass();
    /**
     * <code>optional string tradingClass = 12;</code>
     * @return The tradingClass.
     */
    java.lang.String getTradingClass();
    /**
     * <code>optional string tradingClass = 12;</code>
     * @return The bytes for tradingClass.
     */
    com.google.protobuf.ByteString
        getTradingClassBytes();

    /**
     * <code>optional string secIdType = 13;</code>
     * @return Whether the secIdType field is set.
     */
    boolean hasSecIdType();
    /**
     * <code>optional string secIdType = 13;</code>
     * @return The secIdType.
     */
    java.lang.String getSecIdType();
    /**
     * <code>optional string secIdType = 13;</code>
     * @return The bytes for secIdType.
     */
    com.google.protobuf.ByteString
        getSecIdTypeBytes();

    /**
     * <code>optional string secId = 14;</code>
     * @return Whether the secId field is set.
     */
    boolean hasSecId();
    /**
     * <code>optional string secId = 14;</code>
     * @return The secId.
     */
    java.lang.String getSecId();
    /**
     * <code>optional string secId = 14;</code>
     * @return The bytes for secId.
     */
    com.google.protobuf.ByteString
        getSecIdBytes();

    /**
     * <code>optional string description = 15;</code>
     * @return Whether the description field is set.
     */
    boolean hasDescription();
    /**
     * <code>optional string description = 15;</code>
     * @return The description.
     */
    java.lang.String getDescription();
    /**
     * <code>optional string description = 15;</code>
     * @return The bytes for description.
     */
    com.google.protobuf.ByteString
        getDescriptionBytes();

    /**
     * <code>optional string issuerId = 16;</code>
     * @return Whether the issuerId field is set.
     */
    boolean hasIssuerId();
    /**
     * <code>optional string issuerId = 16;</code>
     * @return The issuerId.
     */
    java.lang.String getIssuerId();
    /**
     * <code>optional string issuerId = 16;</code>
     * @return The bytes for issuerId.
     */
    com.google.protobuf.ByteString
        getIssuerIdBytes();

    /**
     * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
     * @return Whether the deltaNeutralContract field is set.
     */
    boolean hasDeltaNeutralContract();
    /**
     * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
     * @return The deltaNeutralContract.
     */
    com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract getDeltaNeutralContract();
    /**
     * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
     */
    com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContractOrBuilder getDeltaNeutralContractOrBuilder();

    /**
     * <code>optional bool includeExpired = 18;</code>
     * @return Whether the includeExpired field is set.
     */
    boolean hasIncludeExpired();
    /**
     * <code>optional bool includeExpired = 18;</code>
     * @return The includeExpired.
     */
    boolean getIncludeExpired();

    /**
     * <code>optional string comboLegsDescrip = 19;</code>
     * @return Whether the comboLegsDescrip field is set.
     */
    boolean hasComboLegsDescrip();
    /**
     * <code>optional string comboLegsDescrip = 19;</code>
     * @return The comboLegsDescrip.
     */
    java.lang.String getComboLegsDescrip();
    /**
     * <code>optional string comboLegsDescrip = 19;</code>
     * @return The bytes for comboLegsDescrip.
     */
    com.google.protobuf.ByteString
        getComboLegsDescripBytes();

    /**
     * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
     */
    java.util.List<com.ib.client.protobuf.ComboLegProto.ComboLeg> 
        getComboLegsList();
    /**
     * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
     */
    com.ib.client.protobuf.ComboLegProto.ComboLeg getComboLegs(int index);
    /**
     * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
     */
    int getComboLegsCount();
    /**
     * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
     */
    java.util.List<? extends com.ib.client.protobuf.ComboLegProto.ComboLegOrBuilder> 
        getComboLegsOrBuilderList();
    /**
     * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
     */
    com.ib.client.protobuf.ComboLegProto.ComboLegOrBuilder getComboLegsOrBuilder(
        int index);
  }
  /**
   * Protobuf type {@code protobuf.Contract}
   */
  public static final class Contract extends
      com.google.protobuf.GeneratedMessage implements
      // @@protoc_insertion_point(message_implements:protobuf.Contract)
      ContractOrBuilder {
  private static final long serialVersionUID = 0L;
    static {
      com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
        com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
        /* major= */ 4,
        /* minor= */ 29,
        /* patch= */ 3,
        /* suffix= */ "",
        Contract.class.getName());
    }
    // Use Contract.newBuilder() to construct.
    private Contract(com.google.protobuf.GeneratedMessage.Builder<?> builder) {
      super(builder);
    }
    private Contract() {
      symbol_ = "";
      secType_ = "";
      lastTradeDateOrContractMonth_ = "";
      right_ = "";
      exchange_ = "";
      primaryExch_ = "";
      currency_ = "";
      localSymbol_ = "";
      tradingClass_ = "";
      secIdType_ = "";
      secId_ = "";
      description_ = "";
      issuerId_ = "";
      comboLegsDescrip_ = "";
      comboLegs_ = java.util.Collections.emptyList();
    }

    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return com.ib.client.protobuf.ContractProto.internal_static_protobuf_Contract_descriptor;
    }

    @java.lang.Override
    protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return com.ib.client.protobuf.ContractProto.internal_static_protobuf_Contract_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              com.ib.client.protobuf.ContractProto.Contract.class, com.ib.client.protobuf.ContractProto.Contract.Builder.class);
    }

    private int bitField0_;
    public static final int CONID_FIELD_NUMBER = 1;
    private int conId_ = 0;
    /**
     * <code>optional int32 conId = 1;</code>
     * @return Whether the conId field is set.
     */
    @java.lang.Override
    public boolean hasConId() {
      return ((bitField0_ & 0x00000001) != 0);
    }
    /**
     * <code>optional int32 conId = 1;</code>
     * @return The conId.
     */
    @java.lang.Override
    public int getConId() {
      return conId_;
    }

    public static final int SYMBOL_FIELD_NUMBER = 2;
    @SuppressWarnings("serial")
    private volatile java.lang.Object symbol_ = "";
    /**
     * <code>optional string symbol = 2;</code>
     * @return Whether the symbol field is set.
     */
    @java.lang.Override
    public boolean hasSymbol() {
      return ((bitField0_ & 0x00000002) != 0);
    }
    /**
     * <code>optional string symbol = 2;</code>
     * @return The symbol.
     */
    @java.lang.Override
    public java.lang.String getSymbol() {
      java.lang.Object ref = symbol_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        symbol_ = s;
        return s;
      }
    }
    /**
     * <code>optional string symbol = 2;</code>
     * @return The bytes for symbol.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getSymbolBytes() {
      java.lang.Object ref = symbol_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        symbol_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int SECTYPE_FIELD_NUMBER = 3;
    @SuppressWarnings("serial")
    private volatile java.lang.Object secType_ = "";
    /**
     * <code>optional string secType = 3;</code>
     * @return Whether the secType field is set.
     */
    @java.lang.Override
    public boolean hasSecType() {
      return ((bitField0_ & 0x00000004) != 0);
    }
    /**
     * <code>optional string secType = 3;</code>
     * @return The secType.
     */
    @java.lang.Override
    public java.lang.String getSecType() {
      java.lang.Object ref = secType_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        secType_ = s;
        return s;
      }
    }
    /**
     * <code>optional string secType = 3;</code>
     * @return The bytes for secType.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getSecTypeBytes() {
      java.lang.Object ref = secType_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        secType_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int LASTTRADEDATEORCONTRACTMONTH_FIELD_NUMBER = 4;
    @SuppressWarnings("serial")
    private volatile java.lang.Object lastTradeDateOrContractMonth_ = "";
    /**
     * <code>optional string lastTradeDateOrContractMonth = 4;</code>
     * @return Whether the lastTradeDateOrContractMonth field is set.
     */
    @java.lang.Override
    public boolean hasLastTradeDateOrContractMonth() {
      return ((bitField0_ & 0x00000008) != 0);
    }
    /**
     * <code>optional string lastTradeDateOrContractMonth = 4;</code>
     * @return The lastTradeDateOrContractMonth.
     */
    @java.lang.Override
    public java.lang.String getLastTradeDateOrContractMonth() {
      java.lang.Object ref = lastTradeDateOrContractMonth_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        lastTradeDateOrContractMonth_ = s;
        return s;
      }
    }
    /**
     * <code>optional string lastTradeDateOrContractMonth = 4;</code>
     * @return The bytes for lastTradeDateOrContractMonth.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getLastTradeDateOrContractMonthBytes() {
      java.lang.Object ref = lastTradeDateOrContractMonth_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        lastTradeDateOrContractMonth_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int STRIKE_FIELD_NUMBER = 5;
    private double strike_ = 0D;
    /**
     * <code>optional double strike = 5;</code>
     * @return Whether the strike field is set.
     */
    @java.lang.Override
    public boolean hasStrike() {
      return ((bitField0_ & 0x00000010) != 0);
    }
    /**
     * <code>optional double strike = 5;</code>
     * @return The strike.
     */
    @java.lang.Override
    public double getStrike() {
      return strike_;
    }

    public static final int RIGHT_FIELD_NUMBER = 6;
    @SuppressWarnings("serial")
    private volatile java.lang.Object right_ = "";
    /**
     * <code>optional string right = 6;</code>
     * @return Whether the right field is set.
     */
    @java.lang.Override
    public boolean hasRight() {
      return ((bitField0_ & 0x00000020) != 0);
    }
    /**
     * <code>optional string right = 6;</code>
     * @return The right.
     */
    @java.lang.Override
    public java.lang.String getRight() {
      java.lang.Object ref = right_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        right_ = s;
        return s;
      }
    }
    /**
     * <code>optional string right = 6;</code>
     * @return The bytes for right.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getRightBytes() {
      java.lang.Object ref = right_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        right_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int MULTIPLIER_FIELD_NUMBER = 7;
    private double multiplier_ = 0D;
    /**
     * <code>optional double multiplier = 7;</code>
     * @return Whether the multiplier field is set.
     */
    @java.lang.Override
    public boolean hasMultiplier() {
      return ((bitField0_ & 0x00000040) != 0);
    }
    /**
     * <code>optional double multiplier = 7;</code>
     * @return The multiplier.
     */
    @java.lang.Override
    public double getMultiplier() {
      return multiplier_;
    }

    public static final int EXCHANGE_FIELD_NUMBER = 8;
    @SuppressWarnings("serial")
    private volatile java.lang.Object exchange_ = "";
    /**
     * <code>optional string exchange = 8;</code>
     * @return Whether the exchange field is set.
     */
    @java.lang.Override
    public boolean hasExchange() {
      return ((bitField0_ & 0x00000080) != 0);
    }
    /**
     * <code>optional string exchange = 8;</code>
     * @return The exchange.
     */
    @java.lang.Override
    public java.lang.String getExchange() {
      java.lang.Object ref = exchange_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        exchange_ = s;
        return s;
      }
    }
    /**
     * <code>optional string exchange = 8;</code>
     * @return The bytes for exchange.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getExchangeBytes() {
      java.lang.Object ref = exchange_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        exchange_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int PRIMARYEXCH_FIELD_NUMBER = 9;
    @SuppressWarnings("serial")
    private volatile java.lang.Object primaryExch_ = "";
    /**
     * <code>optional string primaryExch = 9;</code>
     * @return Whether the primaryExch field is set.
     */
    @java.lang.Override
    public boolean hasPrimaryExch() {
      return ((bitField0_ & 0x00000100) != 0);
    }
    /**
     * <code>optional string primaryExch = 9;</code>
     * @return The primaryExch.
     */
    @java.lang.Override
    public java.lang.String getPrimaryExch() {
      java.lang.Object ref = primaryExch_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        primaryExch_ = s;
        return s;
      }
    }
    /**
     * <code>optional string primaryExch = 9;</code>
     * @return The bytes for primaryExch.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getPrimaryExchBytes() {
      java.lang.Object ref = primaryExch_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        primaryExch_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int CURRENCY_FIELD_NUMBER = 10;
    @SuppressWarnings("serial")
    private volatile java.lang.Object currency_ = "";
    /**
     * <code>optional string currency = 10;</code>
     * @return Whether the currency field is set.
     */
    @java.lang.Override
    public boolean hasCurrency() {
      return ((bitField0_ & 0x00000200) != 0);
    }
    /**
     * <code>optional string currency = 10;</code>
     * @return The currency.
     */
    @java.lang.Override
    public java.lang.String getCurrency() {
      java.lang.Object ref = currency_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        currency_ = s;
        return s;
      }
    }
    /**
     * <code>optional string currency = 10;</code>
     * @return The bytes for currency.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getCurrencyBytes() {
      java.lang.Object ref = currency_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        currency_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int LOCALSYMBOL_FIELD_NUMBER = 11;
    @SuppressWarnings("serial")
    private volatile java.lang.Object localSymbol_ = "";
    /**
     * <code>optional string localSymbol = 11;</code>
     * @return Whether the localSymbol field is set.
     */
    @java.lang.Override
    public boolean hasLocalSymbol() {
      return ((bitField0_ & 0x00000400) != 0);
    }
    /**
     * <code>optional string localSymbol = 11;</code>
     * @return The localSymbol.
     */
    @java.lang.Override
    public java.lang.String getLocalSymbol() {
      java.lang.Object ref = localSymbol_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        localSymbol_ = s;
        return s;
      }
    }
    /**
     * <code>optional string localSymbol = 11;</code>
     * @return The bytes for localSymbol.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getLocalSymbolBytes() {
      java.lang.Object ref = localSymbol_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        localSymbol_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int TRADINGCLASS_FIELD_NUMBER = 12;
    @SuppressWarnings("serial")
    private volatile java.lang.Object tradingClass_ = "";
    /**
     * <code>optional string tradingClass = 12;</code>
     * @return Whether the tradingClass field is set.
     */
    @java.lang.Override
    public boolean hasTradingClass() {
      return ((bitField0_ & 0x00000800) != 0);
    }
    /**
     * <code>optional string tradingClass = 12;</code>
     * @return The tradingClass.
     */
    @java.lang.Override
    public java.lang.String getTradingClass() {
      java.lang.Object ref = tradingClass_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        tradingClass_ = s;
        return s;
      }
    }
    /**
     * <code>optional string tradingClass = 12;</code>
     * @return The bytes for tradingClass.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getTradingClassBytes() {
      java.lang.Object ref = tradingClass_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        tradingClass_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int SECIDTYPE_FIELD_NUMBER = 13;
    @SuppressWarnings("serial")
    private volatile java.lang.Object secIdType_ = "";
    /**
     * <code>optional string secIdType = 13;</code>
     * @return Whether the secIdType field is set.
     */
    @java.lang.Override
    public boolean hasSecIdType() {
      return ((bitField0_ & 0x00001000) != 0);
    }
    /**
     * <code>optional string secIdType = 13;</code>
     * @return The secIdType.
     */
    @java.lang.Override
    public java.lang.String getSecIdType() {
      java.lang.Object ref = secIdType_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        secIdType_ = s;
        return s;
      }
    }
    /**
     * <code>optional string secIdType = 13;</code>
     * @return The bytes for secIdType.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getSecIdTypeBytes() {
      java.lang.Object ref = secIdType_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        secIdType_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int SECID_FIELD_NUMBER = 14;
    @SuppressWarnings("serial")
    private volatile java.lang.Object secId_ = "";
    /**
     * <code>optional string secId = 14;</code>
     * @return Whether the secId field is set.
     */
    @java.lang.Override
    public boolean hasSecId() {
      return ((bitField0_ & 0x00002000) != 0);
    }
    /**
     * <code>optional string secId = 14;</code>
     * @return The secId.
     */
    @java.lang.Override
    public java.lang.String getSecId() {
      java.lang.Object ref = secId_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        secId_ = s;
        return s;
      }
    }
    /**
     * <code>optional string secId = 14;</code>
     * @return The bytes for secId.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getSecIdBytes() {
      java.lang.Object ref = secId_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        secId_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int DESCRIPTION_FIELD_NUMBER = 15;
    @SuppressWarnings("serial")
    private volatile java.lang.Object description_ = "";
    /**
     * <code>optional string description = 15;</code>
     * @return Whether the description field is set.
     */
    @java.lang.Override
    public boolean hasDescription() {
      return ((bitField0_ & 0x00004000) != 0);
    }
    /**
     * <code>optional string description = 15;</code>
     * @return The description.
     */
    @java.lang.Override
    public java.lang.String getDescription() {
      java.lang.Object ref = description_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        description_ = s;
        return s;
      }
    }
    /**
     * <code>optional string description = 15;</code>
     * @return The bytes for description.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getDescriptionBytes() {
      java.lang.Object ref = description_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        description_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int ISSUERID_FIELD_NUMBER = 16;
    @SuppressWarnings("serial")
    private volatile java.lang.Object issuerId_ = "";
    /**
     * <code>optional string issuerId = 16;</code>
     * @return Whether the issuerId field is set.
     */
    @java.lang.Override
    public boolean hasIssuerId() {
      return ((bitField0_ & 0x00008000) != 0);
    }
    /**
     * <code>optional string issuerId = 16;</code>
     * @return The issuerId.
     */
    @java.lang.Override
    public java.lang.String getIssuerId() {
      java.lang.Object ref = issuerId_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        issuerId_ = s;
        return s;
      }
    }
    /**
     * <code>optional string issuerId = 16;</code>
     * @return The bytes for issuerId.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getIssuerIdBytes() {
      java.lang.Object ref = issuerId_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        issuerId_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int DELTANEUTRALCONTRACT_FIELD_NUMBER = 17;
    private com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract deltaNeutralContract_;
    /**
     * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
     * @return Whether the deltaNeutralContract field is set.
     */
    @java.lang.Override
    public boolean hasDeltaNeutralContract() {
      return ((bitField0_ & 0x00010000) != 0);
    }
    /**
     * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
     * @return The deltaNeutralContract.
     */
    @java.lang.Override
    public com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract getDeltaNeutralContract() {
      return deltaNeutralContract_ == null ? com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.getDefaultInstance() : deltaNeutralContract_;
    }
    /**
     * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
     */
    @java.lang.Override
    public com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContractOrBuilder getDeltaNeutralContractOrBuilder() {
      return deltaNeutralContract_ == null ? com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.getDefaultInstance() : deltaNeutralContract_;
    }

    public static final int INCLUDEEXPIRED_FIELD_NUMBER = 18;
    private boolean includeExpired_ = false;
    /**
     * <code>optional bool includeExpired = 18;</code>
     * @return Whether the includeExpired field is set.
     */
    @java.lang.Override
    public boolean hasIncludeExpired() {
      return ((bitField0_ & 0x00020000) != 0);
    }
    /**
     * <code>optional bool includeExpired = 18;</code>
     * @return The includeExpired.
     */
    @java.lang.Override
    public boolean getIncludeExpired() {
      return includeExpired_;
    }

    public static final int COMBOLEGSDESCRIP_FIELD_NUMBER = 19;
    @SuppressWarnings("serial")
    private volatile java.lang.Object comboLegsDescrip_ = "";
    /**
     * <code>optional string comboLegsDescrip = 19;</code>
     * @return Whether the comboLegsDescrip field is set.
     */
    @java.lang.Override
    public boolean hasComboLegsDescrip() {
      return ((bitField0_ & 0x00040000) != 0);
    }
    /**
     * <code>optional string comboLegsDescrip = 19;</code>
     * @return The comboLegsDescrip.
     */
    @java.lang.Override
    public java.lang.String getComboLegsDescrip() {
      java.lang.Object ref = comboLegsDescrip_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        comboLegsDescrip_ = s;
        return s;
      }
    }
    /**
     * <code>optional string comboLegsDescrip = 19;</code>
     * @return The bytes for comboLegsDescrip.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getComboLegsDescripBytes() {
      java.lang.Object ref = comboLegsDescrip_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        comboLegsDescrip_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int COMBOLEGS_FIELD_NUMBER = 20;
    @SuppressWarnings("serial")
    private java.util.List<com.ib.client.protobuf.ComboLegProto.ComboLeg> comboLegs_;
    /**
     * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
     */
    @java.lang.Override
    public java.util.List<com.ib.client.protobuf.ComboLegProto.ComboLeg> getComboLegsList() {
      return comboLegs_;
    }
    /**
     * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
     */
    @java.lang.Override
    public java.util.List<? extends com.ib.client.protobuf.ComboLegProto.ComboLegOrBuilder> 
        getComboLegsOrBuilderList() {
      return comboLegs_;
    }
    /**
     * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
     */
    @java.lang.Override
    public int getComboLegsCount() {
      return comboLegs_.size();
    }
    /**
     * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
     */
    @java.lang.Override
    public com.ib.client.protobuf.ComboLegProto.ComboLeg getComboLegs(int index) {
      return comboLegs_.get(index);
    }
    /**
     * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
     */
    @java.lang.Override
    public com.ib.client.protobuf.ComboLegProto.ComboLegOrBuilder getComboLegsOrBuilder(
        int index) {
      return comboLegs_.get(index);
    }

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      if (((bitField0_ & 0x00000001) != 0)) {
        output.writeInt32(1, conId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 2, symbol_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 3, secType_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 4, lastTradeDateOrContractMonth_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        output.writeDouble(5, strike_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 6, right_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        output.writeDouble(7, multiplier_);
      }
      if (((bitField0_ & 0x00000080) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 8, exchange_);
      }
      if (((bitField0_ & 0x00000100) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 9, primaryExch_);
      }
      if (((bitField0_ & 0x00000200) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 10, currency_);
      }
      if (((bitField0_ & 0x00000400) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 11, localSymbol_);
      }
      if (((bitField0_ & 0x00000800) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 12, tradingClass_);
      }
      if (((bitField0_ & 0x00001000) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 13, secIdType_);
      }
      if (((bitField0_ & 0x00002000) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 14, secId_);
      }
      if (((bitField0_ & 0x00004000) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 15, description_);
      }
      if (((bitField0_ & 0x00008000) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 16, issuerId_);
      }
      if (((bitField0_ & 0x00010000) != 0)) {
        output.writeMessage(17, getDeltaNeutralContract());
      }
      if (((bitField0_ & 0x00020000) != 0)) {
        output.writeBool(18, includeExpired_);
      }
      if (((bitField0_ & 0x00040000) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 19, comboLegsDescrip_);
      }
      for (int i = 0; i < comboLegs_.size(); i++) {
        output.writeMessage(20, comboLegs_.get(i));
      }
      getUnknownFields().writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      if (((bitField0_ & 0x00000001) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(1, conId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(2, symbol_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(3, secType_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(4, lastTradeDateOrContractMonth_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeDoubleSize(5, strike_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(6, right_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeDoubleSize(7, multiplier_);
      }
      if (((bitField0_ & 0x00000080) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(8, exchange_);
      }
      if (((bitField0_ & 0x00000100) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(9, primaryExch_);
      }
      if (((bitField0_ & 0x00000200) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(10, currency_);
      }
      if (((bitField0_ & 0x00000400) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(11, localSymbol_);
      }
      if (((bitField0_ & 0x00000800) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(12, tradingClass_);
      }
      if (((bitField0_ & 0x00001000) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(13, secIdType_);
      }
      if (((bitField0_ & 0x00002000) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(14, secId_);
      }
      if (((bitField0_ & 0x00004000) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(15, description_);
      }
      if (((bitField0_ & 0x00008000) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(16, issuerId_);
      }
      if (((bitField0_ & 0x00010000) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeMessageSize(17, getDeltaNeutralContract());
      }
      if (((bitField0_ & 0x00020000) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeBoolSize(18, includeExpired_);
      }
      if (((bitField0_ & 0x00040000) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(19, comboLegsDescrip_);
      }
      for (int i = 0; i < comboLegs_.size(); i++) {
        size += com.google.protobuf.CodedOutputStream
          .computeMessageSize(20, comboLegs_.get(i));
      }
      size += getUnknownFields().getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof com.ib.client.protobuf.ContractProto.Contract)) {
        return super.equals(obj);
      }
      com.ib.client.protobuf.ContractProto.Contract other = (com.ib.client.protobuf.ContractProto.Contract) obj;

      if (hasConId() != other.hasConId()) return false;
      if (hasConId()) {
        if (getConId()
            != other.getConId()) return false;
      }
      if (hasSymbol() != other.hasSymbol()) return false;
      if (hasSymbol()) {
        if (!getSymbol()
            .equals(other.getSymbol())) return false;
      }
      if (hasSecType() != other.hasSecType()) return false;
      if (hasSecType()) {
        if (!getSecType()
            .equals(other.getSecType())) return false;
      }
      if (hasLastTradeDateOrContractMonth() != other.hasLastTradeDateOrContractMonth()) return false;
      if (hasLastTradeDateOrContractMonth()) {
        if (!getLastTradeDateOrContractMonth()
            .equals(other.getLastTradeDateOrContractMonth())) return false;
      }
      if (hasStrike() != other.hasStrike()) return false;
      if (hasStrike()) {
        if (java.lang.Double.doubleToLongBits(getStrike())
            != java.lang.Double.doubleToLongBits(
                other.getStrike())) return false;
      }
      if (hasRight() != other.hasRight()) return false;
      if (hasRight()) {
        if (!getRight()
            .equals(other.getRight())) return false;
      }
      if (hasMultiplier() != other.hasMultiplier()) return false;
      if (hasMultiplier()) {
        if (java.lang.Double.doubleToLongBits(getMultiplier())
            != java.lang.Double.doubleToLongBits(
                other.getMultiplier())) return false;
      }
      if (hasExchange() != other.hasExchange()) return false;
      if (hasExchange()) {
        if (!getExchange()
            .equals(other.getExchange())) return false;
      }
      if (hasPrimaryExch() != other.hasPrimaryExch()) return false;
      if (hasPrimaryExch()) {
        if (!getPrimaryExch()
            .equals(other.getPrimaryExch())) return false;
      }
      if (hasCurrency() != other.hasCurrency()) return false;
      if (hasCurrency()) {
        if (!getCurrency()
            .equals(other.getCurrency())) return false;
      }
      if (hasLocalSymbol() != other.hasLocalSymbol()) return false;
      if (hasLocalSymbol()) {
        if (!getLocalSymbol()
            .equals(other.getLocalSymbol())) return false;
      }
      if (hasTradingClass() != other.hasTradingClass()) return false;
      if (hasTradingClass()) {
        if (!getTradingClass()
            .equals(other.getTradingClass())) return false;
      }
      if (hasSecIdType() != other.hasSecIdType()) return false;
      if (hasSecIdType()) {
        if (!getSecIdType()
            .equals(other.getSecIdType())) return false;
      }
      if (hasSecId() != other.hasSecId()) return false;
      if (hasSecId()) {
        if (!getSecId()
            .equals(other.getSecId())) return false;
      }
      if (hasDescription() != other.hasDescription()) return false;
      if (hasDescription()) {
        if (!getDescription()
            .equals(other.getDescription())) return false;
      }
      if (hasIssuerId() != other.hasIssuerId()) return false;
      if (hasIssuerId()) {
        if (!getIssuerId()
            .equals(other.getIssuerId())) return false;
      }
      if (hasDeltaNeutralContract() != other.hasDeltaNeutralContract()) return false;
      if (hasDeltaNeutralContract()) {
        if (!getDeltaNeutralContract()
            .equals(other.getDeltaNeutralContract())) return false;
      }
      if (hasIncludeExpired() != other.hasIncludeExpired()) return false;
      if (hasIncludeExpired()) {
        if (getIncludeExpired()
            != other.getIncludeExpired()) return false;
      }
      if (hasComboLegsDescrip() != other.hasComboLegsDescrip()) return false;
      if (hasComboLegsDescrip()) {
        if (!getComboLegsDescrip()
            .equals(other.getComboLegsDescrip())) return false;
      }
      if (!getComboLegsList()
          .equals(other.getComboLegsList())) return false;
      if (!getUnknownFields().equals(other.getUnknownFields())) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      if (hasConId()) {
        hash = (37 * hash) + CONID_FIELD_NUMBER;
        hash = (53 * hash) + getConId();
      }
      if (hasSymbol()) {
        hash = (37 * hash) + SYMBOL_FIELD_NUMBER;
        hash = (53 * hash) + getSymbol().hashCode();
      }
      if (hasSecType()) {
        hash = (37 * hash) + SECTYPE_FIELD_NUMBER;
        hash = (53 * hash) + getSecType().hashCode();
      }
      if (hasLastTradeDateOrContractMonth()) {
        hash = (37 * hash) + LASTTRADEDATEORCONTRACTMONTH_FIELD_NUMBER;
        hash = (53 * hash) + getLastTradeDateOrContractMonth().hashCode();
      }
      if (hasStrike()) {
        hash = (37 * hash) + STRIKE_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            java.lang.Double.doubleToLongBits(getStrike()));
      }
      if (hasRight()) {
        hash = (37 * hash) + RIGHT_FIELD_NUMBER;
        hash = (53 * hash) + getRight().hashCode();
      }
      if (hasMultiplier()) {
        hash = (37 * hash) + MULTIPLIER_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            java.lang.Double.doubleToLongBits(getMultiplier()));
      }
      if (hasExchange()) {
        hash = (37 * hash) + EXCHANGE_FIELD_NUMBER;
        hash = (53 * hash) + getExchange().hashCode();
      }
      if (hasPrimaryExch()) {
        hash = (37 * hash) + PRIMARYEXCH_FIELD_NUMBER;
        hash = (53 * hash) + getPrimaryExch().hashCode();
      }
      if (hasCurrency()) {
        hash = (37 * hash) + CURRENCY_FIELD_NUMBER;
        hash = (53 * hash) + getCurrency().hashCode();
      }
      if (hasLocalSymbol()) {
        hash = (37 * hash) + LOCALSYMBOL_FIELD_NUMBER;
        hash = (53 * hash) + getLocalSymbol().hashCode();
      }
      if (hasTradingClass()) {
        hash = (37 * hash) + TRADINGCLASS_FIELD_NUMBER;
        hash = (53 * hash) + getTradingClass().hashCode();
      }
      if (hasSecIdType()) {
        hash = (37 * hash) + SECIDTYPE_FIELD_NUMBER;
        hash = (53 * hash) + getSecIdType().hashCode();
      }
      if (hasSecId()) {
        hash = (37 * hash) + SECID_FIELD_NUMBER;
        hash = (53 * hash) + getSecId().hashCode();
      }
      if (hasDescription()) {
        hash = (37 * hash) + DESCRIPTION_FIELD_NUMBER;
        hash = (53 * hash) + getDescription().hashCode();
      }
      if (hasIssuerId()) {
        hash = (37 * hash) + ISSUERID_FIELD_NUMBER;
        hash = (53 * hash) + getIssuerId().hashCode();
      }
      if (hasDeltaNeutralContract()) {
        hash = (37 * hash) + DELTANEUTRALCONTRACT_FIELD_NUMBER;
        hash = (53 * hash) + getDeltaNeutralContract().hashCode();
      }
      if (hasIncludeExpired()) {
        hash = (37 * hash) + INCLUDEEXPIRED_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashBoolean(
            getIncludeExpired());
      }
      if (hasComboLegsDescrip()) {
        hash = (37 * hash) + COMBOLEGSDESCRIP_FIELD_NUMBER;
        hash = (53 * hash) + getComboLegsDescrip().hashCode();
      }
      if (getComboLegsCount() > 0) {
        hash = (37 * hash) + COMBOLEGS_FIELD_NUMBER;
        hash = (53 * hash) + getComboLegsList().hashCode();
      }
      hash = (29 * hash) + getUnknownFields().hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static com.ib.client.protobuf.ContractProto.Contract parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ContractProto.Contract parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ContractProto.Contract parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ContractProto.Contract parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ContractProto.Contract parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ContractProto.Contract parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ContractProto.Contract parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.ContractProto.Contract parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    public static com.ib.client.protobuf.ContractProto.Contract parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input);
    }

    public static com.ib.client.protobuf.ContractProto.Contract parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static com.ib.client.protobuf.ContractProto.Contract parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.ContractProto.Contract parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(com.ib.client.protobuf.ContractProto.Contract prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessage.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code protobuf.Contract}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessage.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:protobuf.Contract)
        com.ib.client.protobuf.ContractProto.ContractOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return com.ib.client.protobuf.ContractProto.internal_static_protobuf_Contract_descriptor;
      }

      @java.lang.Override
      protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return com.ib.client.protobuf.ContractProto.internal_static_protobuf_Contract_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                com.ib.client.protobuf.ContractProto.Contract.class, com.ib.client.protobuf.ContractProto.Contract.Builder.class);
      }

      // Construct using com.ib.client.protobuf.ContractProto.Contract.newBuilder()
      private Builder() {
        maybeForceBuilderInitialization();
      }

      private Builder(
          com.google.protobuf.GeneratedMessage.BuilderParent parent) {
        super(parent);
        maybeForceBuilderInitialization();
      }
      private void maybeForceBuilderInitialization() {
        if (com.google.protobuf.GeneratedMessage
                .alwaysUseFieldBuilders) {
          getDeltaNeutralContractFieldBuilder();
          getComboLegsFieldBuilder();
        }
      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        bitField0_ = 0;
        conId_ = 0;
        symbol_ = "";
        secType_ = "";
        lastTradeDateOrContractMonth_ = "";
        strike_ = 0D;
        right_ = "";
        multiplier_ = 0D;
        exchange_ = "";
        primaryExch_ = "";
        currency_ = "";
        localSymbol_ = "";
        tradingClass_ = "";
        secIdType_ = "";
        secId_ = "";
        description_ = "";
        issuerId_ = "";
        deltaNeutralContract_ = null;
        if (deltaNeutralContractBuilder_ != null) {
          deltaNeutralContractBuilder_.dispose();
          deltaNeutralContractBuilder_ = null;
        }
        includeExpired_ = false;
        comboLegsDescrip_ = "";
        if (comboLegsBuilder_ == null) {
          comboLegs_ = java.util.Collections.emptyList();
        } else {
          comboLegs_ = null;
          comboLegsBuilder_.clear();
        }
        bitField0_ = (bitField0_ & ~0x00080000);
        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return com.ib.client.protobuf.ContractProto.internal_static_protobuf_Contract_descriptor;
      }

      @java.lang.Override
      public com.ib.client.protobuf.ContractProto.Contract getDefaultInstanceForType() {
        return com.ib.client.protobuf.ContractProto.Contract.getDefaultInstance();
      }

      @java.lang.Override
      public com.ib.client.protobuf.ContractProto.Contract build() {
        com.ib.client.protobuf.ContractProto.Contract result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public com.ib.client.protobuf.ContractProto.Contract buildPartial() {
        com.ib.client.protobuf.ContractProto.Contract result = new com.ib.client.protobuf.ContractProto.Contract(this);
        buildPartialRepeatedFields(result);
        if (bitField0_ != 0) { buildPartial0(result); }
        onBuilt();
        return result;
      }

      private void buildPartialRepeatedFields(com.ib.client.protobuf.ContractProto.Contract result) {
        if (comboLegsBuilder_ == null) {
          if (((bitField0_ & 0x00080000) != 0)) {
            comboLegs_ = java.util.Collections.unmodifiableList(comboLegs_);
            bitField0_ = (bitField0_ & ~0x00080000);
          }
          result.comboLegs_ = comboLegs_;
        } else {
          result.comboLegs_ = comboLegsBuilder_.build();
        }
      }

      private void buildPartial0(com.ib.client.protobuf.ContractProto.Contract result) {
        int from_bitField0_ = bitField0_;
        int to_bitField0_ = 0;
        if (((from_bitField0_ & 0x00000001) != 0)) {
          result.conId_ = conId_;
          to_bitField0_ |= 0x00000001;
        }
        if (((from_bitField0_ & 0x00000002) != 0)) {
          result.symbol_ = symbol_;
          to_bitField0_ |= 0x00000002;
        }
        if (((from_bitField0_ & 0x00000004) != 0)) {
          result.secType_ = secType_;
          to_bitField0_ |= 0x00000004;
        }
        if (((from_bitField0_ & 0x00000008) != 0)) {
          result.lastTradeDateOrContractMonth_ = lastTradeDateOrContractMonth_;
          to_bitField0_ |= 0x00000008;
        }
        if (((from_bitField0_ & 0x00000010) != 0)) {
          result.strike_ = strike_;
          to_bitField0_ |= 0x00000010;
        }
        if (((from_bitField0_ & 0x00000020) != 0)) {
          result.right_ = right_;
          to_bitField0_ |= 0x00000020;
        }
        if (((from_bitField0_ & 0x00000040) != 0)) {
          result.multiplier_ = multiplier_;
          to_bitField0_ |= 0x00000040;
        }
        if (((from_bitField0_ & 0x00000080) != 0)) {
          result.exchange_ = exchange_;
          to_bitField0_ |= 0x00000080;
        }
        if (((from_bitField0_ & 0x00000100) != 0)) {
          result.primaryExch_ = primaryExch_;
          to_bitField0_ |= 0x00000100;
        }
        if (((from_bitField0_ & 0x00000200) != 0)) {
          result.currency_ = currency_;
          to_bitField0_ |= 0x00000200;
        }
        if (((from_bitField0_ & 0x00000400) != 0)) {
          result.localSymbol_ = localSymbol_;
          to_bitField0_ |= 0x00000400;
        }
        if (((from_bitField0_ & 0x00000800) != 0)) {
          result.tradingClass_ = tradingClass_;
          to_bitField0_ |= 0x00000800;
        }
        if (((from_bitField0_ & 0x00001000) != 0)) {
          result.secIdType_ = secIdType_;
          to_bitField0_ |= 0x00001000;
        }
        if (((from_bitField0_ & 0x00002000) != 0)) {
          result.secId_ = secId_;
          to_bitField0_ |= 0x00002000;
        }
        if (((from_bitField0_ & 0x00004000) != 0)) {
          result.description_ = description_;
          to_bitField0_ |= 0x00004000;
        }
        if (((from_bitField0_ & 0x00008000) != 0)) {
          result.issuerId_ = issuerId_;
          to_bitField0_ |= 0x00008000;
        }
        if (((from_bitField0_ & 0x00010000) != 0)) {
          result.deltaNeutralContract_ = deltaNeutralContractBuilder_ == null
              ? deltaNeutralContract_
              : deltaNeutralContractBuilder_.build();
          to_bitField0_ |= 0x00010000;
        }
        if (((from_bitField0_ & 0x00020000) != 0)) {
          result.includeExpired_ = includeExpired_;
          to_bitField0_ |= 0x00020000;
        }
        if (((from_bitField0_ & 0x00040000) != 0)) {
          result.comboLegsDescrip_ = comboLegsDescrip_;
          to_bitField0_ |= 0x00040000;
        }
        result.bitField0_ |= to_bitField0_;
      }

      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof com.ib.client.protobuf.ContractProto.Contract) {
          return mergeFrom((com.ib.client.protobuf.ContractProto.Contract)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(com.ib.client.protobuf.ContractProto.Contract other) {
        if (other == com.ib.client.protobuf.ContractProto.Contract.getDefaultInstance()) return this;
        if (other.hasConId()) {
          setConId(other.getConId());
        }
        if (other.hasSymbol()) {
          symbol_ = other.symbol_;
          bitField0_ |= 0x00000002;
          onChanged();
        }
        if (other.hasSecType()) {
          secType_ = other.secType_;
          bitField0_ |= 0x00000004;
          onChanged();
        }
        if (other.hasLastTradeDateOrContractMonth()) {
          lastTradeDateOrContractMonth_ = other.lastTradeDateOrContractMonth_;
          bitField0_ |= 0x00000008;
          onChanged();
        }
        if (other.hasStrike()) {
          setStrike(other.getStrike());
        }
        if (other.hasRight()) {
          right_ = other.right_;
          bitField0_ |= 0x00000020;
          onChanged();
        }
        if (other.hasMultiplier()) {
          setMultiplier(other.getMultiplier());
        }
        if (other.hasExchange()) {
          exchange_ = other.exchange_;
          bitField0_ |= 0x00000080;
          onChanged();
        }
        if (other.hasPrimaryExch()) {
          primaryExch_ = other.primaryExch_;
          bitField0_ |= 0x00000100;
          onChanged();
        }
        if (other.hasCurrency()) {
          currency_ = other.currency_;
          bitField0_ |= 0x00000200;
          onChanged();
        }
        if (other.hasLocalSymbol()) {
          localSymbol_ = other.localSymbol_;
          bitField0_ |= 0x00000400;
          onChanged();
        }
        if (other.hasTradingClass()) {
          tradingClass_ = other.tradingClass_;
          bitField0_ |= 0x00000800;
          onChanged();
        }
        if (other.hasSecIdType()) {
          secIdType_ = other.secIdType_;
          bitField0_ |= 0x00001000;
          onChanged();
        }
        if (other.hasSecId()) {
          secId_ = other.secId_;
          bitField0_ |= 0x00002000;
          onChanged();
        }
        if (other.hasDescription()) {
          description_ = other.description_;
          bitField0_ |= 0x00004000;
          onChanged();
        }
        if (other.hasIssuerId()) {
          issuerId_ = other.issuerId_;
          bitField0_ |= 0x00008000;
          onChanged();
        }
        if (other.hasDeltaNeutralContract()) {
          mergeDeltaNeutralContract(other.getDeltaNeutralContract());
        }
        if (other.hasIncludeExpired()) {
          setIncludeExpired(other.getIncludeExpired());
        }
        if (other.hasComboLegsDescrip()) {
          comboLegsDescrip_ = other.comboLegsDescrip_;
          bitField0_ |= 0x00040000;
          onChanged();
        }
        if (comboLegsBuilder_ == null) {
          if (!other.comboLegs_.isEmpty()) {
            if (comboLegs_.isEmpty()) {
              comboLegs_ = other.comboLegs_;
              bitField0_ = (bitField0_ & ~0x00080000);
            } else {
              ensureComboLegsIsMutable();
              comboLegs_.addAll(other.comboLegs_);
            }
            onChanged();
          }
        } else {
          if (!other.comboLegs_.isEmpty()) {
            if (comboLegsBuilder_.isEmpty()) {
              comboLegsBuilder_.dispose();
              comboLegsBuilder_ = null;
              comboLegs_ = other.comboLegs_;
              bitField0_ = (bitField0_ & ~0x00080000);
              comboLegsBuilder_ = 
                com.google.protobuf.GeneratedMessage.alwaysUseFieldBuilders ?
                   getComboLegsFieldBuilder() : null;
            } else {
              comboLegsBuilder_.addAllMessages(other.comboLegs_);
            }
          }
        }
        this.mergeUnknownFields(other.getUnknownFields());
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        if (extensionRegistry == null) {
          throw new java.lang.NullPointerException();
        }
        try {
          boolean done = false;
          while (!done) {
            int tag = input.readTag();
            switch (tag) {
              case 0:
                done = true;
                break;
              case 8: {
                conId_ = input.readInt32();
                bitField0_ |= 0x00000001;
                break;
              } // case 8
              case 18: {
                symbol_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000002;
                break;
              } // case 18
              case 26: {
                secType_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000004;
                break;
              } // case 26
              case 34: {
                lastTradeDateOrContractMonth_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000008;
                break;
              } // case 34
              case 41: {
                strike_ = input.readDouble();
                bitField0_ |= 0x00000010;
                break;
              } // case 41
              case 50: {
                right_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000020;
                break;
              } // case 50
              case 57: {
                multiplier_ = input.readDouble();
                bitField0_ |= 0x00000040;
                break;
              } // case 57
              case 66: {
                exchange_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000080;
                break;
              } // case 66
              case 74: {
                primaryExch_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000100;
                break;
              } // case 74
              case 82: {
                currency_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000200;
                break;
              } // case 82
              case 90: {
                localSymbol_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000400;
                break;
              } // case 90
              case 98: {
                tradingClass_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000800;
                break;
              } // case 98
              case 106: {
                secIdType_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00001000;
                break;
              } // case 106
              case 114: {
                secId_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00002000;
                break;
              } // case 114
              case 122: {
                description_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00004000;
                break;
              } // case 122
              case 130: {
                issuerId_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00008000;
                break;
              } // case 130
              case 138: {
                input.readMessage(
                    getDeltaNeutralContractFieldBuilder().getBuilder(),
                    extensionRegistry);
                bitField0_ |= 0x00010000;
                break;
              } // case 138
              case 144: {
                includeExpired_ = input.readBool();
                bitField0_ |= 0x00020000;
                break;
              } // case 144
              case 154: {
                comboLegsDescrip_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00040000;
                break;
              } // case 154
              case 162: {
                com.ib.client.protobuf.ComboLegProto.ComboLeg m =
                    input.readMessage(
                        com.ib.client.protobuf.ComboLegProto.ComboLeg.parser(),
                        extensionRegistry);
                if (comboLegsBuilder_ == null) {
                  ensureComboLegsIsMutable();
                  comboLegs_.add(m);
                } else {
                  comboLegsBuilder_.addMessage(m);
                }
                break;
              } // case 162
              default: {
                if (!super.parseUnknownField(input, extensionRegistry, tag)) {
                  done = true; // was an endgroup tag
                }
                break;
              } // default:
            } // switch (tag)
          } // while (!done)
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.unwrapIOException();
        } finally {
          onChanged();
        } // finally
        return this;
      }
      private int bitField0_;

      private int conId_ ;
      /**
       * <code>optional int32 conId = 1;</code>
       * @return Whether the conId field is set.
       */
      @java.lang.Override
      public boolean hasConId() {
        return ((bitField0_ & 0x00000001) != 0);
      }
      /**
       * <code>optional int32 conId = 1;</code>
       * @return The conId.
       */
      @java.lang.Override
      public int getConId() {
        return conId_;
      }
      /**
       * <code>optional int32 conId = 1;</code>
       * @param value The conId to set.
       * @return This builder for chaining.
       */
      public Builder setConId(int value) {

        conId_ = value;
        bitField0_ |= 0x00000001;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 conId = 1;</code>
       * @return This builder for chaining.
       */
      public Builder clearConId() {
        bitField0_ = (bitField0_ & ~0x00000001);
        conId_ = 0;
        onChanged();
        return this;
      }

      private java.lang.Object symbol_ = "";
      /**
       * <code>optional string symbol = 2;</code>
       * @return Whether the symbol field is set.
       */
      public boolean hasSymbol() {
        return ((bitField0_ & 0x00000002) != 0);
      }
      /**
       * <code>optional string symbol = 2;</code>
       * @return The symbol.
       */
      public java.lang.String getSymbol() {
        java.lang.Object ref = symbol_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          symbol_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string symbol = 2;</code>
       * @return The bytes for symbol.
       */
      public com.google.protobuf.ByteString
          getSymbolBytes() {
        java.lang.Object ref = symbol_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          symbol_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string symbol = 2;</code>
       * @param value The symbol to set.
       * @return This builder for chaining.
       */
      public Builder setSymbol(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        symbol_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }
      /**
       * <code>optional string symbol = 2;</code>
       * @return This builder for chaining.
       */
      public Builder clearSymbol() {
        symbol_ = getDefaultInstance().getSymbol();
        bitField0_ = (bitField0_ & ~0x00000002);
        onChanged();
        return this;
      }
      /**
       * <code>optional string symbol = 2;</code>
       * @param value The bytes for symbol to set.
       * @return This builder for chaining.
       */
      public Builder setSymbolBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        symbol_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }

      private java.lang.Object secType_ = "";
      /**
       * <code>optional string secType = 3;</code>
       * @return Whether the secType field is set.
       */
      public boolean hasSecType() {
        return ((bitField0_ & 0x00000004) != 0);
      }
      /**
       * <code>optional string secType = 3;</code>
       * @return The secType.
       */
      public java.lang.String getSecType() {
        java.lang.Object ref = secType_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          secType_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string secType = 3;</code>
       * @return The bytes for secType.
       */
      public com.google.protobuf.ByteString
          getSecTypeBytes() {
        java.lang.Object ref = secType_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          secType_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string secType = 3;</code>
       * @param value The secType to set.
       * @return This builder for chaining.
       */
      public Builder setSecType(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        secType_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }
      /**
       * <code>optional string secType = 3;</code>
       * @return This builder for chaining.
       */
      public Builder clearSecType() {
        secType_ = getDefaultInstance().getSecType();
        bitField0_ = (bitField0_ & ~0x00000004);
        onChanged();
        return this;
      }
      /**
       * <code>optional string secType = 3;</code>
       * @param value The bytes for secType to set.
       * @return This builder for chaining.
       */
      public Builder setSecTypeBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        secType_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }

      private java.lang.Object lastTradeDateOrContractMonth_ = "";
      /**
       * <code>optional string lastTradeDateOrContractMonth = 4;</code>
       * @return Whether the lastTradeDateOrContractMonth field is set.
       */
      public boolean hasLastTradeDateOrContractMonth() {
        return ((bitField0_ & 0x00000008) != 0);
      }
      /**
       * <code>optional string lastTradeDateOrContractMonth = 4;</code>
       * @return The lastTradeDateOrContractMonth.
       */
      public java.lang.String getLastTradeDateOrContractMonth() {
        java.lang.Object ref = lastTradeDateOrContractMonth_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          lastTradeDateOrContractMonth_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string lastTradeDateOrContractMonth = 4;</code>
       * @return The bytes for lastTradeDateOrContractMonth.
       */
      public com.google.protobuf.ByteString
          getLastTradeDateOrContractMonthBytes() {
        java.lang.Object ref = lastTradeDateOrContractMonth_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          lastTradeDateOrContractMonth_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string lastTradeDateOrContractMonth = 4;</code>
       * @param value The lastTradeDateOrContractMonth to set.
       * @return This builder for chaining.
       */
      public Builder setLastTradeDateOrContractMonth(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        lastTradeDateOrContractMonth_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }
      /**
       * <code>optional string lastTradeDateOrContractMonth = 4;</code>
       * @return This builder for chaining.
       */
      public Builder clearLastTradeDateOrContractMonth() {
        lastTradeDateOrContractMonth_ = getDefaultInstance().getLastTradeDateOrContractMonth();
        bitField0_ = (bitField0_ & ~0x00000008);
        onChanged();
        return this;
      }
      /**
       * <code>optional string lastTradeDateOrContractMonth = 4;</code>
       * @param value The bytes for lastTradeDateOrContractMonth to set.
       * @return This builder for chaining.
       */
      public Builder setLastTradeDateOrContractMonthBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        lastTradeDateOrContractMonth_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }

      private double strike_ ;
      /**
       * <code>optional double strike = 5;</code>
       * @return Whether the strike field is set.
       */
      @java.lang.Override
      public boolean hasStrike() {
        return ((bitField0_ & 0x00000010) != 0);
      }
      /**
       * <code>optional double strike = 5;</code>
       * @return The strike.
       */
      @java.lang.Override
      public double getStrike() {
        return strike_;
      }
      /**
       * <code>optional double strike = 5;</code>
       * @param value The strike to set.
       * @return This builder for chaining.
       */
      public Builder setStrike(double value) {

        strike_ = value;
        bitField0_ |= 0x00000010;
        onChanged();
        return this;
      }
      /**
       * <code>optional double strike = 5;</code>
       * @return This builder for chaining.
       */
      public Builder clearStrike() {
        bitField0_ = (bitField0_ & ~0x00000010);
        strike_ = 0D;
        onChanged();
        return this;
      }

      private java.lang.Object right_ = "";
      /**
       * <code>optional string right = 6;</code>
       * @return Whether the right field is set.
       */
      public boolean hasRight() {
        return ((bitField0_ & 0x00000020) != 0);
      }
      /**
       * <code>optional string right = 6;</code>
       * @return The right.
       */
      public java.lang.String getRight() {
        java.lang.Object ref = right_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          right_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string right = 6;</code>
       * @return The bytes for right.
       */
      public com.google.protobuf.ByteString
          getRightBytes() {
        java.lang.Object ref = right_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          right_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string right = 6;</code>
       * @param value The right to set.
       * @return This builder for chaining.
       */
      public Builder setRight(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        right_ = value;
        bitField0_ |= 0x00000020;
        onChanged();
        return this;
      }
      /**
       * <code>optional string right = 6;</code>
       * @return This builder for chaining.
       */
      public Builder clearRight() {
        right_ = getDefaultInstance().getRight();
        bitField0_ = (bitField0_ & ~0x00000020);
        onChanged();
        return this;
      }
      /**
       * <code>optional string right = 6;</code>
       * @param value The bytes for right to set.
       * @return This builder for chaining.
       */
      public Builder setRightBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        right_ = value;
        bitField0_ |= 0x00000020;
        onChanged();
        return this;
      }

      private double multiplier_ ;
      /**
       * <code>optional double multiplier = 7;</code>
       * @return Whether the multiplier field is set.
       */
      @java.lang.Override
      public boolean hasMultiplier() {
        return ((bitField0_ & 0x00000040) != 0);
      }
      /**
       * <code>optional double multiplier = 7;</code>
       * @return The multiplier.
       */
      @java.lang.Override
      public double getMultiplier() {
        return multiplier_;
      }
      /**
       * <code>optional double multiplier = 7;</code>
       * @param value The multiplier to set.
       * @return This builder for chaining.
       */
      public Builder setMultiplier(double value) {

        multiplier_ = value;
        bitField0_ |= 0x00000040;
        onChanged();
        return this;
      }
      /**
       * <code>optional double multiplier = 7;</code>
       * @return This builder for chaining.
       */
      public Builder clearMultiplier() {
        bitField0_ = (bitField0_ & ~0x00000040);
        multiplier_ = 0D;
        onChanged();
        return this;
      }

      private java.lang.Object exchange_ = "";
      /**
       * <code>optional string exchange = 8;</code>
       * @return Whether the exchange field is set.
       */
      public boolean hasExchange() {
        return ((bitField0_ & 0x00000080) != 0);
      }
      /**
       * <code>optional string exchange = 8;</code>
       * @return The exchange.
       */
      public java.lang.String getExchange() {
        java.lang.Object ref = exchange_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          exchange_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string exchange = 8;</code>
       * @return The bytes for exchange.
       */
      public com.google.protobuf.ByteString
          getExchangeBytes() {
        java.lang.Object ref = exchange_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          exchange_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string exchange = 8;</code>
       * @param value The exchange to set.
       * @return This builder for chaining.
       */
      public Builder setExchange(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        exchange_ = value;
        bitField0_ |= 0x00000080;
        onChanged();
        return this;
      }
      /**
       * <code>optional string exchange = 8;</code>
       * @return This builder for chaining.
       */
      public Builder clearExchange() {
        exchange_ = getDefaultInstance().getExchange();
        bitField0_ = (bitField0_ & ~0x00000080);
        onChanged();
        return this;
      }
      /**
       * <code>optional string exchange = 8;</code>
       * @param value The bytes for exchange to set.
       * @return This builder for chaining.
       */
      public Builder setExchangeBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        exchange_ = value;
        bitField0_ |= 0x00000080;
        onChanged();
        return this;
      }

      private java.lang.Object primaryExch_ = "";
      /**
       * <code>optional string primaryExch = 9;</code>
       * @return Whether the primaryExch field is set.
       */
      public boolean hasPrimaryExch() {
        return ((bitField0_ & 0x00000100) != 0);
      }
      /**
       * <code>optional string primaryExch = 9;</code>
       * @return The primaryExch.
       */
      public java.lang.String getPrimaryExch() {
        java.lang.Object ref = primaryExch_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          primaryExch_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string primaryExch = 9;</code>
       * @return The bytes for primaryExch.
       */
      public com.google.protobuf.ByteString
          getPrimaryExchBytes() {
        java.lang.Object ref = primaryExch_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          primaryExch_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string primaryExch = 9;</code>
       * @param value The primaryExch to set.
       * @return This builder for chaining.
       */
      public Builder setPrimaryExch(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        primaryExch_ = value;
        bitField0_ |= 0x00000100;
        onChanged();
        return this;
      }
      /**
       * <code>optional string primaryExch = 9;</code>
       * @return This builder for chaining.
       */
      public Builder clearPrimaryExch() {
        primaryExch_ = getDefaultInstance().getPrimaryExch();
        bitField0_ = (bitField0_ & ~0x00000100);
        onChanged();
        return this;
      }
      /**
       * <code>optional string primaryExch = 9;</code>
       * @param value The bytes for primaryExch to set.
       * @return This builder for chaining.
       */
      public Builder setPrimaryExchBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        primaryExch_ = value;
        bitField0_ |= 0x00000100;
        onChanged();
        return this;
      }

      private java.lang.Object currency_ = "";
      /**
       * <code>optional string currency = 10;</code>
       * @return Whether the currency field is set.
       */
      public boolean hasCurrency() {
        return ((bitField0_ & 0x00000200) != 0);
      }
      /**
       * <code>optional string currency = 10;</code>
       * @return The currency.
       */
      public java.lang.String getCurrency() {
        java.lang.Object ref = currency_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          currency_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string currency = 10;</code>
       * @return The bytes for currency.
       */
      public com.google.protobuf.ByteString
          getCurrencyBytes() {
        java.lang.Object ref = currency_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          currency_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string currency = 10;</code>
       * @param value The currency to set.
       * @return This builder for chaining.
       */
      public Builder setCurrency(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        currency_ = value;
        bitField0_ |= 0x00000200;
        onChanged();
        return this;
      }
      /**
       * <code>optional string currency = 10;</code>
       * @return This builder for chaining.
       */
      public Builder clearCurrency() {
        currency_ = getDefaultInstance().getCurrency();
        bitField0_ = (bitField0_ & ~0x00000200);
        onChanged();
        return this;
      }
      /**
       * <code>optional string currency = 10;</code>
       * @param value The bytes for currency to set.
       * @return This builder for chaining.
       */
      public Builder setCurrencyBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        currency_ = value;
        bitField0_ |= 0x00000200;
        onChanged();
        return this;
      }

      private java.lang.Object localSymbol_ = "";
      /**
       * <code>optional string localSymbol = 11;</code>
       * @return Whether the localSymbol field is set.
       */
      public boolean hasLocalSymbol() {
        return ((bitField0_ & 0x00000400) != 0);
      }
      /**
       * <code>optional string localSymbol = 11;</code>
       * @return The localSymbol.
       */
      public java.lang.String getLocalSymbol() {
        java.lang.Object ref = localSymbol_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          localSymbol_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string localSymbol = 11;</code>
       * @return The bytes for localSymbol.
       */
      public com.google.protobuf.ByteString
          getLocalSymbolBytes() {
        java.lang.Object ref = localSymbol_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          localSymbol_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string localSymbol = 11;</code>
       * @param value The localSymbol to set.
       * @return This builder for chaining.
       */
      public Builder setLocalSymbol(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        localSymbol_ = value;
        bitField0_ |= 0x00000400;
        onChanged();
        return this;
      }
      /**
       * <code>optional string localSymbol = 11;</code>
       * @return This builder for chaining.
       */
      public Builder clearLocalSymbol() {
        localSymbol_ = getDefaultInstance().getLocalSymbol();
        bitField0_ = (bitField0_ & ~0x00000400);
        onChanged();
        return this;
      }
      /**
       * <code>optional string localSymbol = 11;</code>
       * @param value The bytes for localSymbol to set.
       * @return This builder for chaining.
       */
      public Builder setLocalSymbolBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        localSymbol_ = value;
        bitField0_ |= 0x00000400;
        onChanged();
        return this;
      }

      private java.lang.Object tradingClass_ = "";
      /**
       * <code>optional string tradingClass = 12;</code>
       * @return Whether the tradingClass field is set.
       */
      public boolean hasTradingClass() {
        return ((bitField0_ & 0x00000800) != 0);
      }
      /**
       * <code>optional string tradingClass = 12;</code>
       * @return The tradingClass.
       */
      public java.lang.String getTradingClass() {
        java.lang.Object ref = tradingClass_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          tradingClass_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string tradingClass = 12;</code>
       * @return The bytes for tradingClass.
       */
      public com.google.protobuf.ByteString
          getTradingClassBytes() {
        java.lang.Object ref = tradingClass_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          tradingClass_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string tradingClass = 12;</code>
       * @param value The tradingClass to set.
       * @return This builder for chaining.
       */
      public Builder setTradingClass(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        tradingClass_ = value;
        bitField0_ |= 0x00000800;
        onChanged();
        return this;
      }
      /**
       * <code>optional string tradingClass = 12;</code>
       * @return This builder for chaining.
       */
      public Builder clearTradingClass() {
        tradingClass_ = getDefaultInstance().getTradingClass();
        bitField0_ = (bitField0_ & ~0x00000800);
        onChanged();
        return this;
      }
      /**
       * <code>optional string tradingClass = 12;</code>
       * @param value The bytes for tradingClass to set.
       * @return This builder for chaining.
       */
      public Builder setTradingClassBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        tradingClass_ = value;
        bitField0_ |= 0x00000800;
        onChanged();
        return this;
      }

      private java.lang.Object secIdType_ = "";
      /**
       * <code>optional string secIdType = 13;</code>
       * @return Whether the secIdType field is set.
       */
      public boolean hasSecIdType() {
        return ((bitField0_ & 0x00001000) != 0);
      }
      /**
       * <code>optional string secIdType = 13;</code>
       * @return The secIdType.
       */
      public java.lang.String getSecIdType() {
        java.lang.Object ref = secIdType_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          secIdType_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string secIdType = 13;</code>
       * @return The bytes for secIdType.
       */
      public com.google.protobuf.ByteString
          getSecIdTypeBytes() {
        java.lang.Object ref = secIdType_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          secIdType_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string secIdType = 13;</code>
       * @param value The secIdType to set.
       * @return This builder for chaining.
       */
      public Builder setSecIdType(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        secIdType_ = value;
        bitField0_ |= 0x00001000;
        onChanged();
        return this;
      }
      /**
       * <code>optional string secIdType = 13;</code>
       * @return This builder for chaining.
       */
      public Builder clearSecIdType() {
        secIdType_ = getDefaultInstance().getSecIdType();
        bitField0_ = (bitField0_ & ~0x00001000);
        onChanged();
        return this;
      }
      /**
       * <code>optional string secIdType = 13;</code>
       * @param value The bytes for secIdType to set.
       * @return This builder for chaining.
       */
      public Builder setSecIdTypeBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        secIdType_ = value;
        bitField0_ |= 0x00001000;
        onChanged();
        return this;
      }

      private java.lang.Object secId_ = "";
      /**
       * <code>optional string secId = 14;</code>
       * @return Whether the secId field is set.
       */
      public boolean hasSecId() {
        return ((bitField0_ & 0x00002000) != 0);
      }
      /**
       * <code>optional string secId = 14;</code>
       * @return The secId.
       */
      public java.lang.String getSecId() {
        java.lang.Object ref = secId_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          secId_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string secId = 14;</code>
       * @return The bytes for secId.
       */
      public com.google.protobuf.ByteString
          getSecIdBytes() {
        java.lang.Object ref = secId_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          secId_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string secId = 14;</code>
       * @param value The secId to set.
       * @return This builder for chaining.
       */
      public Builder setSecId(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        secId_ = value;
        bitField0_ |= 0x00002000;
        onChanged();
        return this;
      }
      /**
       * <code>optional string secId = 14;</code>
       * @return This builder for chaining.
       */
      public Builder clearSecId() {
        secId_ = getDefaultInstance().getSecId();
        bitField0_ = (bitField0_ & ~0x00002000);
        onChanged();
        return this;
      }
      /**
       * <code>optional string secId = 14;</code>
       * @param value The bytes for secId to set.
       * @return This builder for chaining.
       */
      public Builder setSecIdBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        secId_ = value;
        bitField0_ |= 0x00002000;
        onChanged();
        return this;
      }

      private java.lang.Object description_ = "";
      /**
       * <code>optional string description = 15;</code>
       * @return Whether the description field is set.
       */
      public boolean hasDescription() {
        return ((bitField0_ & 0x00004000) != 0);
      }
      /**
       * <code>optional string description = 15;</code>
       * @return The description.
       */
      public java.lang.String getDescription() {
        java.lang.Object ref = description_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          description_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string description = 15;</code>
       * @return The bytes for description.
       */
      public com.google.protobuf.ByteString
          getDescriptionBytes() {
        java.lang.Object ref = description_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          description_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string description = 15;</code>
       * @param value The description to set.
       * @return This builder for chaining.
       */
      public Builder setDescription(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        description_ = value;
        bitField0_ |= 0x00004000;
        onChanged();
        return this;
      }
      /**
       * <code>optional string description = 15;</code>
       * @return This builder for chaining.
       */
      public Builder clearDescription() {
        description_ = getDefaultInstance().getDescription();
        bitField0_ = (bitField0_ & ~0x00004000);
        onChanged();
        return this;
      }
      /**
       * <code>optional string description = 15;</code>
       * @param value The bytes for description to set.
       * @return This builder for chaining.
       */
      public Builder setDescriptionBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        description_ = value;
        bitField0_ |= 0x00004000;
        onChanged();
        return this;
      }

      private java.lang.Object issuerId_ = "";
      /**
       * <code>optional string issuerId = 16;</code>
       * @return Whether the issuerId field is set.
       */
      public boolean hasIssuerId() {
        return ((bitField0_ & 0x00008000) != 0);
      }
      /**
       * <code>optional string issuerId = 16;</code>
       * @return The issuerId.
       */
      public java.lang.String getIssuerId() {
        java.lang.Object ref = issuerId_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          issuerId_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string issuerId = 16;</code>
       * @return The bytes for issuerId.
       */
      public com.google.protobuf.ByteString
          getIssuerIdBytes() {
        java.lang.Object ref = issuerId_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          issuerId_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string issuerId = 16;</code>
       * @param value The issuerId to set.
       * @return This builder for chaining.
       */
      public Builder setIssuerId(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        issuerId_ = value;
        bitField0_ |= 0x00008000;
        onChanged();
        return this;
      }
      /**
       * <code>optional string issuerId = 16;</code>
       * @return This builder for chaining.
       */
      public Builder clearIssuerId() {
        issuerId_ = getDefaultInstance().getIssuerId();
        bitField0_ = (bitField0_ & ~0x00008000);
        onChanged();
        return this;
      }
      /**
       * <code>optional string issuerId = 16;</code>
       * @param value The bytes for issuerId to set.
       * @return This builder for chaining.
       */
      public Builder setIssuerIdBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        issuerId_ = value;
        bitField0_ |= 0x00008000;
        onChanged();
        return this;
      }

      private com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract deltaNeutralContract_;
      private com.google.protobuf.SingleFieldBuilder<
          com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract, com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.Builder, com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContractOrBuilder> deltaNeutralContractBuilder_;
      /**
       * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
       * @return Whether the deltaNeutralContract field is set.
       */
      public boolean hasDeltaNeutralContract() {
        return ((bitField0_ & 0x00010000) != 0);
      }
      /**
       * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
       * @return The deltaNeutralContract.
       */
      public com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract getDeltaNeutralContract() {
        if (deltaNeutralContractBuilder_ == null) {
          return deltaNeutralContract_ == null ? com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.getDefaultInstance() : deltaNeutralContract_;
        } else {
          return deltaNeutralContractBuilder_.getMessage();
        }
      }
      /**
       * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
       */
      public Builder setDeltaNeutralContract(com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract value) {
        if (deltaNeutralContractBuilder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          deltaNeutralContract_ = value;
        } else {
          deltaNeutralContractBuilder_.setMessage(value);
        }
        bitField0_ |= 0x00010000;
        onChanged();
        return this;
      }
      /**
       * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
       */
      public Builder setDeltaNeutralContract(
          com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.Builder builderForValue) {
        if (deltaNeutralContractBuilder_ == null) {
          deltaNeutralContract_ = builderForValue.build();
        } else {
          deltaNeutralContractBuilder_.setMessage(builderForValue.build());
        }
        bitField0_ |= 0x00010000;
        onChanged();
        return this;
      }
      /**
       * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
       */
      public Builder mergeDeltaNeutralContract(com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract value) {
        if (deltaNeutralContractBuilder_ == null) {
          if (((bitField0_ & 0x00010000) != 0) &&
            deltaNeutralContract_ != null &&
            deltaNeutralContract_ != com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.getDefaultInstance()) {
            getDeltaNeutralContractBuilder().mergeFrom(value);
          } else {
            deltaNeutralContract_ = value;
          }
        } else {
          deltaNeutralContractBuilder_.mergeFrom(value);
        }
        if (deltaNeutralContract_ != null) {
          bitField0_ |= 0x00010000;
          onChanged();
        }
        return this;
      }
      /**
       * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
       */
      public Builder clearDeltaNeutralContract() {
        bitField0_ = (bitField0_ & ~0x00010000);
        deltaNeutralContract_ = null;
        if (deltaNeutralContractBuilder_ != null) {
          deltaNeutralContractBuilder_.dispose();
          deltaNeutralContractBuilder_ = null;
        }
        onChanged();
        return this;
      }
      /**
       * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
       */
      public com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.Builder getDeltaNeutralContractBuilder() {
        bitField0_ |= 0x00010000;
        onChanged();
        return getDeltaNeutralContractFieldBuilder().getBuilder();
      }
      /**
       * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
       */
      public com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContractOrBuilder getDeltaNeutralContractOrBuilder() {
        if (deltaNeutralContractBuilder_ != null) {
          return deltaNeutralContractBuilder_.getMessageOrBuilder();
        } else {
          return deltaNeutralContract_ == null ?
              com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.getDefaultInstance() : deltaNeutralContract_;
        }
      }
      /**
       * <code>optional .protobuf.DeltaNeutralContract deltaNeutralContract = 17;</code>
       */
      private com.google.protobuf.SingleFieldBuilder<
          com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract, com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.Builder, com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContractOrBuilder> 
          getDeltaNeutralContractFieldBuilder() {
        if (deltaNeutralContractBuilder_ == null) {
          deltaNeutralContractBuilder_ = new com.google.protobuf.SingleFieldBuilder<
              com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract, com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.Builder, com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContractOrBuilder>(
                  getDeltaNeutralContract(),
                  getParentForChildren(),
                  isClean());
          deltaNeutralContract_ = null;
        }
        return deltaNeutralContractBuilder_;
      }

      private boolean includeExpired_ ;
      /**
       * <code>optional bool includeExpired = 18;</code>
       * @return Whether the includeExpired field is set.
       */
      @java.lang.Override
      public boolean hasIncludeExpired() {
        return ((bitField0_ & 0x00020000) != 0);
      }
      /**
       * <code>optional bool includeExpired = 18;</code>
       * @return The includeExpired.
       */
      @java.lang.Override
      public boolean getIncludeExpired() {
        return includeExpired_;
      }
      /**
       * <code>optional bool includeExpired = 18;</code>
       * @param value The includeExpired to set.
       * @return This builder for chaining.
       */
      public Builder setIncludeExpired(boolean value) {

        includeExpired_ = value;
        bitField0_ |= 0x00020000;
        onChanged();
        return this;
      }
      /**
       * <code>optional bool includeExpired = 18;</code>
       * @return This builder for chaining.
       */
      public Builder clearIncludeExpired() {
        bitField0_ = (bitField0_ & ~0x00020000);
        includeExpired_ = false;
        onChanged();
        return this;
      }

      private java.lang.Object comboLegsDescrip_ = "";
      /**
       * <code>optional string comboLegsDescrip = 19;</code>
       * @return Whether the comboLegsDescrip field is set.
       */
      public boolean hasComboLegsDescrip() {
        return ((bitField0_ & 0x00040000) != 0);
      }
      /**
       * <code>optional string comboLegsDescrip = 19;</code>
       * @return The comboLegsDescrip.
       */
      public java.lang.String getComboLegsDescrip() {
        java.lang.Object ref = comboLegsDescrip_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          comboLegsDescrip_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string comboLegsDescrip = 19;</code>
       * @return The bytes for comboLegsDescrip.
       */
      public com.google.protobuf.ByteString
          getComboLegsDescripBytes() {
        java.lang.Object ref = comboLegsDescrip_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          comboLegsDescrip_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string comboLegsDescrip = 19;</code>
       * @param value The comboLegsDescrip to set.
       * @return This builder for chaining.
       */
      public Builder setComboLegsDescrip(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        comboLegsDescrip_ = value;
        bitField0_ |= 0x00040000;
        onChanged();
        return this;
      }
      /**
       * <code>optional string comboLegsDescrip = 19;</code>
       * @return This builder for chaining.
       */
      public Builder clearComboLegsDescrip() {
        comboLegsDescrip_ = getDefaultInstance().getComboLegsDescrip();
        bitField0_ = (bitField0_ & ~0x00040000);
        onChanged();
        return this;
      }
      /**
       * <code>optional string comboLegsDescrip = 19;</code>
       * @param value The bytes for comboLegsDescrip to set.
       * @return This builder for chaining.
       */
      public Builder setComboLegsDescripBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        comboLegsDescrip_ = value;
        bitField0_ |= 0x00040000;
        onChanged();
        return this;
      }

      private java.util.List<com.ib.client.protobuf.ComboLegProto.ComboLeg> comboLegs_ =
        java.util.Collections.emptyList();
      private void ensureComboLegsIsMutable() {
        if (!((bitField0_ & 0x00080000) != 0)) {
          comboLegs_ = new java.util.ArrayList<com.ib.client.protobuf.ComboLegProto.ComboLeg>(comboLegs_);
          bitField0_ |= 0x00080000;
         }
      }

      private com.google.protobuf.RepeatedFieldBuilder<
          com.ib.client.protobuf.ComboLegProto.ComboLeg, com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder, com.ib.client.protobuf.ComboLegProto.ComboLegOrBuilder> comboLegsBuilder_;

      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public java.util.List<com.ib.client.protobuf.ComboLegProto.ComboLeg> getComboLegsList() {
        if (comboLegsBuilder_ == null) {
          return java.util.Collections.unmodifiableList(comboLegs_);
        } else {
          return comboLegsBuilder_.getMessageList();
        }
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public int getComboLegsCount() {
        if (comboLegsBuilder_ == null) {
          return comboLegs_.size();
        } else {
          return comboLegsBuilder_.getCount();
        }
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public com.ib.client.protobuf.ComboLegProto.ComboLeg getComboLegs(int index) {
        if (comboLegsBuilder_ == null) {
          return comboLegs_.get(index);
        } else {
          return comboLegsBuilder_.getMessage(index);
        }
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public Builder setComboLegs(
          int index, com.ib.client.protobuf.ComboLegProto.ComboLeg value) {
        if (comboLegsBuilder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureComboLegsIsMutable();
          comboLegs_.set(index, value);
          onChanged();
        } else {
          comboLegsBuilder_.setMessage(index, value);
        }
        return this;
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public Builder setComboLegs(
          int index, com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder builderForValue) {
        if (comboLegsBuilder_ == null) {
          ensureComboLegsIsMutable();
          comboLegs_.set(index, builderForValue.build());
          onChanged();
        } else {
          comboLegsBuilder_.setMessage(index, builderForValue.build());
        }
        return this;
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public Builder addComboLegs(com.ib.client.protobuf.ComboLegProto.ComboLeg value) {
        if (comboLegsBuilder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureComboLegsIsMutable();
          comboLegs_.add(value);
          onChanged();
        } else {
          comboLegsBuilder_.addMessage(value);
        }
        return this;
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public Builder addComboLegs(
          int index, com.ib.client.protobuf.ComboLegProto.ComboLeg value) {
        if (comboLegsBuilder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureComboLegsIsMutable();
          comboLegs_.add(index, value);
          onChanged();
        } else {
          comboLegsBuilder_.addMessage(index, value);
        }
        return this;
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public Builder addComboLegs(
          com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder builderForValue) {
        if (comboLegsBuilder_ == null) {
          ensureComboLegsIsMutable();
          comboLegs_.add(builderForValue.build());
          onChanged();
        } else {
          comboLegsBuilder_.addMessage(builderForValue.build());
        }
        return this;
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public Builder addComboLegs(
          int index, com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder builderForValue) {
        if (comboLegsBuilder_ == null) {
          ensureComboLegsIsMutable();
          comboLegs_.add(index, builderForValue.build());
          onChanged();
        } else {
          comboLegsBuilder_.addMessage(index, builderForValue.build());
        }
        return this;
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public Builder addAllComboLegs(
          java.lang.Iterable<? extends com.ib.client.protobuf.ComboLegProto.ComboLeg> values) {
        if (comboLegsBuilder_ == null) {
          ensureComboLegsIsMutable();
          com.google.protobuf.AbstractMessageLite.Builder.addAll(
              values, comboLegs_);
          onChanged();
        } else {
          comboLegsBuilder_.addAllMessages(values);
        }
        return this;
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public Builder clearComboLegs() {
        if (comboLegsBuilder_ == null) {
          comboLegs_ = java.util.Collections.emptyList();
          bitField0_ = (bitField0_ & ~0x00080000);
          onChanged();
        } else {
          comboLegsBuilder_.clear();
        }
        return this;
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public Builder removeComboLegs(int index) {
        if (comboLegsBuilder_ == null) {
          ensureComboLegsIsMutable();
          comboLegs_.remove(index);
          onChanged();
        } else {
          comboLegsBuilder_.remove(index);
        }
        return this;
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder getComboLegsBuilder(
          int index) {
        return getComboLegsFieldBuilder().getBuilder(index);
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public com.ib.client.protobuf.ComboLegProto.ComboLegOrBuilder getComboLegsOrBuilder(
          int index) {
        if (comboLegsBuilder_ == null) {
          return comboLegs_.get(index);  } else {
          return comboLegsBuilder_.getMessageOrBuilder(index);
        }
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public java.util.List<? extends com.ib.client.protobuf.ComboLegProto.ComboLegOrBuilder> 
           getComboLegsOrBuilderList() {
        if (comboLegsBuilder_ != null) {
          return comboLegsBuilder_.getMessageOrBuilderList();
        } else {
          return java.util.Collections.unmodifiableList(comboLegs_);
        }
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder addComboLegsBuilder() {
        return getComboLegsFieldBuilder().addBuilder(
            com.ib.client.protobuf.ComboLegProto.ComboLeg.getDefaultInstance());
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder addComboLegsBuilder(
          int index) {
        return getComboLegsFieldBuilder().addBuilder(
            index, com.ib.client.protobuf.ComboLegProto.ComboLeg.getDefaultInstance());
      }
      /**
       * <code>repeated .protobuf.ComboLeg comboLegs = 20;</code>
       */
      public java.util.List<com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder> 
           getComboLegsBuilderList() {
        return getComboLegsFieldBuilder().getBuilderList();
      }
      private com.google.protobuf.RepeatedFieldBuilder<
          com.ib.client.protobuf.ComboLegProto.ComboLeg, com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder, com.ib.client.protobuf.ComboLegProto.ComboLegOrBuilder> 
          getComboLegsFieldBuilder() {
        if (comboLegsBuilder_ == null) {
          comboLegsBuilder_ = new com.google.protobuf.RepeatedFieldBuilder<
              com.ib.client.protobuf.ComboLegProto.ComboLeg, com.ib.client.protobuf.ComboLegProto.ComboLeg.Builder, com.ib.client.protobuf.ComboLegProto.ComboLegOrBuilder>(
                  comboLegs_,
                  ((bitField0_ & 0x00080000) != 0),
                  getParentForChildren(),
                  isClean());
          comboLegs_ = null;
        }
        return comboLegsBuilder_;
      }

      // @@protoc_insertion_point(builder_scope:protobuf.Contract)
    }

    // @@protoc_insertion_point(class_scope:protobuf.Contract)
    private static final com.ib.client.protobuf.ContractProto.Contract DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new com.ib.client.protobuf.ContractProto.Contract();
    }

    public static com.ib.client.protobuf.ContractProto.Contract getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<Contract>
        PARSER = new com.google.protobuf.AbstractParser<Contract>() {
      @java.lang.Override
      public Contract parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        Builder builder = newBuilder();
        try {
          builder.mergeFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.setUnfinishedMessage(builder.buildPartial());
        } catch (com.google.protobuf.UninitializedMessageException e) {
          throw e.asInvalidProtocolBufferException().setUnfinishedMessage(builder.buildPartial());
        } catch (java.io.IOException e) {
          throw new com.google.protobuf.InvalidProtocolBufferException(e)
              .setUnfinishedMessage(builder.buildPartial());
        }
        return builder.buildPartial();
      }
    };

    public static com.google.protobuf.Parser<Contract> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<Contract> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public com.ib.client.protobuf.ContractProto.Contract getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_protobuf_Contract_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessage.FieldAccessorTable
      internal_static_protobuf_Contract_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor
      getDescriptor() {
    return descriptor;
  }
  private static  com.google.protobuf.Descriptors.FileDescriptor
      descriptor;
  static {
    java.lang.String[] descriptorData = {
      "\n\016Contract.proto\022\010protobuf\032\016ComboLeg.pro" +
      "to\032\032DeltaNeutralContract.proto\"\335\006\n\010Contr" +
      "act\022\022\n\005conId\030\001 \001(\005H\000\210\001\001\022\023\n\006symbol\030\002 \001(\tH" +
      "\001\210\001\001\022\024\n\007secType\030\003 \001(\tH\002\210\001\001\022)\n\034lastTradeD" +
      "ateOrContractMonth\030\004 \001(\tH\003\210\001\001\022\023\n\006strike\030" +
      "\005 \001(\001H\004\210\001\001\022\022\n\005right\030\006 \001(\tH\005\210\001\001\022\027\n\nmultip" +
      "lier\030\007 \001(\001H\006\210\001\001\022\025\n\010exchange\030\010 \001(\tH\007\210\001\001\022\030" +
      "\n\013primaryExch\030\t \001(\tH\010\210\001\001\022\025\n\010currency\030\n \001" +
      "(\tH\t\210\001\001\022\030\n\013localSymbol\030\013 \001(\tH\n\210\001\001\022\031\n\014tra" +
      "dingClass\030\014 \001(\tH\013\210\001\001\022\026\n\tsecIdType\030\r \001(\tH" +
      "\014\210\001\001\022\022\n\005secId\030\016 \001(\tH\r\210\001\001\022\030\n\013description\030" +
      "\017 \001(\tH\016\210\001\001\022\025\n\010issuerId\030\020 \001(\tH\017\210\001\001\022A\n\024del" +
      "taNeutralContract\030\021 \001(\0132\036.protobuf.Delta" +
      "NeutralContractH\020\210\001\001\022\033\n\016includeExpired\030\022" +
      " \001(\010H\021\210\001\001\022\035\n\020comboLegsDescrip\030\023 \001(\tH\022\210\001\001" +
      "\022%\n\tcomboLegs\030\024 \003(\0132\022.protobuf.ComboLegB" +
      "\010\n\006_conIdB\t\n\007_symbolB\n\n\010_secTypeB\037\n\035_las" +
      "tTradeDateOrContractMonthB\t\n\007_strikeB\010\n\006" +
      "_rightB\r\n\013_multiplierB\013\n\t_exchangeB\016\n\014_p" +
      "rimaryExchB\013\n\t_currencyB\016\n\014_localSymbolB" +
      "\017\n\r_tradingClassB\014\n\n_secIdTypeB\010\n\006_secId" +
      "B\016\n\014_descriptionB\013\n\t_issuerIdB\027\n\025_deltaN" +
      "eutralContractB\021\n\017_includeExpiredB\023\n\021_co" +
      "mboLegsDescripB8\n\026com.ib.client.protobuf" +
      "B\rContractProto\252\002\016IBApi.protobufb\006proto3"
    };
    descriptor = com.google.protobuf.Descriptors.FileDescriptor
      .internalBuildGeneratedFileFrom(descriptorData,
        new com.google.protobuf.Descriptors.FileDescriptor[] {
          com.ib.client.protobuf.ComboLegProto.getDescriptor(),
          com.ib.client.protobuf.DeltaNeutralContractProto.getDescriptor(),
        });
    internal_static_protobuf_Contract_descriptor =
      getDescriptor().getMessageTypes().get(0);
    internal_static_protobuf_Contract_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessage.FieldAccessorTable(
        internal_static_protobuf_Contract_descriptor,
        new java.lang.String[] { "ConId", "Symbol", "SecType", "LastTradeDateOrContractMonth", "Strike", "Right", "Multiplier", "Exchange", "PrimaryExch", "Currency", "LocalSymbol", "TradingClass", "SecIdType", "SecId", "Description", "IssuerId", "DeltaNeutralContract", "IncludeExpired", "ComboLegsDescrip", "ComboLegs", });
    descriptor.resolveAllFeaturesImmutable();
    com.ib.client.protobuf.ComboLegProto.getDescriptor();
    com.ib.client.protobuf.DeltaNeutralContractProto.getDescriptor();
  }

  // @@protoc_insertion_point(outer_class_scope)
}
