# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: OrderCondition.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'OrderCondition.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14OrderCondition.proto\x12\x08protobuf\"\xea\x03\n\x0eOrderCondition\x12\x11\n\x04type\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12$\n\x17isConjunctionConnection\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12\x13\n\x06isMore\x18\x03 \x01(\x08H\x02\x88\x01\x01\x12\x12\n\x05\x63onId\x18\x04 \x01(\x05H\x03\x88\x01\x01\x12\x15\n\x08\x65xchange\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x13\n\x06symbol\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x14\n\x07secType\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x14\n\x07percent\x18\x08 \x01(\x05H\x07\x88\x01\x01\x12\x1a\n\rchangePercent\x18\t \x01(\x01H\x08\x88\x01\x01\x12\x12\n\x05price\x18\n \x01(\x01H\t\x88\x01\x01\x12\x1a\n\rtriggerMethod\x18\x0b \x01(\x05H\n\x88\x01\x01\x12\x11\n\x04time\x18\x0c \x01(\tH\x0b\x88\x01\x01\x12\x13\n\x06volume\x18\r \x01(\x05H\x0c\x88\x01\x01\x42\x07\n\x05_typeB\x1a\n\x18_isConjunctionConnectionB\t\n\x07_isMoreB\x08\n\x06_conIdB\x0b\n\t_exchangeB\t\n\x07_symbolB\n\n\x08_secTypeB\n\n\x08_percentB\x10\n\x0e_changePercentB\x08\n\x06_priceB\x10\n\x0e_triggerMethodB\x07\n\x05_timeB\t\n\x07_volumeB>\n\x16\x63om.ib.client.protobufB\x13OrderConditionProto\xaa\x02\x0eIBApi.protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'OrderCondition_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.ib.client.protobufB\023OrderConditionProto\252\002\016IBApi.protobuf'
  _globals['_ORDERCONDITION']._serialized_start=35
  _globals['_ORDERCONDITION']._serialized_end=525
# @@protoc_insertion_point(module_scope)
