/* Copyright (C) 2025 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 * and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable. */

syntax = "proto3";

package protobuf;

option java_outer_classname = "OrderStatusProto";
option java_package = "com.ib.client.protobuf";
option csharp_namespace = "IBApi.protobuf";

message OrderStatus {
    optional int32 orderId = 1;
    optional string status = 2;
    optional string filled = 3;
    optional string remaining = 4;
    optional double avgFillPrice = 5;
    optional int64 permId = 6;
    optional int32 parentId = 7;
    optional double lastFillPrice = 8;
    optional int32 clientId = 9;
    optional string whyHeld = 10;
    optional double mktCapPrice = 11;
}
