// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: ErrorMessage.proto
// Protobuf Java Version: 4.29.3

package com.ib.client.protobuf;

public final class ErrorMessageProto {
  private ErrorMessageProto() {}
  static {
    com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
      com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
      /* major= */ 4,
      /* minor= */ 29,
      /* patch= */ 3,
      /* suffix= */ "",
      ErrorMessageProto.class.getName());
  }
  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistryLite registry) {
  }

  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistry registry) {
    registerAllExtensions(
        (com.google.protobuf.ExtensionRegistryLite) registry);
  }
  public interface ErrorMessageOrBuilder extends
      // @@protoc_insertion_point(interface_extends:protobuf.ErrorMessage)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>optional int32 id = 1;</code>
     * @return Whether the id field is set.
     */
    boolean hasId();
    /**
     * <code>optional int32 id = 1;</code>
     * @return The id.
     */
    int getId();

    /**
     * <code>optional int64 errorTime = 2;</code>
     * @return Whether the errorTime field is set.
     */
    boolean hasErrorTime();
    /**
     * <code>optional int64 errorTime = 2;</code>
     * @return The errorTime.
     */
    long getErrorTime();

    /**
     * <code>optional int32 errorCode = 3;</code>
     * @return Whether the errorCode field is set.
     */
    boolean hasErrorCode();
    /**
     * <code>optional int32 errorCode = 3;</code>
     * @return The errorCode.
     */
    int getErrorCode();

    /**
     * <code>optional string errorMsg = 4;</code>
     * @return Whether the errorMsg field is set.
     */
    boolean hasErrorMsg();
    /**
     * <code>optional string errorMsg = 4;</code>
     * @return The errorMsg.
     */
    java.lang.String getErrorMsg();
    /**
     * <code>optional string errorMsg = 4;</code>
     * @return The bytes for errorMsg.
     */
    com.google.protobuf.ByteString
        getErrorMsgBytes();

    /**
     * <code>optional string advancedOrderRejectJson = 5;</code>
     * @return Whether the advancedOrderRejectJson field is set.
     */
    boolean hasAdvancedOrderRejectJson();
    /**
     * <code>optional string advancedOrderRejectJson = 5;</code>
     * @return The advancedOrderRejectJson.
     */
    java.lang.String getAdvancedOrderRejectJson();
    /**
     * <code>optional string advancedOrderRejectJson = 5;</code>
     * @return The bytes for advancedOrderRejectJson.
     */
    com.google.protobuf.ByteString
        getAdvancedOrderRejectJsonBytes();
  }
  /**
   * Protobuf type {@code protobuf.ErrorMessage}
   */
  public static final class ErrorMessage extends
      com.google.protobuf.GeneratedMessage implements
      // @@protoc_insertion_point(message_implements:protobuf.ErrorMessage)
      ErrorMessageOrBuilder {
  private static final long serialVersionUID = 0L;
    static {
      com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
        com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
        /* major= */ 4,
        /* minor= */ 29,
        /* patch= */ 3,
        /* suffix= */ "",
        ErrorMessage.class.getName());
    }
    // Use ErrorMessage.newBuilder() to construct.
    private ErrorMessage(com.google.protobuf.GeneratedMessage.Builder<?> builder) {
      super(builder);
    }
    private ErrorMessage() {
      errorMsg_ = "";
      advancedOrderRejectJson_ = "";
    }

    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return com.ib.client.protobuf.ErrorMessageProto.internal_static_protobuf_ErrorMessage_descriptor;
    }

    @java.lang.Override
    protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return com.ib.client.protobuf.ErrorMessageProto.internal_static_protobuf_ErrorMessage_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              com.ib.client.protobuf.ErrorMessageProto.ErrorMessage.class, com.ib.client.protobuf.ErrorMessageProto.ErrorMessage.Builder.class);
    }

    private int bitField0_;
    public static final int ID_FIELD_NUMBER = 1;
    private int id_ = 0;
    /**
     * <code>optional int32 id = 1;</code>
     * @return Whether the id field is set.
     */
    @java.lang.Override
    public boolean hasId() {
      return ((bitField0_ & 0x00000001) != 0);
    }
    /**
     * <code>optional int32 id = 1;</code>
     * @return The id.
     */
    @java.lang.Override
    public int getId() {
      return id_;
    }

    public static final int ERRORTIME_FIELD_NUMBER = 2;
    private long errorTime_ = 0L;
    /**
     * <code>optional int64 errorTime = 2;</code>
     * @return Whether the errorTime field is set.
     */
    @java.lang.Override
    public boolean hasErrorTime() {
      return ((bitField0_ & 0x00000002) != 0);
    }
    /**
     * <code>optional int64 errorTime = 2;</code>
     * @return The errorTime.
     */
    @java.lang.Override
    public long getErrorTime() {
      return errorTime_;
    }

    public static final int ERRORCODE_FIELD_NUMBER = 3;
    private int errorCode_ = 0;
    /**
     * <code>optional int32 errorCode = 3;</code>
     * @return Whether the errorCode field is set.
     */
    @java.lang.Override
    public boolean hasErrorCode() {
      return ((bitField0_ & 0x00000004) != 0);
    }
    /**
     * <code>optional int32 errorCode = 3;</code>
     * @return The errorCode.
     */
    @java.lang.Override
    public int getErrorCode() {
      return errorCode_;
    }

    public static final int ERRORMSG_FIELD_NUMBER = 4;
    @SuppressWarnings("serial")
    private volatile java.lang.Object errorMsg_ = "";
    /**
     * <code>optional string errorMsg = 4;</code>
     * @return Whether the errorMsg field is set.
     */
    @java.lang.Override
    public boolean hasErrorMsg() {
      return ((bitField0_ & 0x00000008) != 0);
    }
    /**
     * <code>optional string errorMsg = 4;</code>
     * @return The errorMsg.
     */
    @java.lang.Override
    public java.lang.String getErrorMsg() {
      java.lang.Object ref = errorMsg_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        errorMsg_ = s;
        return s;
      }
    }
    /**
     * <code>optional string errorMsg = 4;</code>
     * @return The bytes for errorMsg.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getErrorMsgBytes() {
      java.lang.Object ref = errorMsg_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        errorMsg_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int ADVANCEDORDERREJECTJSON_FIELD_NUMBER = 5;
    @SuppressWarnings("serial")
    private volatile java.lang.Object advancedOrderRejectJson_ = "";
    /**
     * <code>optional string advancedOrderRejectJson = 5;</code>
     * @return Whether the advancedOrderRejectJson field is set.
     */
    @java.lang.Override
    public boolean hasAdvancedOrderRejectJson() {
      return ((bitField0_ & 0x00000010) != 0);
    }
    /**
     * <code>optional string advancedOrderRejectJson = 5;</code>
     * @return The advancedOrderRejectJson.
     */
    @java.lang.Override
    public java.lang.String getAdvancedOrderRejectJson() {
      java.lang.Object ref = advancedOrderRejectJson_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        advancedOrderRejectJson_ = s;
        return s;
      }
    }
    /**
     * <code>optional string advancedOrderRejectJson = 5;</code>
     * @return The bytes for advancedOrderRejectJson.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getAdvancedOrderRejectJsonBytes() {
      java.lang.Object ref = advancedOrderRejectJson_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        advancedOrderRejectJson_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      if (((bitField0_ & 0x00000001) != 0)) {
        output.writeInt32(1, id_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        output.writeInt64(2, errorTime_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        output.writeInt32(3, errorCode_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 4, errorMsg_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 5, advancedOrderRejectJson_);
      }
      getUnknownFields().writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      if (((bitField0_ & 0x00000001) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(1, id_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt64Size(2, errorTime_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(3, errorCode_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(4, errorMsg_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(5, advancedOrderRejectJson_);
      }
      size += getUnknownFields().getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof com.ib.client.protobuf.ErrorMessageProto.ErrorMessage)) {
        return super.equals(obj);
      }
      com.ib.client.protobuf.ErrorMessageProto.ErrorMessage other = (com.ib.client.protobuf.ErrorMessageProto.ErrorMessage) obj;

      if (hasId() != other.hasId()) return false;
      if (hasId()) {
        if (getId()
            != other.getId()) return false;
      }
      if (hasErrorTime() != other.hasErrorTime()) return false;
      if (hasErrorTime()) {
        if (getErrorTime()
            != other.getErrorTime()) return false;
      }
      if (hasErrorCode() != other.hasErrorCode()) return false;
      if (hasErrorCode()) {
        if (getErrorCode()
            != other.getErrorCode()) return false;
      }
      if (hasErrorMsg() != other.hasErrorMsg()) return false;
      if (hasErrorMsg()) {
        if (!getErrorMsg()
            .equals(other.getErrorMsg())) return false;
      }
      if (hasAdvancedOrderRejectJson() != other.hasAdvancedOrderRejectJson()) return false;
      if (hasAdvancedOrderRejectJson()) {
        if (!getAdvancedOrderRejectJson()
            .equals(other.getAdvancedOrderRejectJson())) return false;
      }
      if (!getUnknownFields().equals(other.getUnknownFields())) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      if (hasId()) {
        hash = (37 * hash) + ID_FIELD_NUMBER;
        hash = (53 * hash) + getId();
      }
      if (hasErrorTime()) {
        hash = (37 * hash) + ERRORTIME_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            getErrorTime());
      }
      if (hasErrorCode()) {
        hash = (37 * hash) + ERRORCODE_FIELD_NUMBER;
        hash = (53 * hash) + getErrorCode();
      }
      if (hasErrorMsg()) {
        hash = (37 * hash) + ERRORMSG_FIELD_NUMBER;
        hash = (53 * hash) + getErrorMsg().hashCode();
      }
      if (hasAdvancedOrderRejectJson()) {
        hash = (37 * hash) + ADVANCEDORDERREJECTJSON_FIELD_NUMBER;
        hash = (53 * hash) + getAdvancedOrderRejectJson().hashCode();
      }
      hash = (29 * hash) + getUnknownFields().hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input);
    }

    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(com.ib.client.protobuf.ErrorMessageProto.ErrorMessage prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessage.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code protobuf.ErrorMessage}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessage.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:protobuf.ErrorMessage)
        com.ib.client.protobuf.ErrorMessageProto.ErrorMessageOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return com.ib.client.protobuf.ErrorMessageProto.internal_static_protobuf_ErrorMessage_descriptor;
      }

      @java.lang.Override
      protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return com.ib.client.protobuf.ErrorMessageProto.internal_static_protobuf_ErrorMessage_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                com.ib.client.protobuf.ErrorMessageProto.ErrorMessage.class, com.ib.client.protobuf.ErrorMessageProto.ErrorMessage.Builder.class);
      }

      // Construct using com.ib.client.protobuf.ErrorMessageProto.ErrorMessage.newBuilder()
      private Builder() {

      }

      private Builder(
          com.google.protobuf.GeneratedMessage.BuilderParent parent) {
        super(parent);

      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        bitField0_ = 0;
        id_ = 0;
        errorTime_ = 0L;
        errorCode_ = 0;
        errorMsg_ = "";
        advancedOrderRejectJson_ = "";
        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return com.ib.client.protobuf.ErrorMessageProto.internal_static_protobuf_ErrorMessage_descriptor;
      }

      @java.lang.Override
      public com.ib.client.protobuf.ErrorMessageProto.ErrorMessage getDefaultInstanceForType() {
        return com.ib.client.protobuf.ErrorMessageProto.ErrorMessage.getDefaultInstance();
      }

      @java.lang.Override
      public com.ib.client.protobuf.ErrorMessageProto.ErrorMessage build() {
        com.ib.client.protobuf.ErrorMessageProto.ErrorMessage result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public com.ib.client.protobuf.ErrorMessageProto.ErrorMessage buildPartial() {
        com.ib.client.protobuf.ErrorMessageProto.ErrorMessage result = new com.ib.client.protobuf.ErrorMessageProto.ErrorMessage(this);
        if (bitField0_ != 0) { buildPartial0(result); }
        onBuilt();
        return result;
      }

      private void buildPartial0(com.ib.client.protobuf.ErrorMessageProto.ErrorMessage result) {
        int from_bitField0_ = bitField0_;
        int to_bitField0_ = 0;
        if (((from_bitField0_ & 0x00000001) != 0)) {
          result.id_ = id_;
          to_bitField0_ |= 0x00000001;
        }
        if (((from_bitField0_ & 0x00000002) != 0)) {
          result.errorTime_ = errorTime_;
          to_bitField0_ |= 0x00000002;
        }
        if (((from_bitField0_ & 0x00000004) != 0)) {
          result.errorCode_ = errorCode_;
          to_bitField0_ |= 0x00000004;
        }
        if (((from_bitField0_ & 0x00000008) != 0)) {
          result.errorMsg_ = errorMsg_;
          to_bitField0_ |= 0x00000008;
        }
        if (((from_bitField0_ & 0x00000010) != 0)) {
          result.advancedOrderRejectJson_ = advancedOrderRejectJson_;
          to_bitField0_ |= 0x00000010;
        }
        result.bitField0_ |= to_bitField0_;
      }

      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof com.ib.client.protobuf.ErrorMessageProto.ErrorMessage) {
          return mergeFrom((com.ib.client.protobuf.ErrorMessageProto.ErrorMessage)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(com.ib.client.protobuf.ErrorMessageProto.ErrorMessage other) {
        if (other == com.ib.client.protobuf.ErrorMessageProto.ErrorMessage.getDefaultInstance()) return this;
        if (other.hasId()) {
          setId(other.getId());
        }
        if (other.hasErrorTime()) {
          setErrorTime(other.getErrorTime());
        }
        if (other.hasErrorCode()) {
          setErrorCode(other.getErrorCode());
        }
        if (other.hasErrorMsg()) {
          errorMsg_ = other.errorMsg_;
          bitField0_ |= 0x00000008;
          onChanged();
        }
        if (other.hasAdvancedOrderRejectJson()) {
          advancedOrderRejectJson_ = other.advancedOrderRejectJson_;
          bitField0_ |= 0x00000010;
          onChanged();
        }
        this.mergeUnknownFields(other.getUnknownFields());
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        if (extensionRegistry == null) {
          throw new java.lang.NullPointerException();
        }
        try {
          boolean done = false;
          while (!done) {
            int tag = input.readTag();
            switch (tag) {
              case 0:
                done = true;
                break;
              case 8: {
                id_ = input.readInt32();
                bitField0_ |= 0x00000001;
                break;
              } // case 8
              case 16: {
                errorTime_ = input.readInt64();
                bitField0_ |= 0x00000002;
                break;
              } // case 16
              case 24: {
                errorCode_ = input.readInt32();
                bitField0_ |= 0x00000004;
                break;
              } // case 24
              case 34: {
                errorMsg_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000008;
                break;
              } // case 34
              case 42: {
                advancedOrderRejectJson_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000010;
                break;
              } // case 42
              default: {
                if (!super.parseUnknownField(input, extensionRegistry, tag)) {
                  done = true; // was an endgroup tag
                }
                break;
              } // default:
            } // switch (tag)
          } // while (!done)
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.unwrapIOException();
        } finally {
          onChanged();
        } // finally
        return this;
      }
      private int bitField0_;

      private int id_ ;
      /**
       * <code>optional int32 id = 1;</code>
       * @return Whether the id field is set.
       */
      @java.lang.Override
      public boolean hasId() {
        return ((bitField0_ & 0x00000001) != 0);
      }
      /**
       * <code>optional int32 id = 1;</code>
       * @return The id.
       */
      @java.lang.Override
      public int getId() {
        return id_;
      }
      /**
       * <code>optional int32 id = 1;</code>
       * @param value The id to set.
       * @return This builder for chaining.
       */
      public Builder setId(int value) {

        id_ = value;
        bitField0_ |= 0x00000001;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 id = 1;</code>
       * @return This builder for chaining.
       */
      public Builder clearId() {
        bitField0_ = (bitField0_ & ~0x00000001);
        id_ = 0;
        onChanged();
        return this;
      }

      private long errorTime_ ;
      /**
       * <code>optional int64 errorTime = 2;</code>
       * @return Whether the errorTime field is set.
       */
      @java.lang.Override
      public boolean hasErrorTime() {
        return ((bitField0_ & 0x00000002) != 0);
      }
      /**
       * <code>optional int64 errorTime = 2;</code>
       * @return The errorTime.
       */
      @java.lang.Override
      public long getErrorTime() {
        return errorTime_;
      }
      /**
       * <code>optional int64 errorTime = 2;</code>
       * @param value The errorTime to set.
       * @return This builder for chaining.
       */
      public Builder setErrorTime(long value) {

        errorTime_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }
      /**
       * <code>optional int64 errorTime = 2;</code>
       * @return This builder for chaining.
       */
      public Builder clearErrorTime() {
        bitField0_ = (bitField0_ & ~0x00000002);
        errorTime_ = 0L;
        onChanged();
        return this;
      }

      private int errorCode_ ;
      /**
       * <code>optional int32 errorCode = 3;</code>
       * @return Whether the errorCode field is set.
       */
      @java.lang.Override
      public boolean hasErrorCode() {
        return ((bitField0_ & 0x00000004) != 0);
      }
      /**
       * <code>optional int32 errorCode = 3;</code>
       * @return The errorCode.
       */
      @java.lang.Override
      public int getErrorCode() {
        return errorCode_;
      }
      /**
       * <code>optional int32 errorCode = 3;</code>
       * @param value The errorCode to set.
       * @return This builder for chaining.
       */
      public Builder setErrorCode(int value) {

        errorCode_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 errorCode = 3;</code>
       * @return This builder for chaining.
       */
      public Builder clearErrorCode() {
        bitField0_ = (bitField0_ & ~0x00000004);
        errorCode_ = 0;
        onChanged();
        return this;
      }

      private java.lang.Object errorMsg_ = "";
      /**
       * <code>optional string errorMsg = 4;</code>
       * @return Whether the errorMsg field is set.
       */
      public boolean hasErrorMsg() {
        return ((bitField0_ & 0x00000008) != 0);
      }
      /**
       * <code>optional string errorMsg = 4;</code>
       * @return The errorMsg.
       */
      public java.lang.String getErrorMsg() {
        java.lang.Object ref = errorMsg_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          errorMsg_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string errorMsg = 4;</code>
       * @return The bytes for errorMsg.
       */
      public com.google.protobuf.ByteString
          getErrorMsgBytes() {
        java.lang.Object ref = errorMsg_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          errorMsg_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string errorMsg = 4;</code>
       * @param value The errorMsg to set.
       * @return This builder for chaining.
       */
      public Builder setErrorMsg(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        errorMsg_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }
      /**
       * <code>optional string errorMsg = 4;</code>
       * @return This builder for chaining.
       */
      public Builder clearErrorMsg() {
        errorMsg_ = getDefaultInstance().getErrorMsg();
        bitField0_ = (bitField0_ & ~0x00000008);
        onChanged();
        return this;
      }
      /**
       * <code>optional string errorMsg = 4;</code>
       * @param value The bytes for errorMsg to set.
       * @return This builder for chaining.
       */
      public Builder setErrorMsgBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        errorMsg_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }

      private java.lang.Object advancedOrderRejectJson_ = "";
      /**
       * <code>optional string advancedOrderRejectJson = 5;</code>
       * @return Whether the advancedOrderRejectJson field is set.
       */
      public boolean hasAdvancedOrderRejectJson() {
        return ((bitField0_ & 0x00000010) != 0);
      }
      /**
       * <code>optional string advancedOrderRejectJson = 5;</code>
       * @return The advancedOrderRejectJson.
       */
      public java.lang.String getAdvancedOrderRejectJson() {
        java.lang.Object ref = advancedOrderRejectJson_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          advancedOrderRejectJson_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string advancedOrderRejectJson = 5;</code>
       * @return The bytes for advancedOrderRejectJson.
       */
      public com.google.protobuf.ByteString
          getAdvancedOrderRejectJsonBytes() {
        java.lang.Object ref = advancedOrderRejectJson_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          advancedOrderRejectJson_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string advancedOrderRejectJson = 5;</code>
       * @param value The advancedOrderRejectJson to set.
       * @return This builder for chaining.
       */
      public Builder setAdvancedOrderRejectJson(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        advancedOrderRejectJson_ = value;
        bitField0_ |= 0x00000010;
        onChanged();
        return this;
      }
      /**
       * <code>optional string advancedOrderRejectJson = 5;</code>
       * @return This builder for chaining.
       */
      public Builder clearAdvancedOrderRejectJson() {
        advancedOrderRejectJson_ = getDefaultInstance().getAdvancedOrderRejectJson();
        bitField0_ = (bitField0_ & ~0x00000010);
        onChanged();
        return this;
      }
      /**
       * <code>optional string advancedOrderRejectJson = 5;</code>
       * @param value The bytes for advancedOrderRejectJson to set.
       * @return This builder for chaining.
       */
      public Builder setAdvancedOrderRejectJsonBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        advancedOrderRejectJson_ = value;
        bitField0_ |= 0x00000010;
        onChanged();
        return this;
      }

      // @@protoc_insertion_point(builder_scope:protobuf.ErrorMessage)
    }

    // @@protoc_insertion_point(class_scope:protobuf.ErrorMessage)
    private static final com.ib.client.protobuf.ErrorMessageProto.ErrorMessage DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new com.ib.client.protobuf.ErrorMessageProto.ErrorMessage();
    }

    public static com.ib.client.protobuf.ErrorMessageProto.ErrorMessage getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<ErrorMessage>
        PARSER = new com.google.protobuf.AbstractParser<ErrorMessage>() {
      @java.lang.Override
      public ErrorMessage parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        Builder builder = newBuilder();
        try {
          builder.mergeFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.setUnfinishedMessage(builder.buildPartial());
        } catch (com.google.protobuf.UninitializedMessageException e) {
          throw e.asInvalidProtocolBufferException().setUnfinishedMessage(builder.buildPartial());
        } catch (java.io.IOException e) {
          throw new com.google.protobuf.InvalidProtocolBufferException(e)
              .setUnfinishedMessage(builder.buildPartial());
        }
        return builder.buildPartial();
      }
    };

    public static com.google.protobuf.Parser<ErrorMessage> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<ErrorMessage> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public com.ib.client.protobuf.ErrorMessageProto.ErrorMessage getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_protobuf_ErrorMessage_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessage.FieldAccessorTable
      internal_static_protobuf_ErrorMessage_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor
      getDescriptor() {
    return descriptor;
  }
  private static  com.google.protobuf.Descriptors.FileDescriptor
      descriptor;
  static {
    java.lang.String[] descriptorData = {
      "\n\022ErrorMessage.proto\022\010protobuf\"\330\001\n\014Error" +
      "Message\022\017\n\002id\030\001 \001(\005H\000\210\001\001\022\026\n\terrorTime\030\002 " +
      "\001(\003H\001\210\001\001\022\026\n\terrorCode\030\003 \001(\005H\002\210\001\001\022\025\n\010erro" +
      "rMsg\030\004 \001(\tH\003\210\001\001\022$\n\027advancedOrderRejectJs" +
      "on\030\005 \001(\tH\004\210\001\001B\005\n\003_idB\014\n\n_errorTimeB\014\n\n_e" +
      "rrorCodeB\013\n\t_errorMsgB\032\n\030_advancedOrderR" +
      "ejectJsonB<\n\026com.ib.client.protobufB\021Err" +
      "orMessageProto\252\002\016IBApi.protobufb\006proto3"
    };
    descriptor = com.google.protobuf.Descriptors.FileDescriptor
      .internalBuildGeneratedFileFrom(descriptorData,
        new com.google.protobuf.Descriptors.FileDescriptor[] {
        });
    internal_static_protobuf_ErrorMessage_descriptor =
      getDescriptor().getMessageTypes().get(0);
    internal_static_protobuf_ErrorMessage_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessage.FieldAccessorTable(
        internal_static_protobuf_ErrorMessage_descriptor,
        new java.lang.String[] { "Id", "ErrorTime", "ErrorCode", "ErrorMsg", "AdvancedOrderRejectJson", });
    descriptor.resolveAllFeaturesImmutable();
  }

  // @@protoc_insertion_point(outer_class_scope)
}
