/* Copyright (C) 2025 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 * and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable. */

syntax = "proto3";

package protobuf;

option java_outer_classname = "ComboLegProto";
option java_package = "com.ib.client.protobuf";
option csharp_namespace = "IBApi.protobuf";

message ComboLeg {
    optional int32 conId = 1;
    optional int32 ratio = 2;
    optional string action = 3;
    optional string exchange = 4;
    optional int32 openClose = 5;
    optional int32 shortSalesSlot = 6;
    optional string designatedLocation = 7;
    optional int32 exemptCode = 8;
    optional double perLegPrice = 9;
}
