/* Copyright (C) 2025 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 * and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable. */

syntax = "proto3";

package protobuf;

option java_outer_classname = "ErrorMessageProto";
option java_package = "com.ib.client.protobuf";
option csharp_namespace = "IBApi.protobuf";

message ErrorMessage {
    optional int32 id = 1;
    optional int64 errorTime = 2;
    optional int32 errorCode = 3;
    optional string errorMsg = 4;
    optional string advancedOrderRejectJson = 5;
}
