// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: GlobalCancelRequest.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_GlobalCancelRequest_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_GlobalCancelRequest_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3012000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3012004 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_table_driven.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/inlined_string_field.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "OrderCancel.pb.h"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_GlobalCancelRequest_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_GlobalCancelRequest_2eproto {
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTableField entries[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::AuxillaryParseTableField aux[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTable schema[1]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::FieldMetadata field_metadata[];
  static const ::PROTOBUF_NAMESPACE_ID::internal::SerializationTable serialization_table[];
  static const ::PROTOBUF_NAMESPACE_ID::uint32 offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_GlobalCancelRequest_2eproto;
namespace protobuf {
class GlobalCancelRequest;
class GlobalCancelRequestDefaultTypeInternal;
extern GlobalCancelRequestDefaultTypeInternal _GlobalCancelRequest_default_instance_;
}  // namespace protobuf
PROTOBUF_NAMESPACE_OPEN
template<> ::protobuf::GlobalCancelRequest* Arena::CreateMaybeMessage<::protobuf::GlobalCancelRequest>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace protobuf {

// ===================================================================

class GlobalCancelRequest PROTOBUF_FINAL :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:protobuf.GlobalCancelRequest) */ {
 public:
  inline GlobalCancelRequest() : GlobalCancelRequest(nullptr) {};
  virtual ~GlobalCancelRequest();

  GlobalCancelRequest(const GlobalCancelRequest& from);
  GlobalCancelRequest(GlobalCancelRequest&& from) noexcept
    : GlobalCancelRequest() {
    *this = ::std::move(from);
  }

  inline GlobalCancelRequest& operator=(const GlobalCancelRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline GlobalCancelRequest& operator=(GlobalCancelRequest&& from) noexcept {
    if (GetArena() == from.GetArena()) {
      if (this != &from) InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return GetMetadataStatic().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return GetMetadataStatic().reflection;
  }
  static const GlobalCancelRequest& default_instance();

  static void InitAsDefaultInstance();  // FOR INTERNAL USE ONLY
  static inline const GlobalCancelRequest* internal_default_instance() {
    return reinterpret_cast<const GlobalCancelRequest*>(
               &_GlobalCancelRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(GlobalCancelRequest& a, GlobalCancelRequest& b) {
    a.Swap(&b);
  }
  inline void Swap(GlobalCancelRequest* other) {
    if (other == this) return;
    if (GetArena() == other->GetArena()) {
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(GlobalCancelRequest* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  inline GlobalCancelRequest* New() const final {
    return CreateMaybeMessage<GlobalCancelRequest>(nullptr);
  }

  GlobalCancelRequest* New(::PROTOBUF_NAMESPACE_ID::Arena* arena) const final {
    return CreateMaybeMessage<GlobalCancelRequest>(arena);
  }
  void CopyFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void MergeFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void CopyFrom(const GlobalCancelRequest& from);
  void MergeFrom(const GlobalCancelRequest& from);
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  ::PROTOBUF_NAMESPACE_ID::uint8* _InternalSerialize(
      ::PROTOBUF_NAMESPACE_ID::uint8* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  inline void SharedCtor();
  inline void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(GlobalCancelRequest* other);
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "protobuf.GlobalCancelRequest";
  }
  protected:
  explicit GlobalCancelRequest(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  private:
  static void ArenaDtor(void* object);
  inline void RegisterArenaDtor(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  public:

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;
  private:
  static ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadataStatic() {
    ::PROTOBUF_NAMESPACE_ID::internal::AssignDescriptors(&::descriptor_table_GlobalCancelRequest_2eproto);
    return ::descriptor_table_GlobalCancelRequest_2eproto.file_level_metadata[kIndexInFileMessages];
  }

  public:

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kOrderCancelFieldNumber = 1,
  };
  // .protobuf.OrderCancel orderCancel = 1;
  bool has_ordercancel() const;
  private:
  bool _internal_has_ordercancel() const;
  public:
  void clear_ordercancel();
  const ::protobuf::OrderCancel& ordercancel() const;
  ::protobuf::OrderCancel* release_ordercancel();
  ::protobuf::OrderCancel* mutable_ordercancel();
  void set_allocated_ordercancel(::protobuf::OrderCancel* ordercancel);
  private:
  const ::protobuf::OrderCancel& _internal_ordercancel() const;
  ::protobuf::OrderCancel* _internal_mutable_ordercancel();
  public:
  void unsafe_arena_set_allocated_ordercancel(
      ::protobuf::OrderCancel* ordercancel);
  ::protobuf::OrderCancel* unsafe_arena_release_ordercancel();

  // @@protoc_insertion_point(class_scope:protobuf.GlobalCancelRequest)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  ::protobuf::OrderCancel* ordercancel_;
  friend struct ::TableStruct_GlobalCancelRequest_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// GlobalCancelRequest

// .protobuf.OrderCancel orderCancel = 1;
inline bool GlobalCancelRequest::_internal_has_ordercancel() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  PROTOBUF_ASSUME(!value || ordercancel_ != nullptr);
  return value;
}
inline bool GlobalCancelRequest::has_ordercancel() const {
  return _internal_has_ordercancel();
}
inline const ::protobuf::OrderCancel& GlobalCancelRequest::_internal_ordercancel() const {
  const ::protobuf::OrderCancel* p = ordercancel_;
  return p != nullptr ? *p : *reinterpret_cast<const ::protobuf::OrderCancel*>(
      &::protobuf::_OrderCancel_default_instance_);
}
inline const ::protobuf::OrderCancel& GlobalCancelRequest::ordercancel() const {
  // @@protoc_insertion_point(field_get:protobuf.GlobalCancelRequest.orderCancel)
  return _internal_ordercancel();
}
inline void GlobalCancelRequest::unsafe_arena_set_allocated_ordercancel(
    ::protobuf::OrderCancel* ordercancel) {
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(ordercancel_);
  }
  ordercancel_ = ordercancel;
  if (ordercancel) {
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:protobuf.GlobalCancelRequest.orderCancel)
}
inline ::protobuf::OrderCancel* GlobalCancelRequest::release_ordercancel() {
  auto temp = unsafe_arena_release_ordercancel();
  if (GetArena() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
  return temp;
}
inline ::protobuf::OrderCancel* GlobalCancelRequest::unsafe_arena_release_ordercancel() {
  // @@protoc_insertion_point(field_release:protobuf.GlobalCancelRequest.orderCancel)
  _has_bits_[0] &= ~0x00000001u;
  ::protobuf::OrderCancel* temp = ordercancel_;
  ordercancel_ = nullptr;
  return temp;
}
inline ::protobuf::OrderCancel* GlobalCancelRequest::_internal_mutable_ordercancel() {
  _has_bits_[0] |= 0x00000001u;
  if (ordercancel_ == nullptr) {
    auto* p = CreateMaybeMessage<::protobuf::OrderCancel>(GetArena());
    ordercancel_ = p;
  }
  return ordercancel_;
}
inline ::protobuf::OrderCancel* GlobalCancelRequest::mutable_ordercancel() {
  // @@protoc_insertion_point(field_mutable:protobuf.GlobalCancelRequest.orderCancel)
  return _internal_mutable_ordercancel();
}
inline void GlobalCancelRequest::set_allocated_ordercancel(::protobuf::OrderCancel* ordercancel) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArena();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(ordercancel_);
  }
  if (ordercancel) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(ordercancel)->GetArena();
    if (message_arena != submessage_arena) {
      ordercancel = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, ordercancel, submessage_arena);
    }
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  ordercancel_ = ordercancel;
  // @@protoc_insertion_point(field_set_allocated:protobuf.GlobalCancelRequest.orderCancel)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)

}  // namespace protobuf

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_GlobalCancelRequest_2eproto
