/* Copyright (C) 2019 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 * and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable. */

#pragma once
#ifdef _MSC_VER
#ifndef TWS_API_CLIENT_STDAFX_H
#define TWS_API_CLIENT_STDAFX_H

#include "platformspecific.h"

#include <string>
#include <deque>
#include <vector>
#include <algorithm>
#include <limits.h>
#include <sstream>
#include <iomanip>
#include <stdio.h>
#include <assert.h>
#include <memory>
#include <ostream>
#include <string.h>
#include <assert.h>
#include <atomic>
#include <bitset>
#include <cstdlib>
#include <sstream>
#include <stdexcept>
#include <set>
#include <map>
#include <tuple>
#include <ios>
#include <float.h>

#endif
#endif
