// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: PlaceOrderRequest.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_PlaceOrderRequest_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_PlaceOrderRequest_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3012000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3012004 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_table_driven.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/inlined_string_field.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "Contract.pb.h"
#include "Order.pb.h"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_PlaceOrderRequest_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_PlaceOrderRequest_2eproto {
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTableField entries[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::AuxillaryParseTableField aux[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTable schema[1]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::FieldMetadata field_metadata[];
  static const ::PROTOBUF_NAMESPACE_ID::internal::SerializationTable serialization_table[];
  static const ::PROTOBUF_NAMESPACE_ID::uint32 offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_PlaceOrderRequest_2eproto;
namespace protobuf {
class PlaceOrderRequest;
class PlaceOrderRequestDefaultTypeInternal;
extern PlaceOrderRequestDefaultTypeInternal _PlaceOrderRequest_default_instance_;
}  // namespace protobuf
PROTOBUF_NAMESPACE_OPEN
template<> ::protobuf::PlaceOrderRequest* Arena::CreateMaybeMessage<::protobuf::PlaceOrderRequest>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace protobuf {

// ===================================================================

class PlaceOrderRequest PROTOBUF_FINAL :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:protobuf.PlaceOrderRequest) */ {
 public:
  inline PlaceOrderRequest() : PlaceOrderRequest(nullptr) {};
  virtual ~PlaceOrderRequest();

  PlaceOrderRequest(const PlaceOrderRequest& from);
  PlaceOrderRequest(PlaceOrderRequest&& from) noexcept
    : PlaceOrderRequest() {
    *this = ::std::move(from);
  }

  inline PlaceOrderRequest& operator=(const PlaceOrderRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline PlaceOrderRequest& operator=(PlaceOrderRequest&& from) noexcept {
    if (GetArena() == from.GetArena()) {
      if (this != &from) InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return GetMetadataStatic().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return GetMetadataStatic().reflection;
  }
  static const PlaceOrderRequest& default_instance();

  static void InitAsDefaultInstance();  // FOR INTERNAL USE ONLY
  static inline const PlaceOrderRequest* internal_default_instance() {
    return reinterpret_cast<const PlaceOrderRequest*>(
               &_PlaceOrderRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(PlaceOrderRequest& a, PlaceOrderRequest& b) {
    a.Swap(&b);
  }
  inline void Swap(PlaceOrderRequest* other) {
    if (other == this) return;
    if (GetArena() == other->GetArena()) {
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PlaceOrderRequest* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  inline PlaceOrderRequest* New() const final {
    return CreateMaybeMessage<PlaceOrderRequest>(nullptr);
  }

  PlaceOrderRequest* New(::PROTOBUF_NAMESPACE_ID::Arena* arena) const final {
    return CreateMaybeMessage<PlaceOrderRequest>(arena);
  }
  void CopyFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void MergeFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void CopyFrom(const PlaceOrderRequest& from);
  void MergeFrom(const PlaceOrderRequest& from);
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  ::PROTOBUF_NAMESPACE_ID::uint8* _InternalSerialize(
      ::PROTOBUF_NAMESPACE_ID::uint8* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  inline void SharedCtor();
  inline void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(PlaceOrderRequest* other);
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "protobuf.PlaceOrderRequest";
  }
  protected:
  explicit PlaceOrderRequest(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  private:
  static void ArenaDtor(void* object);
  inline void RegisterArenaDtor(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  public:

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;
  private:
  static ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadataStatic() {
    ::PROTOBUF_NAMESPACE_ID::internal::AssignDescriptors(&::descriptor_table_PlaceOrderRequest_2eproto);
    return ::descriptor_table_PlaceOrderRequest_2eproto.file_level_metadata[kIndexInFileMessages];
  }

  public:

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kContractFieldNumber = 2,
    kOrderFieldNumber = 3,
    kOrderIdFieldNumber = 1,
  };
  // .protobuf.Contract contract = 2;
  bool has_contract() const;
  private:
  bool _internal_has_contract() const;
  public:
  void clear_contract();
  const ::protobuf::Contract& contract() const;
  ::protobuf::Contract* release_contract();
  ::protobuf::Contract* mutable_contract();
  void set_allocated_contract(::protobuf::Contract* contract);
  private:
  const ::protobuf::Contract& _internal_contract() const;
  ::protobuf::Contract* _internal_mutable_contract();
  public:
  void unsafe_arena_set_allocated_contract(
      ::protobuf::Contract* contract);
  ::protobuf::Contract* unsafe_arena_release_contract();

  // .protobuf.Order order = 3;
  bool has_order() const;
  private:
  bool _internal_has_order() const;
  public:
  void clear_order();
  const ::protobuf::Order& order() const;
  ::protobuf::Order* release_order();
  ::protobuf::Order* mutable_order();
  void set_allocated_order(::protobuf::Order* order);
  private:
  const ::protobuf::Order& _internal_order() const;
  ::protobuf::Order* _internal_mutable_order();
  public:
  void unsafe_arena_set_allocated_order(
      ::protobuf::Order* order);
  ::protobuf::Order* unsafe_arena_release_order();

  // int32 orderId = 1;
  bool has_orderid() const;
  private:
  bool _internal_has_orderid() const;
  public:
  void clear_orderid();
  ::PROTOBUF_NAMESPACE_ID::int32 orderid() const;
  void set_orderid(::PROTOBUF_NAMESPACE_ID::int32 value);
  private:
  ::PROTOBUF_NAMESPACE_ID::int32 _internal_orderid() const;
  void _internal_set_orderid(::PROTOBUF_NAMESPACE_ID::int32 value);
  public:

  // @@protoc_insertion_point(class_scope:protobuf.PlaceOrderRequest)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  ::protobuf::Contract* contract_;
  ::protobuf::Order* order_;
  ::PROTOBUF_NAMESPACE_ID::int32 orderid_;
  friend struct ::TableStruct_PlaceOrderRequest_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// PlaceOrderRequest

// int32 orderId = 1;
inline bool PlaceOrderRequest::_internal_has_orderid() const {
  bool value = (_has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool PlaceOrderRequest::has_orderid() const {
  return _internal_has_orderid();
}
inline void PlaceOrderRequest::clear_orderid() {
  orderid_ = 0;
  _has_bits_[0] &= ~0x00000004u;
}
inline ::PROTOBUF_NAMESPACE_ID::int32 PlaceOrderRequest::_internal_orderid() const {
  return orderid_;
}
inline ::PROTOBUF_NAMESPACE_ID::int32 PlaceOrderRequest::orderid() const {
  // @@protoc_insertion_point(field_get:protobuf.PlaceOrderRequest.orderId)
  return _internal_orderid();
}
inline void PlaceOrderRequest::_internal_set_orderid(::PROTOBUF_NAMESPACE_ID::int32 value) {
  _has_bits_[0] |= 0x00000004u;
  orderid_ = value;
}
inline void PlaceOrderRequest::set_orderid(::PROTOBUF_NAMESPACE_ID::int32 value) {
  _internal_set_orderid(value);
  // @@protoc_insertion_point(field_set:protobuf.PlaceOrderRequest.orderId)
}

// .protobuf.Contract contract = 2;
inline bool PlaceOrderRequest::_internal_has_contract() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  PROTOBUF_ASSUME(!value || contract_ != nullptr);
  return value;
}
inline bool PlaceOrderRequest::has_contract() const {
  return _internal_has_contract();
}
inline const ::protobuf::Contract& PlaceOrderRequest::_internal_contract() const {
  const ::protobuf::Contract* p = contract_;
  return p != nullptr ? *p : *reinterpret_cast<const ::protobuf::Contract*>(
      &::protobuf::_Contract_default_instance_);
}
inline const ::protobuf::Contract& PlaceOrderRequest::contract() const {
  // @@protoc_insertion_point(field_get:protobuf.PlaceOrderRequest.contract)
  return _internal_contract();
}
inline void PlaceOrderRequest::unsafe_arena_set_allocated_contract(
    ::protobuf::Contract* contract) {
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(contract_);
  }
  contract_ = contract;
  if (contract) {
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:protobuf.PlaceOrderRequest.contract)
}
inline ::protobuf::Contract* PlaceOrderRequest::release_contract() {
  auto temp = unsafe_arena_release_contract();
  if (GetArena() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
  return temp;
}
inline ::protobuf::Contract* PlaceOrderRequest::unsafe_arena_release_contract() {
  // @@protoc_insertion_point(field_release:protobuf.PlaceOrderRequest.contract)
  _has_bits_[0] &= ~0x00000001u;
  ::protobuf::Contract* temp = contract_;
  contract_ = nullptr;
  return temp;
}
inline ::protobuf::Contract* PlaceOrderRequest::_internal_mutable_contract() {
  _has_bits_[0] |= 0x00000001u;
  if (contract_ == nullptr) {
    auto* p = CreateMaybeMessage<::protobuf::Contract>(GetArena());
    contract_ = p;
  }
  return contract_;
}
inline ::protobuf::Contract* PlaceOrderRequest::mutable_contract() {
  // @@protoc_insertion_point(field_mutable:protobuf.PlaceOrderRequest.contract)
  return _internal_mutable_contract();
}
inline void PlaceOrderRequest::set_allocated_contract(::protobuf::Contract* contract) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArena();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(contract_);
  }
  if (contract) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(contract)->GetArena();
    if (message_arena != submessage_arena) {
      contract = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, contract, submessage_arena);
    }
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  contract_ = contract;
  // @@protoc_insertion_point(field_set_allocated:protobuf.PlaceOrderRequest.contract)
}

// .protobuf.Order order = 3;
inline bool PlaceOrderRequest::_internal_has_order() const {
  bool value = (_has_bits_[0] & 0x00000002u) != 0;
  PROTOBUF_ASSUME(!value || order_ != nullptr);
  return value;
}
inline bool PlaceOrderRequest::has_order() const {
  return _internal_has_order();
}
inline const ::protobuf::Order& PlaceOrderRequest::_internal_order() const {
  const ::protobuf::Order* p = order_;
  return p != nullptr ? *p : *reinterpret_cast<const ::protobuf::Order*>(
      &::protobuf::_Order_default_instance_);
}
inline const ::protobuf::Order& PlaceOrderRequest::order() const {
  // @@protoc_insertion_point(field_get:protobuf.PlaceOrderRequest.order)
  return _internal_order();
}
inline void PlaceOrderRequest::unsafe_arena_set_allocated_order(
    ::protobuf::Order* order) {
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(order_);
  }
  order_ = order;
  if (order) {
    _has_bits_[0] |= 0x00000002u;
  } else {
    _has_bits_[0] &= ~0x00000002u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:protobuf.PlaceOrderRequest.order)
}
inline ::protobuf::Order* PlaceOrderRequest::release_order() {
  auto temp = unsafe_arena_release_order();
  if (GetArena() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
  return temp;
}
inline ::protobuf::Order* PlaceOrderRequest::unsafe_arena_release_order() {
  // @@protoc_insertion_point(field_release:protobuf.PlaceOrderRequest.order)
  _has_bits_[0] &= ~0x00000002u;
  ::protobuf::Order* temp = order_;
  order_ = nullptr;
  return temp;
}
inline ::protobuf::Order* PlaceOrderRequest::_internal_mutable_order() {
  _has_bits_[0] |= 0x00000002u;
  if (order_ == nullptr) {
    auto* p = CreateMaybeMessage<::protobuf::Order>(GetArena());
    order_ = p;
  }
  return order_;
}
inline ::protobuf::Order* PlaceOrderRequest::mutable_order() {
  // @@protoc_insertion_point(field_mutable:protobuf.PlaceOrderRequest.order)
  return _internal_mutable_order();
}
inline void PlaceOrderRequest::set_allocated_order(::protobuf::Order* order) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArena();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(order_);
  }
  if (order) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(order)->GetArena();
    if (message_arena != submessage_arena) {
      order = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, order, submessage_arena);
    }
    _has_bits_[0] |= 0x00000002u;
  } else {
    _has_bits_[0] &= ~0x00000002u;
  }
  order_ = order;
  // @@protoc_insertion_point(field_set_allocated:protobuf.PlaceOrderRequest.order)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)

}  // namespace protobuf

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_PlaceOrderRequest_2eproto
