/* Copyright (C) 2025 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 * and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable. */

syntax = "proto3";

package protobuf;

option java_outer_classname = "OrderConditionProto";
option java_package = "com.ib.client.protobuf";
option csharp_namespace = "IBApi.protobuf";

message OrderCondition {
    optional int32 type = 1;
    optional bool isConjunctionConnection = 2;
    optional bool isMore = 3;
    optional int32 conId = 4;
    optional string exchange = 5;
    optional string symbol = 6;
    optional string secType = 7;
    optional int32 percent = 8;
    optional double changePercent = 9;
    optional double price = 10;
    optional int32 triggerMethod = 11;
    optional string time = 12;
    optional int32 volume = 13;
}
