// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: ExecutionDetails.proto
// Protobuf Java Version: 4.29.3

package com.ib.client.protobuf;

public final class ExecutionDetailsProto {
  private ExecutionDetailsProto() {}
  static {
    com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
      com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
      /* major= */ 4,
      /* minor= */ 29,
      /* patch= */ 3,
      /* suffix= */ "",
      ExecutionDetailsProto.class.getName());
  }
  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistryLite registry) {
  }

  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistry registry) {
    registerAllExtensions(
        (com.google.protobuf.ExtensionRegistryLite) registry);
  }
  public interface ExecutionDetailsOrBuilder extends
      // @@protoc_insertion_point(interface_extends:protobuf.ExecutionDetails)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>optional int32 reqId = 1;</code>
     * @return Whether the reqId field is set.
     */
    boolean hasReqId();
    /**
     * <code>optional int32 reqId = 1;</code>
     * @return The reqId.
     */
    int getReqId();

    /**
     * <code>optional .protobuf.Contract contract = 2;</code>
     * @return Whether the contract field is set.
     */
    boolean hasContract();
    /**
     * <code>optional .protobuf.Contract contract = 2;</code>
     * @return The contract.
     */
    com.ib.client.protobuf.ContractProto.Contract getContract();
    /**
     * <code>optional .protobuf.Contract contract = 2;</code>
     */
    com.ib.client.protobuf.ContractProto.ContractOrBuilder getContractOrBuilder();

    /**
     * <code>optional .protobuf.Execution execution = 3;</code>
     * @return Whether the execution field is set.
     */
    boolean hasExecution();
    /**
     * <code>optional .protobuf.Execution execution = 3;</code>
     * @return The execution.
     */
    com.ib.client.protobuf.ExecutionProto.Execution getExecution();
    /**
     * <code>optional .protobuf.Execution execution = 3;</code>
     */
    com.ib.client.protobuf.ExecutionProto.ExecutionOrBuilder getExecutionOrBuilder();
  }
  /**
   * Protobuf type {@code protobuf.ExecutionDetails}
   */
  public static final class ExecutionDetails extends
      com.google.protobuf.GeneratedMessage implements
      // @@protoc_insertion_point(message_implements:protobuf.ExecutionDetails)
      ExecutionDetailsOrBuilder {
  private static final long serialVersionUID = 0L;
    static {
      com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
        com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
        /* major= */ 4,
        /* minor= */ 29,
        /* patch= */ 3,
        /* suffix= */ "",
        ExecutionDetails.class.getName());
    }
    // Use ExecutionDetails.newBuilder() to construct.
    private ExecutionDetails(com.google.protobuf.GeneratedMessage.Builder<?> builder) {
      super(builder);
    }
    private ExecutionDetails() {
    }

    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return com.ib.client.protobuf.ExecutionDetailsProto.internal_static_protobuf_ExecutionDetails_descriptor;
    }

    @java.lang.Override
    protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return com.ib.client.protobuf.ExecutionDetailsProto.internal_static_protobuf_ExecutionDetails_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails.class, com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails.Builder.class);
    }

    private int bitField0_;
    public static final int REQID_FIELD_NUMBER = 1;
    private int reqId_ = 0;
    /**
     * <code>optional int32 reqId = 1;</code>
     * @return Whether the reqId field is set.
     */
    @java.lang.Override
    public boolean hasReqId() {
      return ((bitField0_ & 0x00000001) != 0);
    }
    /**
     * <code>optional int32 reqId = 1;</code>
     * @return The reqId.
     */
    @java.lang.Override
    public int getReqId() {
      return reqId_;
    }

    public static final int CONTRACT_FIELD_NUMBER = 2;
    private com.ib.client.protobuf.ContractProto.Contract contract_;
    /**
     * <code>optional .protobuf.Contract contract = 2;</code>
     * @return Whether the contract field is set.
     */
    @java.lang.Override
    public boolean hasContract() {
      return ((bitField0_ & 0x00000002) != 0);
    }
    /**
     * <code>optional .protobuf.Contract contract = 2;</code>
     * @return The contract.
     */
    @java.lang.Override
    public com.ib.client.protobuf.ContractProto.Contract getContract() {
      return contract_ == null ? com.ib.client.protobuf.ContractProto.Contract.getDefaultInstance() : contract_;
    }
    /**
     * <code>optional .protobuf.Contract contract = 2;</code>
     */
    @java.lang.Override
    public com.ib.client.protobuf.ContractProto.ContractOrBuilder getContractOrBuilder() {
      return contract_ == null ? com.ib.client.protobuf.ContractProto.Contract.getDefaultInstance() : contract_;
    }

    public static final int EXECUTION_FIELD_NUMBER = 3;
    private com.ib.client.protobuf.ExecutionProto.Execution execution_;
    /**
     * <code>optional .protobuf.Execution execution = 3;</code>
     * @return Whether the execution field is set.
     */
    @java.lang.Override
    public boolean hasExecution() {
      return ((bitField0_ & 0x00000004) != 0);
    }
    /**
     * <code>optional .protobuf.Execution execution = 3;</code>
     * @return The execution.
     */
    @java.lang.Override
    public com.ib.client.protobuf.ExecutionProto.Execution getExecution() {
      return execution_ == null ? com.ib.client.protobuf.ExecutionProto.Execution.getDefaultInstance() : execution_;
    }
    /**
     * <code>optional .protobuf.Execution execution = 3;</code>
     */
    @java.lang.Override
    public com.ib.client.protobuf.ExecutionProto.ExecutionOrBuilder getExecutionOrBuilder() {
      return execution_ == null ? com.ib.client.protobuf.ExecutionProto.Execution.getDefaultInstance() : execution_;
    }

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      if (((bitField0_ & 0x00000001) != 0)) {
        output.writeInt32(1, reqId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        output.writeMessage(2, getContract());
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        output.writeMessage(3, getExecution());
      }
      getUnknownFields().writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      if (((bitField0_ & 0x00000001) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(1, reqId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeMessageSize(2, getContract());
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeMessageSize(3, getExecution());
      }
      size += getUnknownFields().getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails)) {
        return super.equals(obj);
      }
      com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails other = (com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails) obj;

      if (hasReqId() != other.hasReqId()) return false;
      if (hasReqId()) {
        if (getReqId()
            != other.getReqId()) return false;
      }
      if (hasContract() != other.hasContract()) return false;
      if (hasContract()) {
        if (!getContract()
            .equals(other.getContract())) return false;
      }
      if (hasExecution() != other.hasExecution()) return false;
      if (hasExecution()) {
        if (!getExecution()
            .equals(other.getExecution())) return false;
      }
      if (!getUnknownFields().equals(other.getUnknownFields())) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      if (hasReqId()) {
        hash = (37 * hash) + REQID_FIELD_NUMBER;
        hash = (53 * hash) + getReqId();
      }
      if (hasContract()) {
        hash = (37 * hash) + CONTRACT_FIELD_NUMBER;
        hash = (53 * hash) + getContract().hashCode();
      }
      if (hasExecution()) {
        hash = (37 * hash) + EXECUTION_FIELD_NUMBER;
        hash = (53 * hash) + getExecution().hashCode();
      }
      hash = (29 * hash) + getUnknownFields().hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input);
    }

    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessage.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code protobuf.ExecutionDetails}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessage.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:protobuf.ExecutionDetails)
        com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetailsOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return com.ib.client.protobuf.ExecutionDetailsProto.internal_static_protobuf_ExecutionDetails_descriptor;
      }

      @java.lang.Override
      protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return com.ib.client.protobuf.ExecutionDetailsProto.internal_static_protobuf_ExecutionDetails_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails.class, com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails.Builder.class);
      }

      // Construct using com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails.newBuilder()
      private Builder() {
        maybeForceBuilderInitialization();
      }

      private Builder(
          com.google.protobuf.GeneratedMessage.BuilderParent parent) {
        super(parent);
        maybeForceBuilderInitialization();
      }
      private void maybeForceBuilderInitialization() {
        if (com.google.protobuf.GeneratedMessage
                .alwaysUseFieldBuilders) {
          getContractFieldBuilder();
          getExecutionFieldBuilder();
        }
      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        bitField0_ = 0;
        reqId_ = 0;
        contract_ = null;
        if (contractBuilder_ != null) {
          contractBuilder_.dispose();
          contractBuilder_ = null;
        }
        execution_ = null;
        if (executionBuilder_ != null) {
          executionBuilder_.dispose();
          executionBuilder_ = null;
        }
        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return com.ib.client.protobuf.ExecutionDetailsProto.internal_static_protobuf_ExecutionDetails_descriptor;
      }

      @java.lang.Override
      public com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails getDefaultInstanceForType() {
        return com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails.getDefaultInstance();
      }

      @java.lang.Override
      public com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails build() {
        com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails buildPartial() {
        com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails result = new com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails(this);
        if (bitField0_ != 0) { buildPartial0(result); }
        onBuilt();
        return result;
      }

      private void buildPartial0(com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails result) {
        int from_bitField0_ = bitField0_;
        int to_bitField0_ = 0;
        if (((from_bitField0_ & 0x00000001) != 0)) {
          result.reqId_ = reqId_;
          to_bitField0_ |= 0x00000001;
        }
        if (((from_bitField0_ & 0x00000002) != 0)) {
          result.contract_ = contractBuilder_ == null
              ? contract_
              : contractBuilder_.build();
          to_bitField0_ |= 0x00000002;
        }
        if (((from_bitField0_ & 0x00000004) != 0)) {
          result.execution_ = executionBuilder_ == null
              ? execution_
              : executionBuilder_.build();
          to_bitField0_ |= 0x00000004;
        }
        result.bitField0_ |= to_bitField0_;
      }

      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails) {
          return mergeFrom((com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails other) {
        if (other == com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails.getDefaultInstance()) return this;
        if (other.hasReqId()) {
          setReqId(other.getReqId());
        }
        if (other.hasContract()) {
          mergeContract(other.getContract());
        }
        if (other.hasExecution()) {
          mergeExecution(other.getExecution());
        }
        this.mergeUnknownFields(other.getUnknownFields());
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        if (extensionRegistry == null) {
          throw new java.lang.NullPointerException();
        }
        try {
          boolean done = false;
          while (!done) {
            int tag = input.readTag();
            switch (tag) {
              case 0:
                done = true;
                break;
              case 8: {
                reqId_ = input.readInt32();
                bitField0_ |= 0x00000001;
                break;
              } // case 8
              case 18: {
                input.readMessage(
                    getContractFieldBuilder().getBuilder(),
                    extensionRegistry);
                bitField0_ |= 0x00000002;
                break;
              } // case 18
              case 26: {
                input.readMessage(
                    getExecutionFieldBuilder().getBuilder(),
                    extensionRegistry);
                bitField0_ |= 0x00000004;
                break;
              } // case 26
              default: {
                if (!super.parseUnknownField(input, extensionRegistry, tag)) {
                  done = true; // was an endgroup tag
                }
                break;
              } // default:
            } // switch (tag)
          } // while (!done)
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.unwrapIOException();
        } finally {
          onChanged();
        } // finally
        return this;
      }
      private int bitField0_;

      private int reqId_ ;
      /**
       * <code>optional int32 reqId = 1;</code>
       * @return Whether the reqId field is set.
       */
      @java.lang.Override
      public boolean hasReqId() {
        return ((bitField0_ & 0x00000001) != 0);
      }
      /**
       * <code>optional int32 reqId = 1;</code>
       * @return The reqId.
       */
      @java.lang.Override
      public int getReqId() {
        return reqId_;
      }
      /**
       * <code>optional int32 reqId = 1;</code>
       * @param value The reqId to set.
       * @return This builder for chaining.
       */
      public Builder setReqId(int value) {

        reqId_ = value;
        bitField0_ |= 0x00000001;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 reqId = 1;</code>
       * @return This builder for chaining.
       */
      public Builder clearReqId() {
        bitField0_ = (bitField0_ & ~0x00000001);
        reqId_ = 0;
        onChanged();
        return this;
      }

      private com.ib.client.protobuf.ContractProto.Contract contract_;
      private com.google.protobuf.SingleFieldBuilder<
          com.ib.client.protobuf.ContractProto.Contract, com.ib.client.protobuf.ContractProto.Contract.Builder, com.ib.client.protobuf.ContractProto.ContractOrBuilder> contractBuilder_;
      /**
       * <code>optional .protobuf.Contract contract = 2;</code>
       * @return Whether the contract field is set.
       */
      public boolean hasContract() {
        return ((bitField0_ & 0x00000002) != 0);
      }
      /**
       * <code>optional .protobuf.Contract contract = 2;</code>
       * @return The contract.
       */
      public com.ib.client.protobuf.ContractProto.Contract getContract() {
        if (contractBuilder_ == null) {
          return contract_ == null ? com.ib.client.protobuf.ContractProto.Contract.getDefaultInstance() : contract_;
        } else {
          return contractBuilder_.getMessage();
        }
      }
      /**
       * <code>optional .protobuf.Contract contract = 2;</code>
       */
      public Builder setContract(com.ib.client.protobuf.ContractProto.Contract value) {
        if (contractBuilder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          contract_ = value;
        } else {
          contractBuilder_.setMessage(value);
        }
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }
      /**
       * <code>optional .protobuf.Contract contract = 2;</code>
       */
      public Builder setContract(
          com.ib.client.protobuf.ContractProto.Contract.Builder builderForValue) {
        if (contractBuilder_ == null) {
          contract_ = builderForValue.build();
        } else {
          contractBuilder_.setMessage(builderForValue.build());
        }
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }
      /**
       * <code>optional .protobuf.Contract contract = 2;</code>
       */
      public Builder mergeContract(com.ib.client.protobuf.ContractProto.Contract value) {
        if (contractBuilder_ == null) {
          if (((bitField0_ & 0x00000002) != 0) &&
            contract_ != null &&
            contract_ != com.ib.client.protobuf.ContractProto.Contract.getDefaultInstance()) {
            getContractBuilder().mergeFrom(value);
          } else {
            contract_ = value;
          }
        } else {
          contractBuilder_.mergeFrom(value);
        }
        if (contract_ != null) {
          bitField0_ |= 0x00000002;
          onChanged();
        }
        return this;
      }
      /**
       * <code>optional .protobuf.Contract contract = 2;</code>
       */
      public Builder clearContract() {
        bitField0_ = (bitField0_ & ~0x00000002);
        contract_ = null;
        if (contractBuilder_ != null) {
          contractBuilder_.dispose();
          contractBuilder_ = null;
        }
        onChanged();
        return this;
      }
      /**
       * <code>optional .protobuf.Contract contract = 2;</code>
       */
      public com.ib.client.protobuf.ContractProto.Contract.Builder getContractBuilder() {
        bitField0_ |= 0x00000002;
        onChanged();
        return getContractFieldBuilder().getBuilder();
      }
      /**
       * <code>optional .protobuf.Contract contract = 2;</code>
       */
      public com.ib.client.protobuf.ContractProto.ContractOrBuilder getContractOrBuilder() {
        if (contractBuilder_ != null) {
          return contractBuilder_.getMessageOrBuilder();
        } else {
          return contract_ == null ?
              com.ib.client.protobuf.ContractProto.Contract.getDefaultInstance() : contract_;
        }
      }
      /**
       * <code>optional .protobuf.Contract contract = 2;</code>
       */
      private com.google.protobuf.SingleFieldBuilder<
          com.ib.client.protobuf.ContractProto.Contract, com.ib.client.protobuf.ContractProto.Contract.Builder, com.ib.client.protobuf.ContractProto.ContractOrBuilder> 
          getContractFieldBuilder() {
        if (contractBuilder_ == null) {
          contractBuilder_ = new com.google.protobuf.SingleFieldBuilder<
              com.ib.client.protobuf.ContractProto.Contract, com.ib.client.protobuf.ContractProto.Contract.Builder, com.ib.client.protobuf.ContractProto.ContractOrBuilder>(
                  getContract(),
                  getParentForChildren(),
                  isClean());
          contract_ = null;
        }
        return contractBuilder_;
      }

      private com.ib.client.protobuf.ExecutionProto.Execution execution_;
      private com.google.protobuf.SingleFieldBuilder<
          com.ib.client.protobuf.ExecutionProto.Execution, com.ib.client.protobuf.ExecutionProto.Execution.Builder, com.ib.client.protobuf.ExecutionProto.ExecutionOrBuilder> executionBuilder_;
      /**
       * <code>optional .protobuf.Execution execution = 3;</code>
       * @return Whether the execution field is set.
       */
      public boolean hasExecution() {
        return ((bitField0_ & 0x00000004) != 0);
      }
      /**
       * <code>optional .protobuf.Execution execution = 3;</code>
       * @return The execution.
       */
      public com.ib.client.protobuf.ExecutionProto.Execution getExecution() {
        if (executionBuilder_ == null) {
          return execution_ == null ? com.ib.client.protobuf.ExecutionProto.Execution.getDefaultInstance() : execution_;
        } else {
          return executionBuilder_.getMessage();
        }
      }
      /**
       * <code>optional .protobuf.Execution execution = 3;</code>
       */
      public Builder setExecution(com.ib.client.protobuf.ExecutionProto.Execution value) {
        if (executionBuilder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          execution_ = value;
        } else {
          executionBuilder_.setMessage(value);
        }
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }
      /**
       * <code>optional .protobuf.Execution execution = 3;</code>
       */
      public Builder setExecution(
          com.ib.client.protobuf.ExecutionProto.Execution.Builder builderForValue) {
        if (executionBuilder_ == null) {
          execution_ = builderForValue.build();
        } else {
          executionBuilder_.setMessage(builderForValue.build());
        }
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }
      /**
       * <code>optional .protobuf.Execution execution = 3;</code>
       */
      public Builder mergeExecution(com.ib.client.protobuf.ExecutionProto.Execution value) {
        if (executionBuilder_ == null) {
          if (((bitField0_ & 0x00000004) != 0) &&
            execution_ != null &&
            execution_ != com.ib.client.protobuf.ExecutionProto.Execution.getDefaultInstance()) {
            getExecutionBuilder().mergeFrom(value);
          } else {
            execution_ = value;
          }
        } else {
          executionBuilder_.mergeFrom(value);
        }
        if (execution_ != null) {
          bitField0_ |= 0x00000004;
          onChanged();
        }
        return this;
      }
      /**
       * <code>optional .protobuf.Execution execution = 3;</code>
       */
      public Builder clearExecution() {
        bitField0_ = (bitField0_ & ~0x00000004);
        execution_ = null;
        if (executionBuilder_ != null) {
          executionBuilder_.dispose();
          executionBuilder_ = null;
        }
        onChanged();
        return this;
      }
      /**
       * <code>optional .protobuf.Execution execution = 3;</code>
       */
      public com.ib.client.protobuf.ExecutionProto.Execution.Builder getExecutionBuilder() {
        bitField0_ |= 0x00000004;
        onChanged();
        return getExecutionFieldBuilder().getBuilder();
      }
      /**
       * <code>optional .protobuf.Execution execution = 3;</code>
       */
      public com.ib.client.protobuf.ExecutionProto.ExecutionOrBuilder getExecutionOrBuilder() {
        if (executionBuilder_ != null) {
          return executionBuilder_.getMessageOrBuilder();
        } else {
          return execution_ == null ?
              com.ib.client.protobuf.ExecutionProto.Execution.getDefaultInstance() : execution_;
        }
      }
      /**
       * <code>optional .protobuf.Execution execution = 3;</code>
       */
      private com.google.protobuf.SingleFieldBuilder<
          com.ib.client.protobuf.ExecutionProto.Execution, com.ib.client.protobuf.ExecutionProto.Execution.Builder, com.ib.client.protobuf.ExecutionProto.ExecutionOrBuilder> 
          getExecutionFieldBuilder() {
        if (executionBuilder_ == null) {
          executionBuilder_ = new com.google.protobuf.SingleFieldBuilder<
              com.ib.client.protobuf.ExecutionProto.Execution, com.ib.client.protobuf.ExecutionProto.Execution.Builder, com.ib.client.protobuf.ExecutionProto.ExecutionOrBuilder>(
                  getExecution(),
                  getParentForChildren(),
                  isClean());
          execution_ = null;
        }
        return executionBuilder_;
      }

      // @@protoc_insertion_point(builder_scope:protobuf.ExecutionDetails)
    }

    // @@protoc_insertion_point(class_scope:protobuf.ExecutionDetails)
    private static final com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails();
    }

    public static com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<ExecutionDetails>
        PARSER = new com.google.protobuf.AbstractParser<ExecutionDetails>() {
      @java.lang.Override
      public ExecutionDetails parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        Builder builder = newBuilder();
        try {
          builder.mergeFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.setUnfinishedMessage(builder.buildPartial());
        } catch (com.google.protobuf.UninitializedMessageException e) {
          throw e.asInvalidProtocolBufferException().setUnfinishedMessage(builder.buildPartial());
        } catch (java.io.IOException e) {
          throw new com.google.protobuf.InvalidProtocolBufferException(e)
              .setUnfinishedMessage(builder.buildPartial());
        }
        return builder.buildPartial();
      }
    };

    public static com.google.protobuf.Parser<ExecutionDetails> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<ExecutionDetails> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public com.ib.client.protobuf.ExecutionDetailsProto.ExecutionDetails getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_protobuf_ExecutionDetails_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessage.FieldAccessorTable
      internal_static_protobuf_ExecutionDetails_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor
      getDescriptor() {
    return descriptor;
  }
  private static  com.google.protobuf.Descriptors.FileDescriptor
      descriptor;
  static {
    java.lang.String[] descriptorData = {
      "\n\026ExecutionDetails.proto\022\010protobuf\032\016Cont" +
      "ract.proto\032\017Execution.proto\"\243\001\n\020Executio" +
      "nDetails\022\022\n\005reqId\030\001 \001(\005H\000\210\001\001\022)\n\010contract" +
      "\030\002 \001(\0132\022.protobuf.ContractH\001\210\001\001\022+\n\texecu" +
      "tion\030\003 \001(\0132\023.protobuf.ExecutionH\002\210\001\001B\010\n\006" +
      "_reqIdB\013\n\t_contractB\014\n\n_executionB@\n\026com" +
      ".ib.client.protobufB\025ExecutionDetailsPro" +
      "to\252\002\016IBApi.protobufb\006proto3"
    };
    descriptor = com.google.protobuf.Descriptors.FileDescriptor
      .internalBuildGeneratedFileFrom(descriptorData,
        new com.google.protobuf.Descriptors.FileDescriptor[] {
          com.ib.client.protobuf.ContractProto.getDescriptor(),
          com.ib.client.protobuf.ExecutionProto.getDescriptor(),
        });
    internal_static_protobuf_ExecutionDetails_descriptor =
      getDescriptor().getMessageTypes().get(0);
    internal_static_protobuf_ExecutionDetails_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessage.FieldAccessorTable(
        internal_static_protobuf_ExecutionDetails_descriptor,
        new java.lang.String[] { "ReqId", "Contract", "Execution", });
    descriptor.resolveAllFeaturesImmutable();
    com.ib.client.protobuf.ContractProto.getDescriptor();
    com.ib.client.protobuf.ExecutionProto.getDescriptor();
  }

  // @@protoc_insertion_point(outer_class_scope)
}
