# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: Order.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'Order.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ibapi.protobuf.OrderCondition_pb2 as OrderCondition__pb2
import ibapi.protobuf.SoftDollarTier_pb2 as SoftDollarTier__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0bOrder.proto\x12\x08protobuf\x1a\x14OrderCondition.proto\x1a\x14SoftDollarTier.proto\"\xbf\x36\n\x05Order\x12\x15\n\x08\x63lientId\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12\x14\n\x07orderId\x18\x02 \x01(\x05H\x01\x88\x01\x01\x12\x13\n\x06permId\x18\x03 \x01(\x03H\x02\x88\x01\x01\x12\x15\n\x08parentId\x18\x04 \x01(\x05H\x03\x88\x01\x01\x12\x13\n\x06\x61\x63tion\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x1a\n\rtotalQuantity\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x18\n\x0b\x64isplaySize\x18\x07 \x01(\x05H\x06\x88\x01\x01\x12\x16\n\torderType\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x15\n\x08lmtPrice\x18\t \x01(\x01H\x08\x88\x01\x01\x12\x15\n\x08\x61uxPrice\x18\n \x01(\x01H\t\x88\x01\x01\x12\x10\n\x03tif\x18\x0b \x01(\tH\n\x88\x01\x01\x12\x14\n\x07\x61\x63\x63ount\x18\x0c \x01(\tH\x0b\x88\x01\x01\x12\x19\n\x0csettlingFirm\x18\r \x01(\tH\x0c\x88\x01\x01\x12\x1c\n\x0f\x63learingAccount\x18\x0e \x01(\tH\r\x88\x01\x01\x12\x1b\n\x0e\x63learingIntent\x18\x0f \x01(\tH\x0e\x88\x01\x01\x12\x16\n\tallOrNone\x18\x10 \x01(\x08H\x0f\x88\x01\x01\x12\x17\n\nblockOrder\x18\x11 \x01(\x08H\x10\x88\x01\x01\x12\x13\n\x06hidden\x18\x12 \x01(\x08H\x11\x88\x01\x01\x12\x17\n\noutsideRth\x18\x13 \x01(\x08H\x12\x88\x01\x01\x12\x18\n\x0bsweepToFill\x18\x14 \x01(\x08H\x13\x88\x01\x01\x12\x1a\n\rpercentOffset\x18\x15 \x01(\x01H\x14\x88\x01\x01\x12\x1c\n\x0ftrailingPercent\x18\x16 \x01(\x01H\x15\x88\x01\x01\x12\x1b\n\x0etrailStopPrice\x18\x17 \x01(\x01H\x16\x88\x01\x01\x12\x13\n\x06minQty\x18\x18 \x01(\x05H\x17\x88\x01\x01\x12\x1a\n\rgoodAfterTime\x18\x19 \x01(\tH\x18\x88\x01\x01\x12\x19\n\x0cgoodTillDate\x18\x1a \x01(\tH\x19\x88\x01\x01\x12\x15\n\x08ocaGroup\x18\x1b \x01(\tH\x1a\x88\x01\x01\x12\x15\n\x08orderRef\x18\x1c \x01(\tH\x1b\x88\x01\x01\x12\x14\n\x07rule80A\x18\x1d \x01(\tH\x1c\x88\x01\x01\x12\x14\n\x07ocaType\x18\x1e \x01(\x05H\x1d\x88\x01\x01\x12\x1a\n\rtriggerMethod\x18\x1f \x01(\x05H\x1e\x88\x01\x01\x12\x1c\n\x0f\x61\x63tiveStartTime\x18  \x01(\tH\x1f\x88\x01\x01\x12\x1b\n\x0e\x61\x63tiveStopTime\x18! \x01(\tH \x88\x01\x01\x12\x14\n\x07\x66\x61Group\x18\" \x01(\tH!\x88\x01\x01\x12\x15\n\x08\x66\x61Method\x18# \x01(\tH\"\x88\x01\x01\x12\x19\n\x0c\x66\x61Percentage\x18$ \x01(\tH#\x88\x01\x01\x12\x17\n\nvolatility\x18% \x01(\x01H$\x88\x01\x01\x12\x1b\n\x0evolatilityType\x18& \x01(\x05H%\x88\x01\x01\x12\x1d\n\x10\x63ontinuousUpdate\x18\' \x01(\x08H&\x88\x01\x01\x12\x1f\n\x12referencePriceType\x18( \x01(\x05H\'\x88\x01\x01\x12\"\n\x15\x64\x65ltaNeutralOrderType\x18) \x01(\tH(\x88\x01\x01\x12!\n\x14\x64\x65ltaNeutralAuxPrice\x18* \x01(\x01H)\x88\x01\x01\x12\x1e\n\x11\x64\x65ltaNeutralConId\x18+ \x01(\x05H*\x88\x01\x01\x12\"\n\x15\x64\x65ltaNeutralOpenClose\x18, \x01(\tH+\x88\x01\x01\x12\"\n\x15\x64\x65ltaNeutralShortSale\x18- \x01(\x08H,\x88\x01\x01\x12&\n\x19\x64\x65ltaNeutralShortSaleSlot\x18. \x01(\x05H-\x88\x01\x01\x12+\n\x1e\x64\x65ltaNeutralDesignatedLocation\x18/ \x01(\tH.\x88\x01\x01\x12\x1f\n\x12scaleInitLevelSize\x18\x30 \x01(\x05H/\x88\x01\x01\x12\x1f\n\x12scaleSubsLevelSize\x18\x31 \x01(\x05H0\x88\x01\x01\x12 \n\x13scalePriceIncrement\x18\x32 \x01(\x01H1\x88\x01\x01\x12\"\n\x15scalePriceAdjustValue\x18\x33 \x01(\x01H2\x88\x01\x01\x12%\n\x18scalePriceAdjustInterval\x18\x34 \x01(\x05H3\x88\x01\x01\x12\x1e\n\x11scaleProfitOffset\x18\x35 \x01(\x01H4\x88\x01\x01\x12\x1b\n\x0escaleAutoReset\x18\x36 \x01(\x08H5\x88\x01\x01\x12\x1e\n\x11scaleInitPosition\x18\x37 \x01(\x05H6\x88\x01\x01\x12\x1d\n\x10scaleInitFillQty\x18\x38 \x01(\x05H7\x88\x01\x01\x12\x1f\n\x12scaleRandomPercent\x18\x39 \x01(\x08H8\x88\x01\x01\x12\x17\n\nscaleTable\x18: \x01(\tH9\x88\x01\x01\x12\x16\n\thedgeType\x18; \x01(\tH:\x88\x01\x01\x12\x17\n\nhedgeParam\x18< \x01(\tH;\x88\x01\x01\x12\x19\n\x0c\x61lgoStrategy\x18= \x01(\tH<\x88\x01\x01\x12\x33\n\nalgoParams\x18> \x03(\x0b\x32\x1f.protobuf.Order.AlgoParamsEntry\x12\x13\n\x06\x61lgoId\x18? \x01(\tH=\x88\x01\x01\x12M\n\x17smartComboRoutingParams\x18@ \x03(\x0b\x32,.protobuf.Order.SmartComboRoutingParamsEntry\x12\x13\n\x06whatIf\x18\x41 \x01(\x08H>\x88\x01\x01\x12\x15\n\x08transmit\x18\x42 \x01(\x08H?\x88\x01\x01\x12*\n\x1doverridePercentageConstraints\x18\x43 \x01(\x08H@\x88\x01\x01\x12\x16\n\topenClose\x18\x44 \x01(\tHA\x88\x01\x01\x12\x13\n\x06origin\x18\x45 \x01(\x05HB\x88\x01\x01\x12\x1a\n\rshortSaleSlot\x18\x46 \x01(\x05HC\x88\x01\x01\x12\x1f\n\x12\x64\x65signatedLocation\x18G \x01(\tHD\x88\x01\x01\x12\x17\n\nexemptCode\x18H \x01(\x05HE\x88\x01\x01\x12%\n\x18\x64\x65ltaNeutralSettlingFirm\x18I \x01(\tHF\x88\x01\x01\x12(\n\x1b\x64\x65ltaNeutralClearingAccount\x18J \x01(\tHG\x88\x01\x01\x12\'\n\x1a\x64\x65ltaNeutralClearingIntent\x18K \x01(\tHH\x88\x01\x01\x12\x1d\n\x10\x64iscretionaryAmt\x18L \x01(\x01HI\x88\x01\x01\x12\x1f\n\x12optOutSmartRouting\x18M \x01(\x08HJ\x88\x01\x01\x12\x1a\n\rstartingPrice\x18N \x01(\x01HK\x88\x01\x01\x12\x1a\n\rstockRefPrice\x18O \x01(\x01HL\x88\x01\x01\x12\x12\n\x05\x64\x65lta\x18P \x01(\x01HM\x88\x01\x01\x12\x1c\n\x0fstockRangeLower\x18Q \x01(\x01HN\x88\x01\x01\x12\x1c\n\x0fstockRangeUpper\x18R \x01(\x01HO\x88\x01\x01\x12\x14\n\x07notHeld\x18S \x01(\x08HP\x88\x01\x01\x12?\n\x10orderMiscOptions\x18T \x03(\x0b\x32%.protobuf.Order.OrderMiscOptionsEntry\x12\x16\n\tsolicited\x18U \x01(\x08HQ\x88\x01\x01\x12\x1a\n\rrandomizeSize\x18V \x01(\x08HR\x88\x01\x01\x12\x1b\n\x0erandomizePrice\x18W \x01(\x08HS\x88\x01\x01\x12 \n\x13referenceContractId\x18X \x01(\x05HT\x88\x01\x01\x12\x1f\n\x12peggedChangeAmount\x18Y \x01(\x01HU\x88\x01\x01\x12)\n\x1cisPeggedChangeAmountDecrease\x18Z \x01(\x08HV\x88\x01\x01\x12\"\n\x15referenceChangeAmount\x18[ \x01(\x01HW\x88\x01\x01\x12 \n\x13referenceExchangeId\x18\\ \x01(\tHX\x88\x01\x01\x12\x1e\n\x11\x61\x64justedOrderType\x18] \x01(\tHY\x88\x01\x01\x12\x19\n\x0ctriggerPrice\x18^ \x01(\x01HZ\x88\x01\x01\x12\x1e\n\x11\x61\x64justedStopPrice\x18_ \x01(\x01H[\x88\x01\x01\x12#\n\x16\x61\x64justedStopLimitPrice\x18` \x01(\x01H\\\x88\x01\x01\x12#\n\x16\x61\x64justedTrailingAmount\x18\x61 \x01(\x01H]\x88\x01\x01\x12#\n\x16\x61\x64justableTrailingUnit\x18\x62 \x01(\x05H^\x88\x01\x01\x12\x1b\n\x0elmtPriceOffset\x18\x63 \x01(\x01H_\x88\x01\x01\x12,\n\nconditions\x18\x64 \x03(\x0b\x32\x18.protobuf.OrderCondition\x12\"\n\x15\x63onditionsCancelOrder\x18\x65 \x01(\x08H`\x88\x01\x01\x12 \n\x13\x63onditionsIgnoreRth\x18\x66 \x01(\x08Ha\x88\x01\x01\x12\x16\n\tmodelCode\x18g \x01(\tHb\x88\x01\x01\x12\x18\n\x0b\x65xtOperator\x18h \x01(\tHc\x88\x01\x01\x12\x35\n\x0esoftDollarTier\x18i \x01(\x0b\x32\x18.protobuf.SoftDollarTierHd\x88\x01\x01\x12\x14\n\x07\x63\x61shQty\x18j \x01(\x01He\x88\x01\x01\x12 \n\x13mifid2DecisionMaker\x18k \x01(\tHf\x88\x01\x01\x12\x1f\n\x12mifid2DecisionAlgo\x18l \x01(\tHg\x88\x01\x01\x12\"\n\x15mifid2ExecutionTrader\x18m \x01(\tHh\x88\x01\x01\x12 \n\x13mifid2ExecutionAlgo\x18n \x01(\tHi\x88\x01\x01\x12%\n\x18\x64ontUseAutoPriceForHedge\x18o \x01(\x08Hj\x88\x01\x01\x12\x1b\n\x0eisOmsContainer\x18p \x01(\x08Hk\x88\x01\x01\x12(\n\x1b\x64iscretionaryUpToLimitPrice\x18q \x01(\x08Hl\x88\x01\x01\x12\x1b\n\x0e\x61utoCancelDate\x18r \x01(\tHm\x88\x01\x01\x12\x1b\n\x0e\x66illedQuantity\x18s \x01(\tHn\x88\x01\x01\x12\x1c\n\x0frefFuturesConId\x18t \x01(\x05Ho\x88\x01\x01\x12\x1d\n\x10\x61utoCancelParent\x18u \x01(\x08Hp\x88\x01\x01\x12\x18\n\x0bshareholder\x18v \x01(\tHq\x88\x01\x01\x12\x1a\n\rimbalanceOnly\x18w \x01(\x08Hr\x88\x01\x01\x12!\n\x14routeMarketableToBbo\x18x \x01(\x08Hs\x88\x01\x01\x12\x19\n\x0cparentPermId\x18y \x01(\x03Ht\x88\x01\x01\x12\x1d\n\x10usePriceMgmtAlgo\x18z \x01(\x05Hu\x88\x01\x01\x12\x15\n\x08\x64uration\x18{ \x01(\x05Hv\x88\x01\x01\x12\x16\n\tpostToAts\x18| \x01(\x05Hw\x88\x01\x01\x12\"\n\x15\x61\x64vancedErrorOverride\x18} \x01(\tHx\x88\x01\x01\x12\x1c\n\x0fmanualOrderTime\x18~ \x01(\tHy\x88\x01\x01\x12\x18\n\x0bminTradeQty\x18\x7f \x01(\x05Hz\x88\x01\x01\x12\x1c\n\x0eminCompeteSize\x18\x80\x01 \x01(\x05H{\x88\x01\x01\x12&\n\x18\x63ompeteAgainstBestOffset\x18\x81\x01 \x01(\x01H|\x88\x01\x01\x12\x1e\n\x10midOffsetAtWhole\x18\x82\x01 \x01(\x01H}\x88\x01\x01\x12\x1d\n\x0fmidOffsetAtHalf\x18\x83\x01 \x01(\x01H~\x88\x01\x01\x12\x1d\n\x0f\x63ustomerAccount\x18\x84\x01 \x01(\tH\x7f\x88\x01\x01\x12#\n\x14professionalCustomer\x18\x85\x01 \x01(\x08H\x80\x01\x88\x01\x01\x12\"\n\x13\x62ondAccruedInterest\x18\x86\x01 \x01(\tH\x81\x01\x88\x01\x01\x12\x1f\n\x10includeOvernight\x18\x87\x01 \x01(\x08H\x82\x01\x88\x01\x01\x12#\n\x14manualOrderIndicator\x18\x88\x01 \x01(\x05H\x83\x01\x88\x01\x01\x12\x18\n\tsubmitter\x18\x89\x01 \x01(\tH\x84\x01\x88\x01\x01\x1a\x31\n\x0f\x41lgoParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a>\n\x1cSmartComboRoutingParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x37\n\x15OrderMiscOptionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x0b\n\t_clientIdB\n\n\x08_orderIdB\t\n\x07_permIdB\x0b\n\t_parentIdB\t\n\x07_actionB\x10\n\x0e_totalQuantityB\x0e\n\x0c_displaySizeB\x0c\n\n_orderTypeB\x0b\n\t_lmtPriceB\x0b\n\t_auxPriceB\x06\n\x04_tifB\n\n\x08_accountB\x0f\n\r_settlingFirmB\x12\n\x10_clearingAccountB\x11\n\x0f_clearingIntentB\x0c\n\n_allOrNoneB\r\n\x0b_blockOrderB\t\n\x07_hiddenB\r\n\x0b_outsideRthB\x0e\n\x0c_sweepToFillB\x10\n\x0e_percentOffsetB\x12\n\x10_trailingPercentB\x11\n\x0f_trailStopPriceB\t\n\x07_minQtyB\x10\n\x0e_goodAfterTimeB\x0f\n\r_goodTillDateB\x0b\n\t_ocaGroupB\x0b\n\t_orderRefB\n\n\x08_rule80AB\n\n\x08_ocaTypeB\x10\n\x0e_triggerMethodB\x12\n\x10_activeStartTimeB\x11\n\x0f_activeStopTimeB\n\n\x08_faGroupB\x0b\n\t_faMethodB\x0f\n\r_faPercentageB\r\n\x0b_volatilityB\x11\n\x0f_volatilityTypeB\x13\n\x11_continuousUpdateB\x15\n\x13_referencePriceTypeB\x18\n\x16_deltaNeutralOrderTypeB\x17\n\x15_deltaNeutralAuxPriceB\x14\n\x12_deltaNeutralConIdB\x18\n\x16_deltaNeutralOpenCloseB\x18\n\x16_deltaNeutralShortSaleB\x1c\n\x1a_deltaNeutralShortSaleSlotB!\n\x1f_deltaNeutralDesignatedLocationB\x15\n\x13_scaleInitLevelSizeB\x15\n\x13_scaleSubsLevelSizeB\x16\n\x14_scalePriceIncrementB\x18\n\x16_scalePriceAdjustValueB\x1b\n\x19_scalePriceAdjustIntervalB\x14\n\x12_scaleProfitOffsetB\x11\n\x0f_scaleAutoResetB\x14\n\x12_scaleInitPositionB\x13\n\x11_scaleInitFillQtyB\x15\n\x13_scaleRandomPercentB\r\n\x0b_scaleTableB\x0c\n\n_hedgeTypeB\r\n\x0b_hedgeParamB\x0f\n\r_algoStrategyB\t\n\x07_algoIdB\t\n\x07_whatIfB\x0b\n\t_transmitB \n\x1e_overridePercentageConstraintsB\x0c\n\n_openCloseB\t\n\x07_originB\x10\n\x0e_shortSaleSlotB\x15\n\x13_designatedLocationB\r\n\x0b_exemptCodeB\x1b\n\x19_deltaNeutralSettlingFirmB\x1e\n\x1c_deltaNeutralClearingAccountB\x1d\n\x1b_deltaNeutralClearingIntentB\x13\n\x11_discretionaryAmtB\x15\n\x13_optOutSmartRoutingB\x10\n\x0e_startingPriceB\x10\n\x0e_stockRefPriceB\x08\n\x06_deltaB\x12\n\x10_stockRangeLowerB\x12\n\x10_stockRangeUpperB\n\n\x08_notHeldB\x0c\n\n_solicitedB\x10\n\x0e_randomizeSizeB\x11\n\x0f_randomizePriceB\x16\n\x14_referenceContractIdB\x15\n\x13_peggedChangeAmountB\x1f\n\x1d_isPeggedChangeAmountDecreaseB\x18\n\x16_referenceChangeAmountB\x16\n\x14_referenceExchangeIdB\x14\n\x12_adjustedOrderTypeB\x0f\n\r_triggerPriceB\x14\n\x12_adjustedStopPriceB\x19\n\x17_adjustedStopLimitPriceB\x19\n\x17_adjustedTrailingAmountB\x19\n\x17_adjustableTrailingUnitB\x11\n\x0f_lmtPriceOffsetB\x18\n\x16_conditionsCancelOrderB\x16\n\x14_conditionsIgnoreRthB\x0c\n\n_modelCodeB\x0e\n\x0c_extOperatorB\x11\n\x0f_softDollarTierB\n\n\x08_cashQtyB\x16\n\x14_mifid2DecisionMakerB\x15\n\x13_mifid2DecisionAlgoB\x18\n\x16_mifid2ExecutionTraderB\x16\n\x14_mifid2ExecutionAlgoB\x1b\n\x19_dontUseAutoPriceForHedgeB\x11\n\x0f_isOmsContainerB\x1e\n\x1c_discretionaryUpToLimitPriceB\x11\n\x0f_autoCancelDateB\x11\n\x0f_filledQuantityB\x12\n\x10_refFuturesConIdB\x13\n\x11_autoCancelParentB\x0e\n\x0c_shareholderB\x10\n\x0e_imbalanceOnlyB\x17\n\x15_routeMarketableToBboB\x0f\n\r_parentPermIdB\x13\n\x11_usePriceMgmtAlgoB\x0b\n\t_durationB\x0c\n\n_postToAtsB\x18\n\x16_advancedErrorOverrideB\x12\n\x10_manualOrderTimeB\x0e\n\x0c_minTradeQtyB\x11\n\x0f_minCompeteSizeB\x1b\n\x19_competeAgainstBestOffsetB\x13\n\x11_midOffsetAtWholeB\x12\n\x10_midOffsetAtHalfB\x12\n\x10_customerAccountB\x17\n\x15_professionalCustomerB\x16\n\x14_bondAccruedInterestB\x13\n\x11_includeOvernightB\x17\n\x15_manualOrderIndicatorB\x0c\n\n_submitterB5\n\x16\x63om.ib.client.protobufB\nOrderProto\xaa\x02\x0eIBApi.protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'Order_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.ib.client.protobufB\nOrderProto\252\002\016IBApi.protobuf'
  _globals['_ORDER_ALGOPARAMSENTRY']._loaded_options = None
  _globals['_ORDER_ALGOPARAMSENTRY']._serialized_options = b'8\001'
  _globals['_ORDER_SMARTCOMBOROUTINGPARAMSENTRY']._loaded_options = None
  _globals['_ORDER_SMARTCOMBOROUTINGPARAMSENTRY']._serialized_options = b'8\001'
  _globals['_ORDER_ORDERMISCOPTIONSENTRY']._loaded_options = None
  _globals['_ORDER_ORDERMISCOPTIONSENTRY']._serialized_options = b'8\001'
  _globals['_ORDER']._serialized_start=70
  _globals['_ORDER']._serialized_end=7045
  _globals['_ORDER_ALGOPARAMSENTRY']._serialized_start=4284
  _globals['_ORDER_ALGOPARAMSENTRY']._serialized_end=4333
  _globals['_ORDER_SMARTCOMBOROUTINGPARAMSENTRY']._serialized_start=4335
  _globals['_ORDER_SMARTCOMBOROUTINGPARAMSENTRY']._serialized_end=4397
  _globals['_ORDER_ORDERMISCOPTIONSENTRY']._serialized_start=4399
  _globals['_ORDER_ORDERMISCOPTIONSENTRY']._serialized_end=4454
# @@protoc_insertion_point(module_scope)
