# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: OrderStatus.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'OrderStatus.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11OrderStatus.proto\x12\x08protobuf\"\xa3\x03\n\x0bOrderStatus\x12\x14\n\x07orderId\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12\x13\n\x06status\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x13\n\x06\x66illed\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x16\n\tremaining\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x19\n\x0c\x61vgFillPrice\x18\x05 \x01(\x01H\x04\x88\x01\x01\x12\x13\n\x06permId\x18\x06 \x01(\x03H\x05\x88\x01\x01\x12\x15\n\x08parentId\x18\x07 \x01(\x05H\x06\x88\x01\x01\x12\x1a\n\rlastFillPrice\x18\x08 \x01(\x01H\x07\x88\x01\x01\x12\x15\n\x08\x63lientId\x18\t \x01(\x05H\x08\x88\x01\x01\x12\x14\n\x07whyHeld\x18\n \x01(\tH\t\x88\x01\x01\x12\x18\n\x0bmktCapPrice\x18\x0b \x01(\x01H\n\x88\x01\x01\x42\n\n\x08_orderIdB\t\n\x07_statusB\t\n\x07_filledB\x0c\n\n_remainingB\x0f\n\r_avgFillPriceB\t\n\x07_permIdB\x0b\n\t_parentIdB\x10\n\x0e_lastFillPriceB\x0b\n\t_clientIdB\n\n\x08_whyHeldB\x0e\n\x0c_mktCapPriceB;\n\x16\x63om.ib.client.protobufB\x10OrderStatusProto\xaa\x02\x0eIBApi.protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'OrderStatus_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.ib.client.protobufB\020OrderStatusProto\252\002\016IBApi.protobuf'
  _globals['_ORDERSTATUS']._serialized_start=32
  _globals['_ORDERSTATUS']._serialized_end=451
# @@protoc_insertion_point(module_scope)
