/* Copyright (C) 2025 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 * and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable. */

syntax = "proto3";

package protobuf;

option java_outer_classname = "OrderAllocationProto";
option java_package = "com.ib.client.protobuf";
option csharp_namespace = "IBApi.protobuf";

message OrderAllocation {
    optional string account = 1;
    optional string position = 2;
    optional string positionDesired = 3;
    optional string positionAfter = 4;
    optional string desiredAllocQty = 5;
    optional string allowedAllocQty = 6;
    optional bool isMonetary = 7;
}
