/* Copyright (C) 2025 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 * and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable. */

syntax = "proto3";

package protobuf;

option java_outer_classname = "ExecutionProto";
option java_package = "com.ib.client.protobuf";
option csharp_namespace = "IBApi.protobuf";

message Execution {
    optional int32 orderId = 1;
    optional string execId = 2;
    optional string time = 3;
    optional string acctNumber = 4;
    optional string exchange = 5;
    optional string side = 6;
    optional string shares = 7;
    optional double price = 8;
    optional int64 permId = 9;
    optional int32 clientId = 10;
    optional bool isLiquidation = 11;
    optional string cumQty = 12;
    optional double avgPrice = 13;
    optional string orderRef = 14;
    optional string evRule = 15;
    optional double evMultiplier = 16;
    optional string modelCode = 17;
    optional int32 lastLiquidity = 18;
    optional bool isPriceRevisionPending = 19;
    optional string submitter = 20;
    optional int32 optExerciseOrLapseType = 21;
}
