# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: OrderAllocation.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'OrderAllocation.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15OrderAllocation.proto\x12\x08protobuf\"\xc3\x02\n\x0fOrderAllocation\x12\x14\n\x07\x61\x63\x63ount\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08position\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1c\n\x0fpositionDesired\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1a\n\rpositionAfter\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1c\n\x0f\x64\x65siredAllocQty\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x1c\n\x0f\x61llowedAllocQty\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x17\n\nisMonetary\x18\x07 \x01(\x08H\x06\x88\x01\x01\x42\n\n\x08_accountB\x0b\n\t_positionB\x12\n\x10_positionDesiredB\x10\n\x0e_positionAfterB\x12\n\x10_desiredAllocQtyB\x12\n\x10_allowedAllocQtyB\r\n\x0b_isMonetaryB?\n\x16\x63om.ib.client.protobufB\x14OrderAllocationProto\xaa\x02\x0eIBApi.protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'OrderAllocation_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.ib.client.protobufB\024OrderAllocationProto\252\002\016IBApi.protobuf'
  _globals['_ORDERALLOCATION']._serialized_start=36
  _globals['_ORDERALLOCATION']._serialized_end=359
# @@protoc_insertion_point(module_scope)
