// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: DeltaNeutralContract.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_DeltaNeutralContract_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_DeltaNeutralContract_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3012000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3012004 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_table_driven.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/inlined_string_field.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_DeltaNeutralContract_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_DeltaNeutralContract_2eproto {
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTableField entries[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::AuxillaryParseTableField aux[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTable schema[1]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::FieldMetadata field_metadata[];
  static const ::PROTOBUF_NAMESPACE_ID::internal::SerializationTable serialization_table[];
  static const ::PROTOBUF_NAMESPACE_ID::uint32 offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_DeltaNeutralContract_2eproto;
namespace protobuf {
class DeltaNeutralContract;
class DeltaNeutralContractDefaultTypeInternal;
extern DeltaNeutralContractDefaultTypeInternal _DeltaNeutralContract_default_instance_;
}  // namespace protobuf
PROTOBUF_NAMESPACE_OPEN
template<> ::protobuf::DeltaNeutralContract* Arena::CreateMaybeMessage<::protobuf::DeltaNeutralContract>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace protobuf {

// ===================================================================

class DeltaNeutralContract PROTOBUF_FINAL :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:protobuf.DeltaNeutralContract) */ {
 public:
  inline DeltaNeutralContract() : DeltaNeutralContract(nullptr) {};
  virtual ~DeltaNeutralContract();

  DeltaNeutralContract(const DeltaNeutralContract& from);
  DeltaNeutralContract(DeltaNeutralContract&& from) noexcept
    : DeltaNeutralContract() {
    *this = ::std::move(from);
  }

  inline DeltaNeutralContract& operator=(const DeltaNeutralContract& from) {
    CopyFrom(from);
    return *this;
  }
  inline DeltaNeutralContract& operator=(DeltaNeutralContract&& from) noexcept {
    if (GetArena() == from.GetArena()) {
      if (this != &from) InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return GetMetadataStatic().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return GetMetadataStatic().reflection;
  }
  static const DeltaNeutralContract& default_instance();

  static void InitAsDefaultInstance();  // FOR INTERNAL USE ONLY
  static inline const DeltaNeutralContract* internal_default_instance() {
    return reinterpret_cast<const DeltaNeutralContract*>(
               &_DeltaNeutralContract_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(DeltaNeutralContract& a, DeltaNeutralContract& b) {
    a.Swap(&b);
  }
  inline void Swap(DeltaNeutralContract* other) {
    if (other == this) return;
    if (GetArena() == other->GetArena()) {
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DeltaNeutralContract* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  inline DeltaNeutralContract* New() const final {
    return CreateMaybeMessage<DeltaNeutralContract>(nullptr);
  }

  DeltaNeutralContract* New(::PROTOBUF_NAMESPACE_ID::Arena* arena) const final {
    return CreateMaybeMessage<DeltaNeutralContract>(arena);
  }
  void CopyFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void MergeFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void CopyFrom(const DeltaNeutralContract& from);
  void MergeFrom(const DeltaNeutralContract& from);
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  ::PROTOBUF_NAMESPACE_ID::uint8* _InternalSerialize(
      ::PROTOBUF_NAMESPACE_ID::uint8* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  inline void SharedCtor();
  inline void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(DeltaNeutralContract* other);
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "protobuf.DeltaNeutralContract";
  }
  protected:
  explicit DeltaNeutralContract(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  private:
  static void ArenaDtor(void* object);
  inline void RegisterArenaDtor(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  public:

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;
  private:
  static ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadataStatic() {
    ::PROTOBUF_NAMESPACE_ID::internal::AssignDescriptors(&::descriptor_table_DeltaNeutralContract_2eproto);
    return ::descriptor_table_DeltaNeutralContract_2eproto.file_level_metadata[kIndexInFileMessages];
  }

  public:

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kDeltaFieldNumber = 2,
    kPriceFieldNumber = 3,
    kConIdFieldNumber = 1,
  };
  // double delta = 2;
  bool has_delta() const;
  private:
  bool _internal_has_delta() const;
  public:
  void clear_delta();
  double delta() const;
  void set_delta(double value);
  private:
  double _internal_delta() const;
  void _internal_set_delta(double value);
  public:

  // double price = 3;
  bool has_price() const;
  private:
  bool _internal_has_price() const;
  public:
  void clear_price();
  double price() const;
  void set_price(double value);
  private:
  double _internal_price() const;
  void _internal_set_price(double value);
  public:

  // int32 conId = 1;
  bool has_conid() const;
  private:
  bool _internal_has_conid() const;
  public:
  void clear_conid();
  ::PROTOBUF_NAMESPACE_ID::int32 conid() const;
  void set_conid(::PROTOBUF_NAMESPACE_ID::int32 value);
  private:
  ::PROTOBUF_NAMESPACE_ID::int32 _internal_conid() const;
  void _internal_set_conid(::PROTOBUF_NAMESPACE_ID::int32 value);
  public:

  // @@protoc_insertion_point(class_scope:protobuf.DeltaNeutralContract)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  double delta_;
  double price_;
  ::PROTOBUF_NAMESPACE_ID::int32 conid_;
  friend struct ::TableStruct_DeltaNeutralContract_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// DeltaNeutralContract

// int32 conId = 1;
inline bool DeltaNeutralContract::_internal_has_conid() const {
  bool value = (_has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool DeltaNeutralContract::has_conid() const {
  return _internal_has_conid();
}
inline void DeltaNeutralContract::clear_conid() {
  conid_ = 0;
  _has_bits_[0] &= ~0x00000004u;
}
inline ::PROTOBUF_NAMESPACE_ID::int32 DeltaNeutralContract::_internal_conid() const {
  return conid_;
}
inline ::PROTOBUF_NAMESPACE_ID::int32 DeltaNeutralContract::conid() const {
  // @@protoc_insertion_point(field_get:protobuf.DeltaNeutralContract.conId)
  return _internal_conid();
}
inline void DeltaNeutralContract::_internal_set_conid(::PROTOBUF_NAMESPACE_ID::int32 value) {
  _has_bits_[0] |= 0x00000004u;
  conid_ = value;
}
inline void DeltaNeutralContract::set_conid(::PROTOBUF_NAMESPACE_ID::int32 value) {
  _internal_set_conid(value);
  // @@protoc_insertion_point(field_set:protobuf.DeltaNeutralContract.conId)
}

// double delta = 2;
inline bool DeltaNeutralContract::_internal_has_delta() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool DeltaNeutralContract::has_delta() const {
  return _internal_has_delta();
}
inline void DeltaNeutralContract::clear_delta() {
  delta_ = 0;
  _has_bits_[0] &= ~0x00000001u;
}
inline double DeltaNeutralContract::_internal_delta() const {
  return delta_;
}
inline double DeltaNeutralContract::delta() const {
  // @@protoc_insertion_point(field_get:protobuf.DeltaNeutralContract.delta)
  return _internal_delta();
}
inline void DeltaNeutralContract::_internal_set_delta(double value) {
  _has_bits_[0] |= 0x00000001u;
  delta_ = value;
}
inline void DeltaNeutralContract::set_delta(double value) {
  _internal_set_delta(value);
  // @@protoc_insertion_point(field_set:protobuf.DeltaNeutralContract.delta)
}

// double price = 3;
inline bool DeltaNeutralContract::_internal_has_price() const {
  bool value = (_has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool DeltaNeutralContract::has_price() const {
  return _internal_has_price();
}
inline void DeltaNeutralContract::clear_price() {
  price_ = 0;
  _has_bits_[0] &= ~0x00000002u;
}
inline double DeltaNeutralContract::_internal_price() const {
  return price_;
}
inline double DeltaNeutralContract::price() const {
  // @@protoc_insertion_point(field_get:protobuf.DeltaNeutralContract.price)
  return _internal_price();
}
inline void DeltaNeutralContract::_internal_set_price(double value) {
  _has_bits_[0] |= 0x00000002u;
  price_ = value;
}
inline void DeltaNeutralContract::set_price(double value) {
  _internal_set_price(value);
  // @@protoc_insertion_point(field_set:protobuf.DeltaNeutralContract.price)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)

}  // namespace protobuf

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_DeltaNeutralContract_2eproto
