// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: OrderStatus.proto
// Protobuf Java Version: 4.29.3

package com.ib.client.protobuf;

public final class OrderStatusProto {
  private OrderStatusProto() {}
  static {
    com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
      com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
      /* major= */ 4,
      /* minor= */ 29,
      /* patch= */ 3,
      /* suffix= */ "",
      OrderStatusProto.class.getName());
  }
  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistryLite registry) {
  }

  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistry registry) {
    registerAllExtensions(
        (com.google.protobuf.ExtensionRegistryLite) registry);
  }
  public interface OrderStatusOrBuilder extends
      // @@protoc_insertion_point(interface_extends:protobuf.OrderStatus)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>optional int32 orderId = 1;</code>
     * @return Whether the orderId field is set.
     */
    boolean hasOrderId();
    /**
     * <code>optional int32 orderId = 1;</code>
     * @return The orderId.
     */
    int getOrderId();

    /**
     * <code>optional string status = 2;</code>
     * @return Whether the status field is set.
     */
    boolean hasStatus();
    /**
     * <code>optional string status = 2;</code>
     * @return The status.
     */
    java.lang.String getStatus();
    /**
     * <code>optional string status = 2;</code>
     * @return The bytes for status.
     */
    com.google.protobuf.ByteString
        getStatusBytes();

    /**
     * <code>optional string filled = 3;</code>
     * @return Whether the filled field is set.
     */
    boolean hasFilled();
    /**
     * <code>optional string filled = 3;</code>
     * @return The filled.
     */
    java.lang.String getFilled();
    /**
     * <code>optional string filled = 3;</code>
     * @return The bytes for filled.
     */
    com.google.protobuf.ByteString
        getFilledBytes();

    /**
     * <code>optional string remaining = 4;</code>
     * @return Whether the remaining field is set.
     */
    boolean hasRemaining();
    /**
     * <code>optional string remaining = 4;</code>
     * @return The remaining.
     */
    java.lang.String getRemaining();
    /**
     * <code>optional string remaining = 4;</code>
     * @return The bytes for remaining.
     */
    com.google.protobuf.ByteString
        getRemainingBytes();

    /**
     * <code>optional double avgFillPrice = 5;</code>
     * @return Whether the avgFillPrice field is set.
     */
    boolean hasAvgFillPrice();
    /**
     * <code>optional double avgFillPrice = 5;</code>
     * @return The avgFillPrice.
     */
    double getAvgFillPrice();

    /**
     * <code>optional int64 permId = 6;</code>
     * @return Whether the permId field is set.
     */
    boolean hasPermId();
    /**
     * <code>optional int64 permId = 6;</code>
     * @return The permId.
     */
    long getPermId();

    /**
     * <code>optional int32 parentId = 7;</code>
     * @return Whether the parentId field is set.
     */
    boolean hasParentId();
    /**
     * <code>optional int32 parentId = 7;</code>
     * @return The parentId.
     */
    int getParentId();

    /**
     * <code>optional double lastFillPrice = 8;</code>
     * @return Whether the lastFillPrice field is set.
     */
    boolean hasLastFillPrice();
    /**
     * <code>optional double lastFillPrice = 8;</code>
     * @return The lastFillPrice.
     */
    double getLastFillPrice();

    /**
     * <code>optional int32 clientId = 9;</code>
     * @return Whether the clientId field is set.
     */
    boolean hasClientId();
    /**
     * <code>optional int32 clientId = 9;</code>
     * @return The clientId.
     */
    int getClientId();

    /**
     * <code>optional string whyHeld = 10;</code>
     * @return Whether the whyHeld field is set.
     */
    boolean hasWhyHeld();
    /**
     * <code>optional string whyHeld = 10;</code>
     * @return The whyHeld.
     */
    java.lang.String getWhyHeld();
    /**
     * <code>optional string whyHeld = 10;</code>
     * @return The bytes for whyHeld.
     */
    com.google.protobuf.ByteString
        getWhyHeldBytes();

    /**
     * <code>optional double mktCapPrice = 11;</code>
     * @return Whether the mktCapPrice field is set.
     */
    boolean hasMktCapPrice();
    /**
     * <code>optional double mktCapPrice = 11;</code>
     * @return The mktCapPrice.
     */
    double getMktCapPrice();
  }
  /**
   * Protobuf type {@code protobuf.OrderStatus}
   */
  public static final class OrderStatus extends
      com.google.protobuf.GeneratedMessage implements
      // @@protoc_insertion_point(message_implements:protobuf.OrderStatus)
      OrderStatusOrBuilder {
  private static final long serialVersionUID = 0L;
    static {
      com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
        com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
        /* major= */ 4,
        /* minor= */ 29,
        /* patch= */ 3,
        /* suffix= */ "",
        OrderStatus.class.getName());
    }
    // Use OrderStatus.newBuilder() to construct.
    private OrderStatus(com.google.protobuf.GeneratedMessage.Builder<?> builder) {
      super(builder);
    }
    private OrderStatus() {
      status_ = "";
      filled_ = "";
      remaining_ = "";
      whyHeld_ = "";
    }

    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return com.ib.client.protobuf.OrderStatusProto.internal_static_protobuf_OrderStatus_descriptor;
    }

    @java.lang.Override
    protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return com.ib.client.protobuf.OrderStatusProto.internal_static_protobuf_OrderStatus_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              com.ib.client.protobuf.OrderStatusProto.OrderStatus.class, com.ib.client.protobuf.OrderStatusProto.OrderStatus.Builder.class);
    }

    private int bitField0_;
    public static final int ORDERID_FIELD_NUMBER = 1;
    private int orderId_ = 0;
    /**
     * <code>optional int32 orderId = 1;</code>
     * @return Whether the orderId field is set.
     */
    @java.lang.Override
    public boolean hasOrderId() {
      return ((bitField0_ & 0x00000001) != 0);
    }
    /**
     * <code>optional int32 orderId = 1;</code>
     * @return The orderId.
     */
    @java.lang.Override
    public int getOrderId() {
      return orderId_;
    }

    public static final int STATUS_FIELD_NUMBER = 2;
    @SuppressWarnings("serial")
    private volatile java.lang.Object status_ = "";
    /**
     * <code>optional string status = 2;</code>
     * @return Whether the status field is set.
     */
    @java.lang.Override
    public boolean hasStatus() {
      return ((bitField0_ & 0x00000002) != 0);
    }
    /**
     * <code>optional string status = 2;</code>
     * @return The status.
     */
    @java.lang.Override
    public java.lang.String getStatus() {
      java.lang.Object ref = status_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        status_ = s;
        return s;
      }
    }
    /**
     * <code>optional string status = 2;</code>
     * @return The bytes for status.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getStatusBytes() {
      java.lang.Object ref = status_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        status_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int FILLED_FIELD_NUMBER = 3;
    @SuppressWarnings("serial")
    private volatile java.lang.Object filled_ = "";
    /**
     * <code>optional string filled = 3;</code>
     * @return Whether the filled field is set.
     */
    @java.lang.Override
    public boolean hasFilled() {
      return ((bitField0_ & 0x00000004) != 0);
    }
    /**
     * <code>optional string filled = 3;</code>
     * @return The filled.
     */
    @java.lang.Override
    public java.lang.String getFilled() {
      java.lang.Object ref = filled_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        filled_ = s;
        return s;
      }
    }
    /**
     * <code>optional string filled = 3;</code>
     * @return The bytes for filled.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getFilledBytes() {
      java.lang.Object ref = filled_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        filled_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int REMAINING_FIELD_NUMBER = 4;
    @SuppressWarnings("serial")
    private volatile java.lang.Object remaining_ = "";
    /**
     * <code>optional string remaining = 4;</code>
     * @return Whether the remaining field is set.
     */
    @java.lang.Override
    public boolean hasRemaining() {
      return ((bitField0_ & 0x00000008) != 0);
    }
    /**
     * <code>optional string remaining = 4;</code>
     * @return The remaining.
     */
    @java.lang.Override
    public java.lang.String getRemaining() {
      java.lang.Object ref = remaining_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        remaining_ = s;
        return s;
      }
    }
    /**
     * <code>optional string remaining = 4;</code>
     * @return The bytes for remaining.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getRemainingBytes() {
      java.lang.Object ref = remaining_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        remaining_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int AVGFILLPRICE_FIELD_NUMBER = 5;
    private double avgFillPrice_ = 0D;
    /**
     * <code>optional double avgFillPrice = 5;</code>
     * @return Whether the avgFillPrice field is set.
     */
    @java.lang.Override
    public boolean hasAvgFillPrice() {
      return ((bitField0_ & 0x00000010) != 0);
    }
    /**
     * <code>optional double avgFillPrice = 5;</code>
     * @return The avgFillPrice.
     */
    @java.lang.Override
    public double getAvgFillPrice() {
      return avgFillPrice_;
    }

    public static final int PERMID_FIELD_NUMBER = 6;
    private long permId_ = 0L;
    /**
     * <code>optional int64 permId = 6;</code>
     * @return Whether the permId field is set.
     */
    @java.lang.Override
    public boolean hasPermId() {
      return ((bitField0_ & 0x00000020) != 0);
    }
    /**
     * <code>optional int64 permId = 6;</code>
     * @return The permId.
     */
    @java.lang.Override
    public long getPermId() {
      return permId_;
    }

    public static final int PARENTID_FIELD_NUMBER = 7;
    private int parentId_ = 0;
    /**
     * <code>optional int32 parentId = 7;</code>
     * @return Whether the parentId field is set.
     */
    @java.lang.Override
    public boolean hasParentId() {
      return ((bitField0_ & 0x00000040) != 0);
    }
    /**
     * <code>optional int32 parentId = 7;</code>
     * @return The parentId.
     */
    @java.lang.Override
    public int getParentId() {
      return parentId_;
    }

    public static final int LASTFILLPRICE_FIELD_NUMBER = 8;
    private double lastFillPrice_ = 0D;
    /**
     * <code>optional double lastFillPrice = 8;</code>
     * @return Whether the lastFillPrice field is set.
     */
    @java.lang.Override
    public boolean hasLastFillPrice() {
      return ((bitField0_ & 0x00000080) != 0);
    }
    /**
     * <code>optional double lastFillPrice = 8;</code>
     * @return The lastFillPrice.
     */
    @java.lang.Override
    public double getLastFillPrice() {
      return lastFillPrice_;
    }

    public static final int CLIENTID_FIELD_NUMBER = 9;
    private int clientId_ = 0;
    /**
     * <code>optional int32 clientId = 9;</code>
     * @return Whether the clientId field is set.
     */
    @java.lang.Override
    public boolean hasClientId() {
      return ((bitField0_ & 0x00000100) != 0);
    }
    /**
     * <code>optional int32 clientId = 9;</code>
     * @return The clientId.
     */
    @java.lang.Override
    public int getClientId() {
      return clientId_;
    }

    public static final int WHYHELD_FIELD_NUMBER = 10;
    @SuppressWarnings("serial")
    private volatile java.lang.Object whyHeld_ = "";
    /**
     * <code>optional string whyHeld = 10;</code>
     * @return Whether the whyHeld field is set.
     */
    @java.lang.Override
    public boolean hasWhyHeld() {
      return ((bitField0_ & 0x00000200) != 0);
    }
    /**
     * <code>optional string whyHeld = 10;</code>
     * @return The whyHeld.
     */
    @java.lang.Override
    public java.lang.String getWhyHeld() {
      java.lang.Object ref = whyHeld_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        whyHeld_ = s;
        return s;
      }
    }
    /**
     * <code>optional string whyHeld = 10;</code>
     * @return The bytes for whyHeld.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getWhyHeldBytes() {
      java.lang.Object ref = whyHeld_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        whyHeld_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int MKTCAPPRICE_FIELD_NUMBER = 11;
    private double mktCapPrice_ = 0D;
    /**
     * <code>optional double mktCapPrice = 11;</code>
     * @return Whether the mktCapPrice field is set.
     */
    @java.lang.Override
    public boolean hasMktCapPrice() {
      return ((bitField0_ & 0x00000400) != 0);
    }
    /**
     * <code>optional double mktCapPrice = 11;</code>
     * @return The mktCapPrice.
     */
    @java.lang.Override
    public double getMktCapPrice() {
      return mktCapPrice_;
    }

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      if (((bitField0_ & 0x00000001) != 0)) {
        output.writeInt32(1, orderId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 2, status_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 3, filled_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 4, remaining_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        output.writeDouble(5, avgFillPrice_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        output.writeInt64(6, permId_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        output.writeInt32(7, parentId_);
      }
      if (((bitField0_ & 0x00000080) != 0)) {
        output.writeDouble(8, lastFillPrice_);
      }
      if (((bitField0_ & 0x00000100) != 0)) {
        output.writeInt32(9, clientId_);
      }
      if (((bitField0_ & 0x00000200) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 10, whyHeld_);
      }
      if (((bitField0_ & 0x00000400) != 0)) {
        output.writeDouble(11, mktCapPrice_);
      }
      getUnknownFields().writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      if (((bitField0_ & 0x00000001) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(1, orderId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(2, status_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(3, filled_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(4, remaining_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeDoubleSize(5, avgFillPrice_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt64Size(6, permId_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(7, parentId_);
      }
      if (((bitField0_ & 0x00000080) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeDoubleSize(8, lastFillPrice_);
      }
      if (((bitField0_ & 0x00000100) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(9, clientId_);
      }
      if (((bitField0_ & 0x00000200) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(10, whyHeld_);
      }
      if (((bitField0_ & 0x00000400) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeDoubleSize(11, mktCapPrice_);
      }
      size += getUnknownFields().getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof com.ib.client.protobuf.OrderStatusProto.OrderStatus)) {
        return super.equals(obj);
      }
      com.ib.client.protobuf.OrderStatusProto.OrderStatus other = (com.ib.client.protobuf.OrderStatusProto.OrderStatus) obj;

      if (hasOrderId() != other.hasOrderId()) return false;
      if (hasOrderId()) {
        if (getOrderId()
            != other.getOrderId()) return false;
      }
      if (hasStatus() != other.hasStatus()) return false;
      if (hasStatus()) {
        if (!getStatus()
            .equals(other.getStatus())) return false;
      }
      if (hasFilled() != other.hasFilled()) return false;
      if (hasFilled()) {
        if (!getFilled()
            .equals(other.getFilled())) return false;
      }
      if (hasRemaining() != other.hasRemaining()) return false;
      if (hasRemaining()) {
        if (!getRemaining()
            .equals(other.getRemaining())) return false;
      }
      if (hasAvgFillPrice() != other.hasAvgFillPrice()) return false;
      if (hasAvgFillPrice()) {
        if (java.lang.Double.doubleToLongBits(getAvgFillPrice())
            != java.lang.Double.doubleToLongBits(
                other.getAvgFillPrice())) return false;
      }
      if (hasPermId() != other.hasPermId()) return false;
      if (hasPermId()) {
        if (getPermId()
            != other.getPermId()) return false;
      }
      if (hasParentId() != other.hasParentId()) return false;
      if (hasParentId()) {
        if (getParentId()
            != other.getParentId()) return false;
      }
      if (hasLastFillPrice() != other.hasLastFillPrice()) return false;
      if (hasLastFillPrice()) {
        if (java.lang.Double.doubleToLongBits(getLastFillPrice())
            != java.lang.Double.doubleToLongBits(
                other.getLastFillPrice())) return false;
      }
      if (hasClientId() != other.hasClientId()) return false;
      if (hasClientId()) {
        if (getClientId()
            != other.getClientId()) return false;
      }
      if (hasWhyHeld() != other.hasWhyHeld()) return false;
      if (hasWhyHeld()) {
        if (!getWhyHeld()
            .equals(other.getWhyHeld())) return false;
      }
      if (hasMktCapPrice() != other.hasMktCapPrice()) return false;
      if (hasMktCapPrice()) {
        if (java.lang.Double.doubleToLongBits(getMktCapPrice())
            != java.lang.Double.doubleToLongBits(
                other.getMktCapPrice())) return false;
      }
      if (!getUnknownFields().equals(other.getUnknownFields())) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      if (hasOrderId()) {
        hash = (37 * hash) + ORDERID_FIELD_NUMBER;
        hash = (53 * hash) + getOrderId();
      }
      if (hasStatus()) {
        hash = (37 * hash) + STATUS_FIELD_NUMBER;
        hash = (53 * hash) + getStatus().hashCode();
      }
      if (hasFilled()) {
        hash = (37 * hash) + FILLED_FIELD_NUMBER;
        hash = (53 * hash) + getFilled().hashCode();
      }
      if (hasRemaining()) {
        hash = (37 * hash) + REMAINING_FIELD_NUMBER;
        hash = (53 * hash) + getRemaining().hashCode();
      }
      if (hasAvgFillPrice()) {
        hash = (37 * hash) + AVGFILLPRICE_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            java.lang.Double.doubleToLongBits(getAvgFillPrice()));
      }
      if (hasPermId()) {
        hash = (37 * hash) + PERMID_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            getPermId());
      }
      if (hasParentId()) {
        hash = (37 * hash) + PARENTID_FIELD_NUMBER;
        hash = (53 * hash) + getParentId();
      }
      if (hasLastFillPrice()) {
        hash = (37 * hash) + LASTFILLPRICE_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            java.lang.Double.doubleToLongBits(getLastFillPrice()));
      }
      if (hasClientId()) {
        hash = (37 * hash) + CLIENTID_FIELD_NUMBER;
        hash = (53 * hash) + getClientId();
      }
      if (hasWhyHeld()) {
        hash = (37 * hash) + WHYHELD_FIELD_NUMBER;
        hash = (53 * hash) + getWhyHeld().hashCode();
      }
      if (hasMktCapPrice()) {
        hash = (37 * hash) + MKTCAPPRICE_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            java.lang.Double.doubleToLongBits(getMktCapPrice()));
      }
      hash = (29 * hash) + getUnknownFields().hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input);
    }

    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(com.ib.client.protobuf.OrderStatusProto.OrderStatus prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessage.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code protobuf.OrderStatus}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessage.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:protobuf.OrderStatus)
        com.ib.client.protobuf.OrderStatusProto.OrderStatusOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return com.ib.client.protobuf.OrderStatusProto.internal_static_protobuf_OrderStatus_descriptor;
      }

      @java.lang.Override
      protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return com.ib.client.protobuf.OrderStatusProto.internal_static_protobuf_OrderStatus_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                com.ib.client.protobuf.OrderStatusProto.OrderStatus.class, com.ib.client.protobuf.OrderStatusProto.OrderStatus.Builder.class);
      }

      // Construct using com.ib.client.protobuf.OrderStatusProto.OrderStatus.newBuilder()
      private Builder() {

      }

      private Builder(
          com.google.protobuf.GeneratedMessage.BuilderParent parent) {
        super(parent);

      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        bitField0_ = 0;
        orderId_ = 0;
        status_ = "";
        filled_ = "";
        remaining_ = "";
        avgFillPrice_ = 0D;
        permId_ = 0L;
        parentId_ = 0;
        lastFillPrice_ = 0D;
        clientId_ = 0;
        whyHeld_ = "";
        mktCapPrice_ = 0D;
        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return com.ib.client.protobuf.OrderStatusProto.internal_static_protobuf_OrderStatus_descriptor;
      }

      @java.lang.Override
      public com.ib.client.protobuf.OrderStatusProto.OrderStatus getDefaultInstanceForType() {
        return com.ib.client.protobuf.OrderStatusProto.OrderStatus.getDefaultInstance();
      }

      @java.lang.Override
      public com.ib.client.protobuf.OrderStatusProto.OrderStatus build() {
        com.ib.client.protobuf.OrderStatusProto.OrderStatus result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public com.ib.client.protobuf.OrderStatusProto.OrderStatus buildPartial() {
        com.ib.client.protobuf.OrderStatusProto.OrderStatus result = new com.ib.client.protobuf.OrderStatusProto.OrderStatus(this);
        if (bitField0_ != 0) { buildPartial0(result); }
        onBuilt();
        return result;
      }

      private void buildPartial0(com.ib.client.protobuf.OrderStatusProto.OrderStatus result) {
        int from_bitField0_ = bitField0_;
        int to_bitField0_ = 0;
        if (((from_bitField0_ & 0x00000001) != 0)) {
          result.orderId_ = orderId_;
          to_bitField0_ |= 0x00000001;
        }
        if (((from_bitField0_ & 0x00000002) != 0)) {
          result.status_ = status_;
          to_bitField0_ |= 0x00000002;
        }
        if (((from_bitField0_ & 0x00000004) != 0)) {
          result.filled_ = filled_;
          to_bitField0_ |= 0x00000004;
        }
        if (((from_bitField0_ & 0x00000008) != 0)) {
          result.remaining_ = remaining_;
          to_bitField0_ |= 0x00000008;
        }
        if (((from_bitField0_ & 0x00000010) != 0)) {
          result.avgFillPrice_ = avgFillPrice_;
          to_bitField0_ |= 0x00000010;
        }
        if (((from_bitField0_ & 0x00000020) != 0)) {
          result.permId_ = permId_;
          to_bitField0_ |= 0x00000020;
        }
        if (((from_bitField0_ & 0x00000040) != 0)) {
          result.parentId_ = parentId_;
          to_bitField0_ |= 0x00000040;
        }
        if (((from_bitField0_ & 0x00000080) != 0)) {
          result.lastFillPrice_ = lastFillPrice_;
          to_bitField0_ |= 0x00000080;
        }
        if (((from_bitField0_ & 0x00000100) != 0)) {
          result.clientId_ = clientId_;
          to_bitField0_ |= 0x00000100;
        }
        if (((from_bitField0_ & 0x00000200) != 0)) {
          result.whyHeld_ = whyHeld_;
          to_bitField0_ |= 0x00000200;
        }
        if (((from_bitField0_ & 0x00000400) != 0)) {
          result.mktCapPrice_ = mktCapPrice_;
          to_bitField0_ |= 0x00000400;
        }
        result.bitField0_ |= to_bitField0_;
      }

      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof com.ib.client.protobuf.OrderStatusProto.OrderStatus) {
          return mergeFrom((com.ib.client.protobuf.OrderStatusProto.OrderStatus)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(com.ib.client.protobuf.OrderStatusProto.OrderStatus other) {
        if (other == com.ib.client.protobuf.OrderStatusProto.OrderStatus.getDefaultInstance()) return this;
        if (other.hasOrderId()) {
          setOrderId(other.getOrderId());
        }
        if (other.hasStatus()) {
          status_ = other.status_;
          bitField0_ |= 0x00000002;
          onChanged();
        }
        if (other.hasFilled()) {
          filled_ = other.filled_;
          bitField0_ |= 0x00000004;
          onChanged();
        }
        if (other.hasRemaining()) {
          remaining_ = other.remaining_;
          bitField0_ |= 0x00000008;
          onChanged();
        }
        if (other.hasAvgFillPrice()) {
          setAvgFillPrice(other.getAvgFillPrice());
        }
        if (other.hasPermId()) {
          setPermId(other.getPermId());
        }
        if (other.hasParentId()) {
          setParentId(other.getParentId());
        }
        if (other.hasLastFillPrice()) {
          setLastFillPrice(other.getLastFillPrice());
        }
        if (other.hasClientId()) {
          setClientId(other.getClientId());
        }
        if (other.hasWhyHeld()) {
          whyHeld_ = other.whyHeld_;
          bitField0_ |= 0x00000200;
          onChanged();
        }
        if (other.hasMktCapPrice()) {
          setMktCapPrice(other.getMktCapPrice());
        }
        this.mergeUnknownFields(other.getUnknownFields());
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        if (extensionRegistry == null) {
          throw new java.lang.NullPointerException();
        }
        try {
          boolean done = false;
          while (!done) {
            int tag = input.readTag();
            switch (tag) {
              case 0:
                done = true;
                break;
              case 8: {
                orderId_ = input.readInt32();
                bitField0_ |= 0x00000001;
                break;
              } // case 8
              case 18: {
                status_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000002;
                break;
              } // case 18
              case 26: {
                filled_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000004;
                break;
              } // case 26
              case 34: {
                remaining_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000008;
                break;
              } // case 34
              case 41: {
                avgFillPrice_ = input.readDouble();
                bitField0_ |= 0x00000010;
                break;
              } // case 41
              case 48: {
                permId_ = input.readInt64();
                bitField0_ |= 0x00000020;
                break;
              } // case 48
              case 56: {
                parentId_ = input.readInt32();
                bitField0_ |= 0x00000040;
                break;
              } // case 56
              case 65: {
                lastFillPrice_ = input.readDouble();
                bitField0_ |= 0x00000080;
                break;
              } // case 65
              case 72: {
                clientId_ = input.readInt32();
                bitField0_ |= 0x00000100;
                break;
              } // case 72
              case 82: {
                whyHeld_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000200;
                break;
              } // case 82
              case 89: {
                mktCapPrice_ = input.readDouble();
                bitField0_ |= 0x00000400;
                break;
              } // case 89
              default: {
                if (!super.parseUnknownField(input, extensionRegistry, tag)) {
                  done = true; // was an endgroup tag
                }
                break;
              } // default:
            } // switch (tag)
          } // while (!done)
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.unwrapIOException();
        } finally {
          onChanged();
        } // finally
        return this;
      }
      private int bitField0_;

      private int orderId_ ;
      /**
       * <code>optional int32 orderId = 1;</code>
       * @return Whether the orderId field is set.
       */
      @java.lang.Override
      public boolean hasOrderId() {
        return ((bitField0_ & 0x00000001) != 0);
      }
      /**
       * <code>optional int32 orderId = 1;</code>
       * @return The orderId.
       */
      @java.lang.Override
      public int getOrderId() {
        return orderId_;
      }
      /**
       * <code>optional int32 orderId = 1;</code>
       * @param value The orderId to set.
       * @return This builder for chaining.
       */
      public Builder setOrderId(int value) {

        orderId_ = value;
        bitField0_ |= 0x00000001;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 orderId = 1;</code>
       * @return This builder for chaining.
       */
      public Builder clearOrderId() {
        bitField0_ = (bitField0_ & ~0x00000001);
        orderId_ = 0;
        onChanged();
        return this;
      }

      private java.lang.Object status_ = "";
      /**
       * <code>optional string status = 2;</code>
       * @return Whether the status field is set.
       */
      public boolean hasStatus() {
        return ((bitField0_ & 0x00000002) != 0);
      }
      /**
       * <code>optional string status = 2;</code>
       * @return The status.
       */
      public java.lang.String getStatus() {
        java.lang.Object ref = status_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          status_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string status = 2;</code>
       * @return The bytes for status.
       */
      public com.google.protobuf.ByteString
          getStatusBytes() {
        java.lang.Object ref = status_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          status_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string status = 2;</code>
       * @param value The status to set.
       * @return This builder for chaining.
       */
      public Builder setStatus(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        status_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }
      /**
       * <code>optional string status = 2;</code>
       * @return This builder for chaining.
       */
      public Builder clearStatus() {
        status_ = getDefaultInstance().getStatus();
        bitField0_ = (bitField0_ & ~0x00000002);
        onChanged();
        return this;
      }
      /**
       * <code>optional string status = 2;</code>
       * @param value The bytes for status to set.
       * @return This builder for chaining.
       */
      public Builder setStatusBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        status_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }

      private java.lang.Object filled_ = "";
      /**
       * <code>optional string filled = 3;</code>
       * @return Whether the filled field is set.
       */
      public boolean hasFilled() {
        return ((bitField0_ & 0x00000004) != 0);
      }
      /**
       * <code>optional string filled = 3;</code>
       * @return The filled.
       */
      public java.lang.String getFilled() {
        java.lang.Object ref = filled_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          filled_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string filled = 3;</code>
       * @return The bytes for filled.
       */
      public com.google.protobuf.ByteString
          getFilledBytes() {
        java.lang.Object ref = filled_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          filled_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string filled = 3;</code>
       * @param value The filled to set.
       * @return This builder for chaining.
       */
      public Builder setFilled(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        filled_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }
      /**
       * <code>optional string filled = 3;</code>
       * @return This builder for chaining.
       */
      public Builder clearFilled() {
        filled_ = getDefaultInstance().getFilled();
        bitField0_ = (bitField0_ & ~0x00000004);
        onChanged();
        return this;
      }
      /**
       * <code>optional string filled = 3;</code>
       * @param value The bytes for filled to set.
       * @return This builder for chaining.
       */
      public Builder setFilledBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        filled_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }

      private java.lang.Object remaining_ = "";
      /**
       * <code>optional string remaining = 4;</code>
       * @return Whether the remaining field is set.
       */
      public boolean hasRemaining() {
        return ((bitField0_ & 0x00000008) != 0);
      }
      /**
       * <code>optional string remaining = 4;</code>
       * @return The remaining.
       */
      public java.lang.String getRemaining() {
        java.lang.Object ref = remaining_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          remaining_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string remaining = 4;</code>
       * @return The bytes for remaining.
       */
      public com.google.protobuf.ByteString
          getRemainingBytes() {
        java.lang.Object ref = remaining_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          remaining_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string remaining = 4;</code>
       * @param value The remaining to set.
       * @return This builder for chaining.
       */
      public Builder setRemaining(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        remaining_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }
      /**
       * <code>optional string remaining = 4;</code>
       * @return This builder for chaining.
       */
      public Builder clearRemaining() {
        remaining_ = getDefaultInstance().getRemaining();
        bitField0_ = (bitField0_ & ~0x00000008);
        onChanged();
        return this;
      }
      /**
       * <code>optional string remaining = 4;</code>
       * @param value The bytes for remaining to set.
       * @return This builder for chaining.
       */
      public Builder setRemainingBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        remaining_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }

      private double avgFillPrice_ ;
      /**
       * <code>optional double avgFillPrice = 5;</code>
       * @return Whether the avgFillPrice field is set.
       */
      @java.lang.Override
      public boolean hasAvgFillPrice() {
        return ((bitField0_ & 0x00000010) != 0);
      }
      /**
       * <code>optional double avgFillPrice = 5;</code>
       * @return The avgFillPrice.
       */
      @java.lang.Override
      public double getAvgFillPrice() {
        return avgFillPrice_;
      }
      /**
       * <code>optional double avgFillPrice = 5;</code>
       * @param value The avgFillPrice to set.
       * @return This builder for chaining.
       */
      public Builder setAvgFillPrice(double value) {

        avgFillPrice_ = value;
        bitField0_ |= 0x00000010;
        onChanged();
        return this;
      }
      /**
       * <code>optional double avgFillPrice = 5;</code>
       * @return This builder for chaining.
       */
      public Builder clearAvgFillPrice() {
        bitField0_ = (bitField0_ & ~0x00000010);
        avgFillPrice_ = 0D;
        onChanged();
        return this;
      }

      private long permId_ ;
      /**
       * <code>optional int64 permId = 6;</code>
       * @return Whether the permId field is set.
       */
      @java.lang.Override
      public boolean hasPermId() {
        return ((bitField0_ & 0x00000020) != 0);
      }
      /**
       * <code>optional int64 permId = 6;</code>
       * @return The permId.
       */
      @java.lang.Override
      public long getPermId() {
        return permId_;
      }
      /**
       * <code>optional int64 permId = 6;</code>
       * @param value The permId to set.
       * @return This builder for chaining.
       */
      public Builder setPermId(long value) {

        permId_ = value;
        bitField0_ |= 0x00000020;
        onChanged();
        return this;
      }
      /**
       * <code>optional int64 permId = 6;</code>
       * @return This builder for chaining.
       */
      public Builder clearPermId() {
        bitField0_ = (bitField0_ & ~0x00000020);
        permId_ = 0L;
        onChanged();
        return this;
      }

      private int parentId_ ;
      /**
       * <code>optional int32 parentId = 7;</code>
       * @return Whether the parentId field is set.
       */
      @java.lang.Override
      public boolean hasParentId() {
        return ((bitField0_ & 0x00000040) != 0);
      }
      /**
       * <code>optional int32 parentId = 7;</code>
       * @return The parentId.
       */
      @java.lang.Override
      public int getParentId() {
        return parentId_;
      }
      /**
       * <code>optional int32 parentId = 7;</code>
       * @param value The parentId to set.
       * @return This builder for chaining.
       */
      public Builder setParentId(int value) {

        parentId_ = value;
        bitField0_ |= 0x00000040;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 parentId = 7;</code>
       * @return This builder for chaining.
       */
      public Builder clearParentId() {
        bitField0_ = (bitField0_ & ~0x00000040);
        parentId_ = 0;
        onChanged();
        return this;
      }

      private double lastFillPrice_ ;
      /**
       * <code>optional double lastFillPrice = 8;</code>
       * @return Whether the lastFillPrice field is set.
       */
      @java.lang.Override
      public boolean hasLastFillPrice() {
        return ((bitField0_ & 0x00000080) != 0);
      }
      /**
       * <code>optional double lastFillPrice = 8;</code>
       * @return The lastFillPrice.
       */
      @java.lang.Override
      public double getLastFillPrice() {
        return lastFillPrice_;
      }
      /**
       * <code>optional double lastFillPrice = 8;</code>
       * @param value The lastFillPrice to set.
       * @return This builder for chaining.
       */
      public Builder setLastFillPrice(double value) {

        lastFillPrice_ = value;
        bitField0_ |= 0x00000080;
        onChanged();
        return this;
      }
      /**
       * <code>optional double lastFillPrice = 8;</code>
       * @return This builder for chaining.
       */
      public Builder clearLastFillPrice() {
        bitField0_ = (bitField0_ & ~0x00000080);
        lastFillPrice_ = 0D;
        onChanged();
        return this;
      }

      private int clientId_ ;
      /**
       * <code>optional int32 clientId = 9;</code>
       * @return Whether the clientId field is set.
       */
      @java.lang.Override
      public boolean hasClientId() {
        return ((bitField0_ & 0x00000100) != 0);
      }
      /**
       * <code>optional int32 clientId = 9;</code>
       * @return The clientId.
       */
      @java.lang.Override
      public int getClientId() {
        return clientId_;
      }
      /**
       * <code>optional int32 clientId = 9;</code>
       * @param value The clientId to set.
       * @return This builder for chaining.
       */
      public Builder setClientId(int value) {

        clientId_ = value;
        bitField0_ |= 0x00000100;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 clientId = 9;</code>
       * @return This builder for chaining.
       */
      public Builder clearClientId() {
        bitField0_ = (bitField0_ & ~0x00000100);
        clientId_ = 0;
        onChanged();
        return this;
      }

      private java.lang.Object whyHeld_ = "";
      /**
       * <code>optional string whyHeld = 10;</code>
       * @return Whether the whyHeld field is set.
       */
      public boolean hasWhyHeld() {
        return ((bitField0_ & 0x00000200) != 0);
      }
      /**
       * <code>optional string whyHeld = 10;</code>
       * @return The whyHeld.
       */
      public java.lang.String getWhyHeld() {
        java.lang.Object ref = whyHeld_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          whyHeld_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string whyHeld = 10;</code>
       * @return The bytes for whyHeld.
       */
      public com.google.protobuf.ByteString
          getWhyHeldBytes() {
        java.lang.Object ref = whyHeld_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          whyHeld_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string whyHeld = 10;</code>
       * @param value The whyHeld to set.
       * @return This builder for chaining.
       */
      public Builder setWhyHeld(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        whyHeld_ = value;
        bitField0_ |= 0x00000200;
        onChanged();
        return this;
      }
      /**
       * <code>optional string whyHeld = 10;</code>
       * @return This builder for chaining.
       */
      public Builder clearWhyHeld() {
        whyHeld_ = getDefaultInstance().getWhyHeld();
        bitField0_ = (bitField0_ & ~0x00000200);
        onChanged();
        return this;
      }
      /**
       * <code>optional string whyHeld = 10;</code>
       * @param value The bytes for whyHeld to set.
       * @return This builder for chaining.
       */
      public Builder setWhyHeldBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        whyHeld_ = value;
        bitField0_ |= 0x00000200;
        onChanged();
        return this;
      }

      private double mktCapPrice_ ;
      /**
       * <code>optional double mktCapPrice = 11;</code>
       * @return Whether the mktCapPrice field is set.
       */
      @java.lang.Override
      public boolean hasMktCapPrice() {
        return ((bitField0_ & 0x00000400) != 0);
      }
      /**
       * <code>optional double mktCapPrice = 11;</code>
       * @return The mktCapPrice.
       */
      @java.lang.Override
      public double getMktCapPrice() {
        return mktCapPrice_;
      }
      /**
       * <code>optional double mktCapPrice = 11;</code>
       * @param value The mktCapPrice to set.
       * @return This builder for chaining.
       */
      public Builder setMktCapPrice(double value) {

        mktCapPrice_ = value;
        bitField0_ |= 0x00000400;
        onChanged();
        return this;
      }
      /**
       * <code>optional double mktCapPrice = 11;</code>
       * @return This builder for chaining.
       */
      public Builder clearMktCapPrice() {
        bitField0_ = (bitField0_ & ~0x00000400);
        mktCapPrice_ = 0D;
        onChanged();
        return this;
      }

      // @@protoc_insertion_point(builder_scope:protobuf.OrderStatus)
    }

    // @@protoc_insertion_point(class_scope:protobuf.OrderStatus)
    private static final com.ib.client.protobuf.OrderStatusProto.OrderStatus DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new com.ib.client.protobuf.OrderStatusProto.OrderStatus();
    }

    public static com.ib.client.protobuf.OrderStatusProto.OrderStatus getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<OrderStatus>
        PARSER = new com.google.protobuf.AbstractParser<OrderStatus>() {
      @java.lang.Override
      public OrderStatus parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        Builder builder = newBuilder();
        try {
          builder.mergeFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.setUnfinishedMessage(builder.buildPartial());
        } catch (com.google.protobuf.UninitializedMessageException e) {
          throw e.asInvalidProtocolBufferException().setUnfinishedMessage(builder.buildPartial());
        } catch (java.io.IOException e) {
          throw new com.google.protobuf.InvalidProtocolBufferException(e)
              .setUnfinishedMessage(builder.buildPartial());
        }
        return builder.buildPartial();
      }
    };

    public static com.google.protobuf.Parser<OrderStatus> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<OrderStatus> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public com.ib.client.protobuf.OrderStatusProto.OrderStatus getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_protobuf_OrderStatus_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessage.FieldAccessorTable
      internal_static_protobuf_OrderStatus_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor
      getDescriptor() {
    return descriptor;
  }
  private static  com.google.protobuf.Descriptors.FileDescriptor
      descriptor;
  static {
    java.lang.String[] descriptorData = {
      "\n\021OrderStatus.proto\022\010protobuf\"\243\003\n\013OrderS" +
      "tatus\022\024\n\007orderId\030\001 \001(\005H\000\210\001\001\022\023\n\006status\030\002 " +
      "\001(\tH\001\210\001\001\022\023\n\006filled\030\003 \001(\tH\002\210\001\001\022\026\n\tremaini" +
      "ng\030\004 \001(\tH\003\210\001\001\022\031\n\014avgFillPrice\030\005 \001(\001H\004\210\001\001" +
      "\022\023\n\006permId\030\006 \001(\003H\005\210\001\001\022\025\n\010parentId\030\007 \001(\005H" +
      "\006\210\001\001\022\032\n\rlastFillPrice\030\010 \001(\001H\007\210\001\001\022\025\n\010clie" +
      "ntId\030\t \001(\005H\010\210\001\001\022\024\n\007whyHeld\030\n \001(\tH\t\210\001\001\022\030\n" +
      "\013mktCapPrice\030\013 \001(\001H\n\210\001\001B\n\n\010_orderIdB\t\n\007_" +
      "statusB\t\n\007_filledB\014\n\n_remainingB\017\n\r_avgF" +
      "illPriceB\t\n\007_permIdB\013\n\t_parentIdB\020\n\016_las" +
      "tFillPriceB\013\n\t_clientIdB\n\n\010_whyHeldB\016\n\014_" +
      "mktCapPriceB;\n\026com.ib.client.protobufB\020O" +
      "rderStatusProto\252\002\016IBApi.protobufb\006proto3"
    };
    descriptor = com.google.protobuf.Descriptors.FileDescriptor
      .internalBuildGeneratedFileFrom(descriptorData,
        new com.google.protobuf.Descriptors.FileDescriptor[] {
        });
    internal_static_protobuf_OrderStatus_descriptor =
      getDescriptor().getMessageTypes().get(0);
    internal_static_protobuf_OrderStatus_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessage.FieldAccessorTable(
        internal_static_protobuf_OrderStatus_descriptor,
        new java.lang.String[] { "OrderId", "Status", "Filled", "Remaining", "AvgFillPrice", "PermId", "ParentId", "LastFillPrice", "ClientId", "WhyHeld", "MktCapPrice", });
    descriptor.resolveAllFeaturesImmutable();
  }

  // @@protoc_insertion_point(outer_class_scope)
}
