// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: DeltaNeutralContract.proto
// Protobuf Java Version: 4.29.3

package com.ib.client.protobuf;

public final class DeltaNeutralContractProto {
  private DeltaNeutralContractProto() {}
  static {
    com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
      com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
      /* major= */ 4,
      /* minor= */ 29,
      /* patch= */ 3,
      /* suffix= */ "",
      DeltaNeutralContractProto.class.getName());
  }
  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistryLite registry) {
  }

  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistry registry) {
    registerAllExtensions(
        (com.google.protobuf.ExtensionRegistryLite) registry);
  }
  public interface DeltaNeutralContractOrBuilder extends
      // @@protoc_insertion_point(interface_extends:protobuf.DeltaNeutralContract)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>optional int32 conId = 1;</code>
     * @return Whether the conId field is set.
     */
    boolean hasConId();
    /**
     * <code>optional int32 conId = 1;</code>
     * @return The conId.
     */
    int getConId();

    /**
     * <code>optional double delta = 2;</code>
     * @return Whether the delta field is set.
     */
    boolean hasDelta();
    /**
     * <code>optional double delta = 2;</code>
     * @return The delta.
     */
    double getDelta();

    /**
     * <code>optional double price = 3;</code>
     * @return Whether the price field is set.
     */
    boolean hasPrice();
    /**
     * <code>optional double price = 3;</code>
     * @return The price.
     */
    double getPrice();
  }
  /**
   * Protobuf type {@code protobuf.DeltaNeutralContract}
   */
  public static final class DeltaNeutralContract extends
      com.google.protobuf.GeneratedMessage implements
      // @@protoc_insertion_point(message_implements:protobuf.DeltaNeutralContract)
      DeltaNeutralContractOrBuilder {
  private static final long serialVersionUID = 0L;
    static {
      com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
        com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
        /* major= */ 4,
        /* minor= */ 29,
        /* patch= */ 3,
        /* suffix= */ "",
        DeltaNeutralContract.class.getName());
    }
    // Use DeltaNeutralContract.newBuilder() to construct.
    private DeltaNeutralContract(com.google.protobuf.GeneratedMessage.Builder<?> builder) {
      super(builder);
    }
    private DeltaNeutralContract() {
    }

    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return com.ib.client.protobuf.DeltaNeutralContractProto.internal_static_protobuf_DeltaNeutralContract_descriptor;
    }

    @java.lang.Override
    protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return com.ib.client.protobuf.DeltaNeutralContractProto.internal_static_protobuf_DeltaNeutralContract_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.class, com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.Builder.class);
    }

    private int bitField0_;
    public static final int CONID_FIELD_NUMBER = 1;
    private int conId_ = 0;
    /**
     * <code>optional int32 conId = 1;</code>
     * @return Whether the conId field is set.
     */
    @java.lang.Override
    public boolean hasConId() {
      return ((bitField0_ & 0x00000001) != 0);
    }
    /**
     * <code>optional int32 conId = 1;</code>
     * @return The conId.
     */
    @java.lang.Override
    public int getConId() {
      return conId_;
    }

    public static final int DELTA_FIELD_NUMBER = 2;
    private double delta_ = 0D;
    /**
     * <code>optional double delta = 2;</code>
     * @return Whether the delta field is set.
     */
    @java.lang.Override
    public boolean hasDelta() {
      return ((bitField0_ & 0x00000002) != 0);
    }
    /**
     * <code>optional double delta = 2;</code>
     * @return The delta.
     */
    @java.lang.Override
    public double getDelta() {
      return delta_;
    }

    public static final int PRICE_FIELD_NUMBER = 3;
    private double price_ = 0D;
    /**
     * <code>optional double price = 3;</code>
     * @return Whether the price field is set.
     */
    @java.lang.Override
    public boolean hasPrice() {
      return ((bitField0_ & 0x00000004) != 0);
    }
    /**
     * <code>optional double price = 3;</code>
     * @return The price.
     */
    @java.lang.Override
    public double getPrice() {
      return price_;
    }

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      if (((bitField0_ & 0x00000001) != 0)) {
        output.writeInt32(1, conId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        output.writeDouble(2, delta_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        output.writeDouble(3, price_);
      }
      getUnknownFields().writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      if (((bitField0_ & 0x00000001) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(1, conId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeDoubleSize(2, delta_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeDoubleSize(3, price_);
      }
      size += getUnknownFields().getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract)) {
        return super.equals(obj);
      }
      com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract other = (com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract) obj;

      if (hasConId() != other.hasConId()) return false;
      if (hasConId()) {
        if (getConId()
            != other.getConId()) return false;
      }
      if (hasDelta() != other.hasDelta()) return false;
      if (hasDelta()) {
        if (java.lang.Double.doubleToLongBits(getDelta())
            != java.lang.Double.doubleToLongBits(
                other.getDelta())) return false;
      }
      if (hasPrice() != other.hasPrice()) return false;
      if (hasPrice()) {
        if (java.lang.Double.doubleToLongBits(getPrice())
            != java.lang.Double.doubleToLongBits(
                other.getPrice())) return false;
      }
      if (!getUnknownFields().equals(other.getUnknownFields())) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      if (hasConId()) {
        hash = (37 * hash) + CONID_FIELD_NUMBER;
        hash = (53 * hash) + getConId();
      }
      if (hasDelta()) {
        hash = (37 * hash) + DELTA_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            java.lang.Double.doubleToLongBits(getDelta()));
      }
      if (hasPrice()) {
        hash = (37 * hash) + PRICE_FIELD_NUMBER;
        hash = (53 * hash) + com.google.protobuf.Internal.hashLong(
            java.lang.Double.doubleToLongBits(getPrice()));
      }
      hash = (29 * hash) + getUnknownFields().hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input);
    }

    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessage.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code protobuf.DeltaNeutralContract}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessage.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:protobuf.DeltaNeutralContract)
        com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContractOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return com.ib.client.protobuf.DeltaNeutralContractProto.internal_static_protobuf_DeltaNeutralContract_descriptor;
      }

      @java.lang.Override
      protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return com.ib.client.protobuf.DeltaNeutralContractProto.internal_static_protobuf_DeltaNeutralContract_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.class, com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.Builder.class);
      }

      // Construct using com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.newBuilder()
      private Builder() {

      }

      private Builder(
          com.google.protobuf.GeneratedMessage.BuilderParent parent) {
        super(parent);

      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        bitField0_ = 0;
        conId_ = 0;
        delta_ = 0D;
        price_ = 0D;
        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return com.ib.client.protobuf.DeltaNeutralContractProto.internal_static_protobuf_DeltaNeutralContract_descriptor;
      }

      @java.lang.Override
      public com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract getDefaultInstanceForType() {
        return com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.getDefaultInstance();
      }

      @java.lang.Override
      public com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract build() {
        com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract buildPartial() {
        com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract result = new com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract(this);
        if (bitField0_ != 0) { buildPartial0(result); }
        onBuilt();
        return result;
      }

      private void buildPartial0(com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract result) {
        int from_bitField0_ = bitField0_;
        int to_bitField0_ = 0;
        if (((from_bitField0_ & 0x00000001) != 0)) {
          result.conId_ = conId_;
          to_bitField0_ |= 0x00000001;
        }
        if (((from_bitField0_ & 0x00000002) != 0)) {
          result.delta_ = delta_;
          to_bitField0_ |= 0x00000002;
        }
        if (((from_bitField0_ & 0x00000004) != 0)) {
          result.price_ = price_;
          to_bitField0_ |= 0x00000004;
        }
        result.bitField0_ |= to_bitField0_;
      }

      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract) {
          return mergeFrom((com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract other) {
        if (other == com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract.getDefaultInstance()) return this;
        if (other.hasConId()) {
          setConId(other.getConId());
        }
        if (other.hasDelta()) {
          setDelta(other.getDelta());
        }
        if (other.hasPrice()) {
          setPrice(other.getPrice());
        }
        this.mergeUnknownFields(other.getUnknownFields());
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        if (extensionRegistry == null) {
          throw new java.lang.NullPointerException();
        }
        try {
          boolean done = false;
          while (!done) {
            int tag = input.readTag();
            switch (tag) {
              case 0:
                done = true;
                break;
              case 8: {
                conId_ = input.readInt32();
                bitField0_ |= 0x00000001;
                break;
              } // case 8
              case 17: {
                delta_ = input.readDouble();
                bitField0_ |= 0x00000002;
                break;
              } // case 17
              case 25: {
                price_ = input.readDouble();
                bitField0_ |= 0x00000004;
                break;
              } // case 25
              default: {
                if (!super.parseUnknownField(input, extensionRegistry, tag)) {
                  done = true; // was an endgroup tag
                }
                break;
              } // default:
            } // switch (tag)
          } // while (!done)
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.unwrapIOException();
        } finally {
          onChanged();
        } // finally
        return this;
      }
      private int bitField0_;

      private int conId_ ;
      /**
       * <code>optional int32 conId = 1;</code>
       * @return Whether the conId field is set.
       */
      @java.lang.Override
      public boolean hasConId() {
        return ((bitField0_ & 0x00000001) != 0);
      }
      /**
       * <code>optional int32 conId = 1;</code>
       * @return The conId.
       */
      @java.lang.Override
      public int getConId() {
        return conId_;
      }
      /**
       * <code>optional int32 conId = 1;</code>
       * @param value The conId to set.
       * @return This builder for chaining.
       */
      public Builder setConId(int value) {

        conId_ = value;
        bitField0_ |= 0x00000001;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 conId = 1;</code>
       * @return This builder for chaining.
       */
      public Builder clearConId() {
        bitField0_ = (bitField0_ & ~0x00000001);
        conId_ = 0;
        onChanged();
        return this;
      }

      private double delta_ ;
      /**
       * <code>optional double delta = 2;</code>
       * @return Whether the delta field is set.
       */
      @java.lang.Override
      public boolean hasDelta() {
        return ((bitField0_ & 0x00000002) != 0);
      }
      /**
       * <code>optional double delta = 2;</code>
       * @return The delta.
       */
      @java.lang.Override
      public double getDelta() {
        return delta_;
      }
      /**
       * <code>optional double delta = 2;</code>
       * @param value The delta to set.
       * @return This builder for chaining.
       */
      public Builder setDelta(double value) {

        delta_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }
      /**
       * <code>optional double delta = 2;</code>
       * @return This builder for chaining.
       */
      public Builder clearDelta() {
        bitField0_ = (bitField0_ & ~0x00000002);
        delta_ = 0D;
        onChanged();
        return this;
      }

      private double price_ ;
      /**
       * <code>optional double price = 3;</code>
       * @return Whether the price field is set.
       */
      @java.lang.Override
      public boolean hasPrice() {
        return ((bitField0_ & 0x00000004) != 0);
      }
      /**
       * <code>optional double price = 3;</code>
       * @return The price.
       */
      @java.lang.Override
      public double getPrice() {
        return price_;
      }
      /**
       * <code>optional double price = 3;</code>
       * @param value The price to set.
       * @return This builder for chaining.
       */
      public Builder setPrice(double value) {

        price_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }
      /**
       * <code>optional double price = 3;</code>
       * @return This builder for chaining.
       */
      public Builder clearPrice() {
        bitField0_ = (bitField0_ & ~0x00000004);
        price_ = 0D;
        onChanged();
        return this;
      }

      // @@protoc_insertion_point(builder_scope:protobuf.DeltaNeutralContract)
    }

    // @@protoc_insertion_point(class_scope:protobuf.DeltaNeutralContract)
    private static final com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract();
    }

    public static com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<DeltaNeutralContract>
        PARSER = new com.google.protobuf.AbstractParser<DeltaNeutralContract>() {
      @java.lang.Override
      public DeltaNeutralContract parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        Builder builder = newBuilder();
        try {
          builder.mergeFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.setUnfinishedMessage(builder.buildPartial());
        } catch (com.google.protobuf.UninitializedMessageException e) {
          throw e.asInvalidProtocolBufferException().setUnfinishedMessage(builder.buildPartial());
        } catch (java.io.IOException e) {
          throw new com.google.protobuf.InvalidProtocolBufferException(e)
              .setUnfinishedMessage(builder.buildPartial());
        }
        return builder.buildPartial();
      }
    };

    public static com.google.protobuf.Parser<DeltaNeutralContract> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<DeltaNeutralContract> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public com.ib.client.protobuf.DeltaNeutralContractProto.DeltaNeutralContract getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_protobuf_DeltaNeutralContract_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessage.FieldAccessorTable
      internal_static_protobuf_DeltaNeutralContract_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor
      getDescriptor() {
    return descriptor;
  }
  private static  com.google.protobuf.Descriptors.FileDescriptor
      descriptor;
  static {
    java.lang.String[] descriptorData = {
      "\n\032DeltaNeutralContract.proto\022\010protobuf\"p" +
      "\n\024DeltaNeutralContract\022\022\n\005conId\030\001 \001(\005H\000\210" +
      "\001\001\022\022\n\005delta\030\002 \001(\001H\001\210\001\001\022\022\n\005price\030\003 \001(\001H\002\210" +
      "\001\001B\010\n\006_conIdB\010\n\006_deltaB\010\n\006_priceBD\n\026com." +
      "ib.client.protobufB\031DeltaNeutralContract" +
      "Proto\252\002\016IBApi.protobufb\006proto3"
    };
    descriptor = com.google.protobuf.Descriptors.FileDescriptor
      .internalBuildGeneratedFileFrom(descriptorData,
        new com.google.protobuf.Descriptors.FileDescriptor[] {
        });
    internal_static_protobuf_DeltaNeutralContract_descriptor =
      getDescriptor().getMessageTypes().get(0);
    internal_static_protobuf_DeltaNeutralContract_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessage.FieldAccessorTable(
        internal_static_protobuf_DeltaNeutralContract_descriptor,
        new java.lang.String[] { "ConId", "Delta", "Price", });
    descriptor.resolveAllFeaturesImmutable();
  }

  // @@protoc_insertion_point(outer_class_scope)
}
