// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: ExecutionFilter.proto
// Protobuf Java Version: 4.29.3

package com.ib.client.protobuf;

public final class ExecutionFilterProto {
  private ExecutionFilterProto() {}
  static {
    com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
      com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
      /* major= */ 4,
      /* minor= */ 29,
      /* patch= */ 3,
      /* suffix= */ "",
      ExecutionFilterProto.class.getName());
  }
  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistryLite registry) {
  }

  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistry registry) {
    registerAllExtensions(
        (com.google.protobuf.ExtensionRegistryLite) registry);
  }
  public interface ExecutionFilterOrBuilder extends
      // @@protoc_insertion_point(interface_extends:protobuf.ExecutionFilter)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>optional int32 clientId = 1;</code>
     * @return Whether the clientId field is set.
     */
    boolean hasClientId();
    /**
     * <code>optional int32 clientId = 1;</code>
     * @return The clientId.
     */
    int getClientId();

    /**
     * <code>optional string acctCode = 2;</code>
     * @return Whether the acctCode field is set.
     */
    boolean hasAcctCode();
    /**
     * <code>optional string acctCode = 2;</code>
     * @return The acctCode.
     */
    java.lang.String getAcctCode();
    /**
     * <code>optional string acctCode = 2;</code>
     * @return The bytes for acctCode.
     */
    com.google.protobuf.ByteString
        getAcctCodeBytes();

    /**
     * <code>optional string time = 3;</code>
     * @return Whether the time field is set.
     */
    boolean hasTime();
    /**
     * <code>optional string time = 3;</code>
     * @return The time.
     */
    java.lang.String getTime();
    /**
     * <code>optional string time = 3;</code>
     * @return The bytes for time.
     */
    com.google.protobuf.ByteString
        getTimeBytes();

    /**
     * <code>optional string symbol = 4;</code>
     * @return Whether the symbol field is set.
     */
    boolean hasSymbol();
    /**
     * <code>optional string symbol = 4;</code>
     * @return The symbol.
     */
    java.lang.String getSymbol();
    /**
     * <code>optional string symbol = 4;</code>
     * @return The bytes for symbol.
     */
    com.google.protobuf.ByteString
        getSymbolBytes();

    /**
     * <code>optional string secType = 5;</code>
     * @return Whether the secType field is set.
     */
    boolean hasSecType();
    /**
     * <code>optional string secType = 5;</code>
     * @return The secType.
     */
    java.lang.String getSecType();
    /**
     * <code>optional string secType = 5;</code>
     * @return The bytes for secType.
     */
    com.google.protobuf.ByteString
        getSecTypeBytes();

    /**
     * <code>optional string exchange = 6;</code>
     * @return Whether the exchange field is set.
     */
    boolean hasExchange();
    /**
     * <code>optional string exchange = 6;</code>
     * @return The exchange.
     */
    java.lang.String getExchange();
    /**
     * <code>optional string exchange = 6;</code>
     * @return The bytes for exchange.
     */
    com.google.protobuf.ByteString
        getExchangeBytes();

    /**
     * <code>optional string side = 7;</code>
     * @return Whether the side field is set.
     */
    boolean hasSide();
    /**
     * <code>optional string side = 7;</code>
     * @return The side.
     */
    java.lang.String getSide();
    /**
     * <code>optional string side = 7;</code>
     * @return The bytes for side.
     */
    com.google.protobuf.ByteString
        getSideBytes();

    /**
     * <code>optional int32 lastNDays = 8;</code>
     * @return Whether the lastNDays field is set.
     */
    boolean hasLastNDays();
    /**
     * <code>optional int32 lastNDays = 8;</code>
     * @return The lastNDays.
     */
    int getLastNDays();

    /**
     * <code>repeated int32 specificDates = 9;</code>
     * @return A list containing the specificDates.
     */
    java.util.List<java.lang.Integer> getSpecificDatesList();
    /**
     * <code>repeated int32 specificDates = 9;</code>
     * @return The count of specificDates.
     */
    int getSpecificDatesCount();
    /**
     * <code>repeated int32 specificDates = 9;</code>
     * @param index The index of the element to return.
     * @return The specificDates at the given index.
     */
    int getSpecificDates(int index);
  }
  /**
   * Protobuf type {@code protobuf.ExecutionFilter}
   */
  public static final class ExecutionFilter extends
      com.google.protobuf.GeneratedMessage implements
      // @@protoc_insertion_point(message_implements:protobuf.ExecutionFilter)
      ExecutionFilterOrBuilder {
  private static final long serialVersionUID = 0L;
    static {
      com.google.protobuf.RuntimeVersion.validateProtobufGencodeVersion(
        com.google.protobuf.RuntimeVersion.RuntimeDomain.PUBLIC,
        /* major= */ 4,
        /* minor= */ 29,
        /* patch= */ 3,
        /* suffix= */ "",
        ExecutionFilter.class.getName());
    }
    // Use ExecutionFilter.newBuilder() to construct.
    private ExecutionFilter(com.google.protobuf.GeneratedMessage.Builder<?> builder) {
      super(builder);
    }
    private ExecutionFilter() {
      acctCode_ = "";
      time_ = "";
      symbol_ = "";
      secType_ = "";
      exchange_ = "";
      side_ = "";
      specificDates_ = emptyIntList();
    }

    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return com.ib.client.protobuf.ExecutionFilterProto.internal_static_protobuf_ExecutionFilter_descriptor;
    }

    @java.lang.Override
    protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return com.ib.client.protobuf.ExecutionFilterProto.internal_static_protobuf_ExecutionFilter_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter.class, com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter.Builder.class);
    }

    private int bitField0_;
    public static final int CLIENTID_FIELD_NUMBER = 1;
    private int clientId_ = 0;
    /**
     * <code>optional int32 clientId = 1;</code>
     * @return Whether the clientId field is set.
     */
    @java.lang.Override
    public boolean hasClientId() {
      return ((bitField0_ & 0x00000001) != 0);
    }
    /**
     * <code>optional int32 clientId = 1;</code>
     * @return The clientId.
     */
    @java.lang.Override
    public int getClientId() {
      return clientId_;
    }

    public static final int ACCTCODE_FIELD_NUMBER = 2;
    @SuppressWarnings("serial")
    private volatile java.lang.Object acctCode_ = "";
    /**
     * <code>optional string acctCode = 2;</code>
     * @return Whether the acctCode field is set.
     */
    @java.lang.Override
    public boolean hasAcctCode() {
      return ((bitField0_ & 0x00000002) != 0);
    }
    /**
     * <code>optional string acctCode = 2;</code>
     * @return The acctCode.
     */
    @java.lang.Override
    public java.lang.String getAcctCode() {
      java.lang.Object ref = acctCode_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        acctCode_ = s;
        return s;
      }
    }
    /**
     * <code>optional string acctCode = 2;</code>
     * @return The bytes for acctCode.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getAcctCodeBytes() {
      java.lang.Object ref = acctCode_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        acctCode_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int TIME_FIELD_NUMBER = 3;
    @SuppressWarnings("serial")
    private volatile java.lang.Object time_ = "";
    /**
     * <code>optional string time = 3;</code>
     * @return Whether the time field is set.
     */
    @java.lang.Override
    public boolean hasTime() {
      return ((bitField0_ & 0x00000004) != 0);
    }
    /**
     * <code>optional string time = 3;</code>
     * @return The time.
     */
    @java.lang.Override
    public java.lang.String getTime() {
      java.lang.Object ref = time_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        time_ = s;
        return s;
      }
    }
    /**
     * <code>optional string time = 3;</code>
     * @return The bytes for time.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getTimeBytes() {
      java.lang.Object ref = time_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        time_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int SYMBOL_FIELD_NUMBER = 4;
    @SuppressWarnings("serial")
    private volatile java.lang.Object symbol_ = "";
    /**
     * <code>optional string symbol = 4;</code>
     * @return Whether the symbol field is set.
     */
    @java.lang.Override
    public boolean hasSymbol() {
      return ((bitField0_ & 0x00000008) != 0);
    }
    /**
     * <code>optional string symbol = 4;</code>
     * @return The symbol.
     */
    @java.lang.Override
    public java.lang.String getSymbol() {
      java.lang.Object ref = symbol_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        symbol_ = s;
        return s;
      }
    }
    /**
     * <code>optional string symbol = 4;</code>
     * @return The bytes for symbol.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getSymbolBytes() {
      java.lang.Object ref = symbol_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        symbol_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int SECTYPE_FIELD_NUMBER = 5;
    @SuppressWarnings("serial")
    private volatile java.lang.Object secType_ = "";
    /**
     * <code>optional string secType = 5;</code>
     * @return Whether the secType field is set.
     */
    @java.lang.Override
    public boolean hasSecType() {
      return ((bitField0_ & 0x00000010) != 0);
    }
    /**
     * <code>optional string secType = 5;</code>
     * @return The secType.
     */
    @java.lang.Override
    public java.lang.String getSecType() {
      java.lang.Object ref = secType_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        secType_ = s;
        return s;
      }
    }
    /**
     * <code>optional string secType = 5;</code>
     * @return The bytes for secType.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getSecTypeBytes() {
      java.lang.Object ref = secType_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        secType_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int EXCHANGE_FIELD_NUMBER = 6;
    @SuppressWarnings("serial")
    private volatile java.lang.Object exchange_ = "";
    /**
     * <code>optional string exchange = 6;</code>
     * @return Whether the exchange field is set.
     */
    @java.lang.Override
    public boolean hasExchange() {
      return ((bitField0_ & 0x00000020) != 0);
    }
    /**
     * <code>optional string exchange = 6;</code>
     * @return The exchange.
     */
    @java.lang.Override
    public java.lang.String getExchange() {
      java.lang.Object ref = exchange_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        exchange_ = s;
        return s;
      }
    }
    /**
     * <code>optional string exchange = 6;</code>
     * @return The bytes for exchange.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getExchangeBytes() {
      java.lang.Object ref = exchange_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        exchange_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int SIDE_FIELD_NUMBER = 7;
    @SuppressWarnings("serial")
    private volatile java.lang.Object side_ = "";
    /**
     * <code>optional string side = 7;</code>
     * @return Whether the side field is set.
     */
    @java.lang.Override
    public boolean hasSide() {
      return ((bitField0_ & 0x00000040) != 0);
    }
    /**
     * <code>optional string side = 7;</code>
     * @return The side.
     */
    @java.lang.Override
    public java.lang.String getSide() {
      java.lang.Object ref = side_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        side_ = s;
        return s;
      }
    }
    /**
     * <code>optional string side = 7;</code>
     * @return The bytes for side.
     */
    @java.lang.Override
    public com.google.protobuf.ByteString
        getSideBytes() {
      java.lang.Object ref = side_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        side_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int LASTNDAYS_FIELD_NUMBER = 8;
    private int lastNDays_ = 0;
    /**
     * <code>optional int32 lastNDays = 8;</code>
     * @return Whether the lastNDays field is set.
     */
    @java.lang.Override
    public boolean hasLastNDays() {
      return ((bitField0_ & 0x00000080) != 0);
    }
    /**
     * <code>optional int32 lastNDays = 8;</code>
     * @return The lastNDays.
     */
    @java.lang.Override
    public int getLastNDays() {
      return lastNDays_;
    }

    public static final int SPECIFICDATES_FIELD_NUMBER = 9;
    @SuppressWarnings("serial")
    private com.google.protobuf.Internal.IntList specificDates_ =
        emptyIntList();
    /**
     * <code>repeated int32 specificDates = 9;</code>
     * @return A list containing the specificDates.
     */
    @java.lang.Override
    public java.util.List<java.lang.Integer>
        getSpecificDatesList() {
      return specificDates_;
    }
    /**
     * <code>repeated int32 specificDates = 9;</code>
     * @return The count of specificDates.
     */
    public int getSpecificDatesCount() {
      return specificDates_.size();
    }
    /**
     * <code>repeated int32 specificDates = 9;</code>
     * @param index The index of the element to return.
     * @return The specificDates at the given index.
     */
    public int getSpecificDates(int index) {
      return specificDates_.getInt(index);
    }
    private int specificDatesMemoizedSerializedSize = -1;

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      getSerializedSize();
      if (((bitField0_ & 0x00000001) != 0)) {
        output.writeInt32(1, clientId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 2, acctCode_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 3, time_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 4, symbol_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 5, secType_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 6, exchange_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        com.google.protobuf.GeneratedMessage.writeString(output, 7, side_);
      }
      if (((bitField0_ & 0x00000080) != 0)) {
        output.writeInt32(8, lastNDays_);
      }
      if (getSpecificDatesList().size() > 0) {
        output.writeUInt32NoTag(74);
        output.writeUInt32NoTag(specificDatesMemoizedSerializedSize);
      }
      for (int i = 0; i < specificDates_.size(); i++) {
        output.writeInt32NoTag(specificDates_.getInt(i));
      }
      getUnknownFields().writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      if (((bitField0_ & 0x00000001) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(1, clientId_);
      }
      if (((bitField0_ & 0x00000002) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(2, acctCode_);
      }
      if (((bitField0_ & 0x00000004) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(3, time_);
      }
      if (((bitField0_ & 0x00000008) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(4, symbol_);
      }
      if (((bitField0_ & 0x00000010) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(5, secType_);
      }
      if (((bitField0_ & 0x00000020) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(6, exchange_);
      }
      if (((bitField0_ & 0x00000040) != 0)) {
        size += com.google.protobuf.GeneratedMessage.computeStringSize(7, side_);
      }
      if (((bitField0_ & 0x00000080) != 0)) {
        size += com.google.protobuf.CodedOutputStream
          .computeInt32Size(8, lastNDays_);
      }
      {
        int dataSize = 0;
        for (int i = 0; i < specificDates_.size(); i++) {
          dataSize += com.google.protobuf.CodedOutputStream
            .computeInt32SizeNoTag(specificDates_.getInt(i));
        }
        size += dataSize;
        if (!getSpecificDatesList().isEmpty()) {
          size += 1;
          size += com.google.protobuf.CodedOutputStream
              .computeInt32SizeNoTag(dataSize);
        }
        specificDatesMemoizedSerializedSize = dataSize;
      }
      size += getUnknownFields().getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter)) {
        return super.equals(obj);
      }
      com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter other = (com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter) obj;

      if (hasClientId() != other.hasClientId()) return false;
      if (hasClientId()) {
        if (getClientId()
            != other.getClientId()) return false;
      }
      if (hasAcctCode() != other.hasAcctCode()) return false;
      if (hasAcctCode()) {
        if (!getAcctCode()
            .equals(other.getAcctCode())) return false;
      }
      if (hasTime() != other.hasTime()) return false;
      if (hasTime()) {
        if (!getTime()
            .equals(other.getTime())) return false;
      }
      if (hasSymbol() != other.hasSymbol()) return false;
      if (hasSymbol()) {
        if (!getSymbol()
            .equals(other.getSymbol())) return false;
      }
      if (hasSecType() != other.hasSecType()) return false;
      if (hasSecType()) {
        if (!getSecType()
            .equals(other.getSecType())) return false;
      }
      if (hasExchange() != other.hasExchange()) return false;
      if (hasExchange()) {
        if (!getExchange()
            .equals(other.getExchange())) return false;
      }
      if (hasSide() != other.hasSide()) return false;
      if (hasSide()) {
        if (!getSide()
            .equals(other.getSide())) return false;
      }
      if (hasLastNDays() != other.hasLastNDays()) return false;
      if (hasLastNDays()) {
        if (getLastNDays()
            != other.getLastNDays()) return false;
      }
      if (!getSpecificDatesList()
          .equals(other.getSpecificDatesList())) return false;
      if (!getUnknownFields().equals(other.getUnknownFields())) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      if (hasClientId()) {
        hash = (37 * hash) + CLIENTID_FIELD_NUMBER;
        hash = (53 * hash) + getClientId();
      }
      if (hasAcctCode()) {
        hash = (37 * hash) + ACCTCODE_FIELD_NUMBER;
        hash = (53 * hash) + getAcctCode().hashCode();
      }
      if (hasTime()) {
        hash = (37 * hash) + TIME_FIELD_NUMBER;
        hash = (53 * hash) + getTime().hashCode();
      }
      if (hasSymbol()) {
        hash = (37 * hash) + SYMBOL_FIELD_NUMBER;
        hash = (53 * hash) + getSymbol().hashCode();
      }
      if (hasSecType()) {
        hash = (37 * hash) + SECTYPE_FIELD_NUMBER;
        hash = (53 * hash) + getSecType().hashCode();
      }
      if (hasExchange()) {
        hash = (37 * hash) + EXCHANGE_FIELD_NUMBER;
        hash = (53 * hash) + getExchange().hashCode();
      }
      if (hasSide()) {
        hash = (37 * hash) + SIDE_FIELD_NUMBER;
        hash = (53 * hash) + getSide().hashCode();
      }
      if (hasLastNDays()) {
        hash = (37 * hash) + LASTNDAYS_FIELD_NUMBER;
        hash = (53 * hash) + getLastNDays();
      }
      if (getSpecificDatesCount() > 0) {
        hash = (37 * hash) + SPECIFICDATES_FIELD_NUMBER;
        hash = (53 * hash) + getSpecificDatesList().hashCode();
      }
      hash = (29 * hash) + getUnknownFields().hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input);
    }

    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input);
    }
    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessage
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessage.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code protobuf.ExecutionFilter}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessage.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:protobuf.ExecutionFilter)
        com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilterOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return com.ib.client.protobuf.ExecutionFilterProto.internal_static_protobuf_ExecutionFilter_descriptor;
      }

      @java.lang.Override
      protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return com.ib.client.protobuf.ExecutionFilterProto.internal_static_protobuf_ExecutionFilter_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter.class, com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter.Builder.class);
      }

      // Construct using com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter.newBuilder()
      private Builder() {

      }

      private Builder(
          com.google.protobuf.GeneratedMessage.BuilderParent parent) {
        super(parent);

      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        bitField0_ = 0;
        clientId_ = 0;
        acctCode_ = "";
        time_ = "";
        symbol_ = "";
        secType_ = "";
        exchange_ = "";
        side_ = "";
        lastNDays_ = 0;
        specificDates_ = emptyIntList();
        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return com.ib.client.protobuf.ExecutionFilterProto.internal_static_protobuf_ExecutionFilter_descriptor;
      }

      @java.lang.Override
      public com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter getDefaultInstanceForType() {
        return com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter.getDefaultInstance();
      }

      @java.lang.Override
      public com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter build() {
        com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter buildPartial() {
        com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter result = new com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter(this);
        if (bitField0_ != 0) { buildPartial0(result); }
        onBuilt();
        return result;
      }

      private void buildPartial0(com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter result) {
        int from_bitField0_ = bitField0_;
        int to_bitField0_ = 0;
        if (((from_bitField0_ & 0x00000001) != 0)) {
          result.clientId_ = clientId_;
          to_bitField0_ |= 0x00000001;
        }
        if (((from_bitField0_ & 0x00000002) != 0)) {
          result.acctCode_ = acctCode_;
          to_bitField0_ |= 0x00000002;
        }
        if (((from_bitField0_ & 0x00000004) != 0)) {
          result.time_ = time_;
          to_bitField0_ |= 0x00000004;
        }
        if (((from_bitField0_ & 0x00000008) != 0)) {
          result.symbol_ = symbol_;
          to_bitField0_ |= 0x00000008;
        }
        if (((from_bitField0_ & 0x00000010) != 0)) {
          result.secType_ = secType_;
          to_bitField0_ |= 0x00000010;
        }
        if (((from_bitField0_ & 0x00000020) != 0)) {
          result.exchange_ = exchange_;
          to_bitField0_ |= 0x00000020;
        }
        if (((from_bitField0_ & 0x00000040) != 0)) {
          result.side_ = side_;
          to_bitField0_ |= 0x00000040;
        }
        if (((from_bitField0_ & 0x00000080) != 0)) {
          result.lastNDays_ = lastNDays_;
          to_bitField0_ |= 0x00000080;
        }
        if (((from_bitField0_ & 0x00000100) != 0)) {
          specificDates_.makeImmutable();
          result.specificDates_ = specificDates_;
        }
        result.bitField0_ |= to_bitField0_;
      }

      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter) {
          return mergeFrom((com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter other) {
        if (other == com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter.getDefaultInstance()) return this;
        if (other.hasClientId()) {
          setClientId(other.getClientId());
        }
        if (other.hasAcctCode()) {
          acctCode_ = other.acctCode_;
          bitField0_ |= 0x00000002;
          onChanged();
        }
        if (other.hasTime()) {
          time_ = other.time_;
          bitField0_ |= 0x00000004;
          onChanged();
        }
        if (other.hasSymbol()) {
          symbol_ = other.symbol_;
          bitField0_ |= 0x00000008;
          onChanged();
        }
        if (other.hasSecType()) {
          secType_ = other.secType_;
          bitField0_ |= 0x00000010;
          onChanged();
        }
        if (other.hasExchange()) {
          exchange_ = other.exchange_;
          bitField0_ |= 0x00000020;
          onChanged();
        }
        if (other.hasSide()) {
          side_ = other.side_;
          bitField0_ |= 0x00000040;
          onChanged();
        }
        if (other.hasLastNDays()) {
          setLastNDays(other.getLastNDays());
        }
        if (!other.specificDates_.isEmpty()) {
          if (specificDates_.isEmpty()) {
            specificDates_ = other.specificDates_;
            specificDates_.makeImmutable();
            bitField0_ |= 0x00000100;
          } else {
            ensureSpecificDatesIsMutable();
            specificDates_.addAll(other.specificDates_);
          }
          onChanged();
        }
        this.mergeUnknownFields(other.getUnknownFields());
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        if (extensionRegistry == null) {
          throw new java.lang.NullPointerException();
        }
        try {
          boolean done = false;
          while (!done) {
            int tag = input.readTag();
            switch (tag) {
              case 0:
                done = true;
                break;
              case 8: {
                clientId_ = input.readInt32();
                bitField0_ |= 0x00000001;
                break;
              } // case 8
              case 18: {
                acctCode_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000002;
                break;
              } // case 18
              case 26: {
                time_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000004;
                break;
              } // case 26
              case 34: {
                symbol_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000008;
                break;
              } // case 34
              case 42: {
                secType_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000010;
                break;
              } // case 42
              case 50: {
                exchange_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000020;
                break;
              } // case 50
              case 58: {
                side_ = input.readStringRequireUtf8();
                bitField0_ |= 0x00000040;
                break;
              } // case 58
              case 64: {
                lastNDays_ = input.readInt32();
                bitField0_ |= 0x00000080;
                break;
              } // case 64
              case 72: {
                int v = input.readInt32();
                ensureSpecificDatesIsMutable();
                specificDates_.addInt(v);
                break;
              } // case 72
              case 74: {
                int length = input.readRawVarint32();
                int limit = input.pushLimit(length);
                ensureSpecificDatesIsMutable();
                while (input.getBytesUntilLimit() > 0) {
                  specificDates_.addInt(input.readInt32());
                }
                input.popLimit(limit);
                break;
              } // case 74
              default: {
                if (!super.parseUnknownField(input, extensionRegistry, tag)) {
                  done = true; // was an endgroup tag
                }
                break;
              } // default:
            } // switch (tag)
          } // while (!done)
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.unwrapIOException();
        } finally {
          onChanged();
        } // finally
        return this;
      }
      private int bitField0_;

      private int clientId_ ;
      /**
       * <code>optional int32 clientId = 1;</code>
       * @return Whether the clientId field is set.
       */
      @java.lang.Override
      public boolean hasClientId() {
        return ((bitField0_ & 0x00000001) != 0);
      }
      /**
       * <code>optional int32 clientId = 1;</code>
       * @return The clientId.
       */
      @java.lang.Override
      public int getClientId() {
        return clientId_;
      }
      /**
       * <code>optional int32 clientId = 1;</code>
       * @param value The clientId to set.
       * @return This builder for chaining.
       */
      public Builder setClientId(int value) {

        clientId_ = value;
        bitField0_ |= 0x00000001;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 clientId = 1;</code>
       * @return This builder for chaining.
       */
      public Builder clearClientId() {
        bitField0_ = (bitField0_ & ~0x00000001);
        clientId_ = 0;
        onChanged();
        return this;
      }

      private java.lang.Object acctCode_ = "";
      /**
       * <code>optional string acctCode = 2;</code>
       * @return Whether the acctCode field is set.
       */
      public boolean hasAcctCode() {
        return ((bitField0_ & 0x00000002) != 0);
      }
      /**
       * <code>optional string acctCode = 2;</code>
       * @return The acctCode.
       */
      public java.lang.String getAcctCode() {
        java.lang.Object ref = acctCode_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          acctCode_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string acctCode = 2;</code>
       * @return The bytes for acctCode.
       */
      public com.google.protobuf.ByteString
          getAcctCodeBytes() {
        java.lang.Object ref = acctCode_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          acctCode_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string acctCode = 2;</code>
       * @param value The acctCode to set.
       * @return This builder for chaining.
       */
      public Builder setAcctCode(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        acctCode_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }
      /**
       * <code>optional string acctCode = 2;</code>
       * @return This builder for chaining.
       */
      public Builder clearAcctCode() {
        acctCode_ = getDefaultInstance().getAcctCode();
        bitField0_ = (bitField0_ & ~0x00000002);
        onChanged();
        return this;
      }
      /**
       * <code>optional string acctCode = 2;</code>
       * @param value The bytes for acctCode to set.
       * @return This builder for chaining.
       */
      public Builder setAcctCodeBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        acctCode_ = value;
        bitField0_ |= 0x00000002;
        onChanged();
        return this;
      }

      private java.lang.Object time_ = "";
      /**
       * <code>optional string time = 3;</code>
       * @return Whether the time field is set.
       */
      public boolean hasTime() {
        return ((bitField0_ & 0x00000004) != 0);
      }
      /**
       * <code>optional string time = 3;</code>
       * @return The time.
       */
      public java.lang.String getTime() {
        java.lang.Object ref = time_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          time_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string time = 3;</code>
       * @return The bytes for time.
       */
      public com.google.protobuf.ByteString
          getTimeBytes() {
        java.lang.Object ref = time_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          time_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string time = 3;</code>
       * @param value The time to set.
       * @return This builder for chaining.
       */
      public Builder setTime(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        time_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }
      /**
       * <code>optional string time = 3;</code>
       * @return This builder for chaining.
       */
      public Builder clearTime() {
        time_ = getDefaultInstance().getTime();
        bitField0_ = (bitField0_ & ~0x00000004);
        onChanged();
        return this;
      }
      /**
       * <code>optional string time = 3;</code>
       * @param value The bytes for time to set.
       * @return This builder for chaining.
       */
      public Builder setTimeBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        time_ = value;
        bitField0_ |= 0x00000004;
        onChanged();
        return this;
      }

      private java.lang.Object symbol_ = "";
      /**
       * <code>optional string symbol = 4;</code>
       * @return Whether the symbol field is set.
       */
      public boolean hasSymbol() {
        return ((bitField0_ & 0x00000008) != 0);
      }
      /**
       * <code>optional string symbol = 4;</code>
       * @return The symbol.
       */
      public java.lang.String getSymbol() {
        java.lang.Object ref = symbol_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          symbol_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string symbol = 4;</code>
       * @return The bytes for symbol.
       */
      public com.google.protobuf.ByteString
          getSymbolBytes() {
        java.lang.Object ref = symbol_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          symbol_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string symbol = 4;</code>
       * @param value The symbol to set.
       * @return This builder for chaining.
       */
      public Builder setSymbol(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        symbol_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }
      /**
       * <code>optional string symbol = 4;</code>
       * @return This builder for chaining.
       */
      public Builder clearSymbol() {
        symbol_ = getDefaultInstance().getSymbol();
        bitField0_ = (bitField0_ & ~0x00000008);
        onChanged();
        return this;
      }
      /**
       * <code>optional string symbol = 4;</code>
       * @param value The bytes for symbol to set.
       * @return This builder for chaining.
       */
      public Builder setSymbolBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        symbol_ = value;
        bitField0_ |= 0x00000008;
        onChanged();
        return this;
      }

      private java.lang.Object secType_ = "";
      /**
       * <code>optional string secType = 5;</code>
       * @return Whether the secType field is set.
       */
      public boolean hasSecType() {
        return ((bitField0_ & 0x00000010) != 0);
      }
      /**
       * <code>optional string secType = 5;</code>
       * @return The secType.
       */
      public java.lang.String getSecType() {
        java.lang.Object ref = secType_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          secType_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string secType = 5;</code>
       * @return The bytes for secType.
       */
      public com.google.protobuf.ByteString
          getSecTypeBytes() {
        java.lang.Object ref = secType_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          secType_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string secType = 5;</code>
       * @param value The secType to set.
       * @return This builder for chaining.
       */
      public Builder setSecType(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        secType_ = value;
        bitField0_ |= 0x00000010;
        onChanged();
        return this;
      }
      /**
       * <code>optional string secType = 5;</code>
       * @return This builder for chaining.
       */
      public Builder clearSecType() {
        secType_ = getDefaultInstance().getSecType();
        bitField0_ = (bitField0_ & ~0x00000010);
        onChanged();
        return this;
      }
      /**
       * <code>optional string secType = 5;</code>
       * @param value The bytes for secType to set.
       * @return This builder for chaining.
       */
      public Builder setSecTypeBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        secType_ = value;
        bitField0_ |= 0x00000010;
        onChanged();
        return this;
      }

      private java.lang.Object exchange_ = "";
      /**
       * <code>optional string exchange = 6;</code>
       * @return Whether the exchange field is set.
       */
      public boolean hasExchange() {
        return ((bitField0_ & 0x00000020) != 0);
      }
      /**
       * <code>optional string exchange = 6;</code>
       * @return The exchange.
       */
      public java.lang.String getExchange() {
        java.lang.Object ref = exchange_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          exchange_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string exchange = 6;</code>
       * @return The bytes for exchange.
       */
      public com.google.protobuf.ByteString
          getExchangeBytes() {
        java.lang.Object ref = exchange_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          exchange_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string exchange = 6;</code>
       * @param value The exchange to set.
       * @return This builder for chaining.
       */
      public Builder setExchange(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        exchange_ = value;
        bitField0_ |= 0x00000020;
        onChanged();
        return this;
      }
      /**
       * <code>optional string exchange = 6;</code>
       * @return This builder for chaining.
       */
      public Builder clearExchange() {
        exchange_ = getDefaultInstance().getExchange();
        bitField0_ = (bitField0_ & ~0x00000020);
        onChanged();
        return this;
      }
      /**
       * <code>optional string exchange = 6;</code>
       * @param value The bytes for exchange to set.
       * @return This builder for chaining.
       */
      public Builder setExchangeBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        exchange_ = value;
        bitField0_ |= 0x00000020;
        onChanged();
        return this;
      }

      private java.lang.Object side_ = "";
      /**
       * <code>optional string side = 7;</code>
       * @return Whether the side field is set.
       */
      public boolean hasSide() {
        return ((bitField0_ & 0x00000040) != 0);
      }
      /**
       * <code>optional string side = 7;</code>
       * @return The side.
       */
      public java.lang.String getSide() {
        java.lang.Object ref = side_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          side_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>optional string side = 7;</code>
       * @return The bytes for side.
       */
      public com.google.protobuf.ByteString
          getSideBytes() {
        java.lang.Object ref = side_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          side_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>optional string side = 7;</code>
       * @param value The side to set.
       * @return This builder for chaining.
       */
      public Builder setSide(
          java.lang.String value) {
        if (value == null) { throw new NullPointerException(); }
        side_ = value;
        bitField0_ |= 0x00000040;
        onChanged();
        return this;
      }
      /**
       * <code>optional string side = 7;</code>
       * @return This builder for chaining.
       */
      public Builder clearSide() {
        side_ = getDefaultInstance().getSide();
        bitField0_ = (bitField0_ & ~0x00000040);
        onChanged();
        return this;
      }
      /**
       * <code>optional string side = 7;</code>
       * @param value The bytes for side to set.
       * @return This builder for chaining.
       */
      public Builder setSideBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) { throw new NullPointerException(); }
        checkByteStringIsUtf8(value);
        side_ = value;
        bitField0_ |= 0x00000040;
        onChanged();
        return this;
      }

      private int lastNDays_ ;
      /**
       * <code>optional int32 lastNDays = 8;</code>
       * @return Whether the lastNDays field is set.
       */
      @java.lang.Override
      public boolean hasLastNDays() {
        return ((bitField0_ & 0x00000080) != 0);
      }
      /**
       * <code>optional int32 lastNDays = 8;</code>
       * @return The lastNDays.
       */
      @java.lang.Override
      public int getLastNDays() {
        return lastNDays_;
      }
      /**
       * <code>optional int32 lastNDays = 8;</code>
       * @param value The lastNDays to set.
       * @return This builder for chaining.
       */
      public Builder setLastNDays(int value) {

        lastNDays_ = value;
        bitField0_ |= 0x00000080;
        onChanged();
        return this;
      }
      /**
       * <code>optional int32 lastNDays = 8;</code>
       * @return This builder for chaining.
       */
      public Builder clearLastNDays() {
        bitField0_ = (bitField0_ & ~0x00000080);
        lastNDays_ = 0;
        onChanged();
        return this;
      }

      private com.google.protobuf.Internal.IntList specificDates_ = emptyIntList();
      private void ensureSpecificDatesIsMutable() {
        if (!specificDates_.isModifiable()) {
          specificDates_ = makeMutableCopy(specificDates_);
        }
        bitField0_ |= 0x00000100;
      }
      /**
       * <code>repeated int32 specificDates = 9;</code>
       * @return A list containing the specificDates.
       */
      public java.util.List<java.lang.Integer>
          getSpecificDatesList() {
        specificDates_.makeImmutable();
        return specificDates_;
      }
      /**
       * <code>repeated int32 specificDates = 9;</code>
       * @return The count of specificDates.
       */
      public int getSpecificDatesCount() {
        return specificDates_.size();
      }
      /**
       * <code>repeated int32 specificDates = 9;</code>
       * @param index The index of the element to return.
       * @return The specificDates at the given index.
       */
      public int getSpecificDates(int index) {
        return specificDates_.getInt(index);
      }
      /**
       * <code>repeated int32 specificDates = 9;</code>
       * @param index The index to set the value at.
       * @param value The specificDates to set.
       * @return This builder for chaining.
       */
      public Builder setSpecificDates(
          int index, int value) {

        ensureSpecificDatesIsMutable();
        specificDates_.setInt(index, value);
        bitField0_ |= 0x00000100;
        onChanged();
        return this;
      }
      /**
       * <code>repeated int32 specificDates = 9;</code>
       * @param value The specificDates to add.
       * @return This builder for chaining.
       */
      public Builder addSpecificDates(int value) {

        ensureSpecificDatesIsMutable();
        specificDates_.addInt(value);
        bitField0_ |= 0x00000100;
        onChanged();
        return this;
      }
      /**
       * <code>repeated int32 specificDates = 9;</code>
       * @param values The specificDates to add.
       * @return This builder for chaining.
       */
      public Builder addAllSpecificDates(
          java.lang.Iterable<? extends java.lang.Integer> values) {
        ensureSpecificDatesIsMutable();
        com.google.protobuf.AbstractMessageLite.Builder.addAll(
            values, specificDates_);
        bitField0_ |= 0x00000100;
        onChanged();
        return this;
      }
      /**
       * <code>repeated int32 specificDates = 9;</code>
       * @return This builder for chaining.
       */
      public Builder clearSpecificDates() {
        specificDates_ = emptyIntList();
        bitField0_ = (bitField0_ & ~0x00000100);
        onChanged();
        return this;
      }

      // @@protoc_insertion_point(builder_scope:protobuf.ExecutionFilter)
    }

    // @@protoc_insertion_point(class_scope:protobuf.ExecutionFilter)
    private static final com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter();
    }

    public static com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<ExecutionFilter>
        PARSER = new com.google.protobuf.AbstractParser<ExecutionFilter>() {
      @java.lang.Override
      public ExecutionFilter parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        Builder builder = newBuilder();
        try {
          builder.mergeFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          throw e.setUnfinishedMessage(builder.buildPartial());
        } catch (com.google.protobuf.UninitializedMessageException e) {
          throw e.asInvalidProtocolBufferException().setUnfinishedMessage(builder.buildPartial());
        } catch (java.io.IOException e) {
          throw new com.google.protobuf.InvalidProtocolBufferException(e)
              .setUnfinishedMessage(builder.buildPartial());
        }
        return builder.buildPartial();
      }
    };

    public static com.google.protobuf.Parser<ExecutionFilter> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<ExecutionFilter> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public com.ib.client.protobuf.ExecutionFilterProto.ExecutionFilter getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_protobuf_ExecutionFilter_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessage.FieldAccessorTable
      internal_static_protobuf_ExecutionFilter_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor
      getDescriptor() {
    return descriptor;
  }
  private static  com.google.protobuf.Descriptors.FileDescriptor
      descriptor;
  static {
    java.lang.String[] descriptorData = {
      "\n\025ExecutionFilter.proto\022\010protobuf\"\264\002\n\017Ex" +
      "ecutionFilter\022\025\n\010clientId\030\001 \001(\005H\000\210\001\001\022\025\n\010" +
      "acctCode\030\002 \001(\tH\001\210\001\001\022\021\n\004time\030\003 \001(\tH\002\210\001\001\022\023" +
      "\n\006symbol\030\004 \001(\tH\003\210\001\001\022\024\n\007secType\030\005 \001(\tH\004\210\001" +
      "\001\022\025\n\010exchange\030\006 \001(\tH\005\210\001\001\022\021\n\004side\030\007 \001(\tH\006" +
      "\210\001\001\022\026\n\tlastNDays\030\010 \001(\005H\007\210\001\001\022\025\n\rspecificD" +
      "ates\030\t \003(\005B\013\n\t_clientIdB\013\n\t_acctCodeB\007\n\005" +
      "_timeB\t\n\007_symbolB\n\n\010_secTypeB\013\n\t_exchang" +
      "eB\007\n\005_sideB\014\n\n_lastNDaysB?\n\026com.ib.clien" +
      "t.protobufB\024ExecutionFilterProto\252\002\016IBApi" +
      ".protobufb\006proto3"
    };
    descriptor = com.google.protobuf.Descriptors.FileDescriptor
      .internalBuildGeneratedFileFrom(descriptorData,
        new com.google.protobuf.Descriptors.FileDescriptor[] {
        });
    internal_static_protobuf_ExecutionFilter_descriptor =
      getDescriptor().getMessageTypes().get(0);
    internal_static_protobuf_ExecutionFilter_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessage.FieldAccessorTable(
        internal_static_protobuf_ExecutionFilter_descriptor,
        new java.lang.String[] { "ClientId", "AcctCode", "Time", "Symbol", "SecType", "Exchange", "Side", "LastNDays", "SpecificDates", });
    descriptor.resolveAllFeaturesImmutable();
  }

  // @@protoc_insertion_point(outer_class_scope)
}
