# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: OrderState.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'OrderState.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ibapi.protobuf.OrderAllocation_pb2 as OrderAllocation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10OrderState.proto\x12\x08protobuf\x1a\x15OrderAllocation.proto\"\xf5\r\n\nOrderState\x12\x13\n\x06status\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1d\n\x10initMarginBefore\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x1e\n\x11maintMarginBefore\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12!\n\x14\x65quityWithLoanBefore\x18\x04 \x01(\x01H\x03\x88\x01\x01\x12\x1d\n\x10initMarginChange\x18\x05 \x01(\x01H\x04\x88\x01\x01\x12\x1e\n\x11maintMarginChange\x18\x06 \x01(\x01H\x05\x88\x01\x01\x12!\n\x14\x65quityWithLoanChange\x18\x07 \x01(\x01H\x06\x88\x01\x01\x12\x1c\n\x0finitMarginAfter\x18\x08 \x01(\x01H\x07\x88\x01\x01\x12\x1d\n\x10maintMarginAfter\x18\t \x01(\x01H\x08\x88\x01\x01\x12 \n\x13\x65quityWithLoanAfter\x18\n \x01(\x01H\t\x88\x01\x01\x12\x1e\n\x11\x63ommissionAndFees\x18\x0b \x01(\x01H\n\x88\x01\x01\x12!\n\x14minCommissionAndFees\x18\x0c \x01(\x01H\x0b\x88\x01\x01\x12!\n\x14maxCommissionAndFees\x18\r \x01(\x01H\x0c\x88\x01\x01\x12&\n\x19\x63ommissionAndFeesCurrency\x18\x0e \x01(\tH\r\x88\x01\x01\x12\x1b\n\x0emarginCurrency\x18\x0f \x01(\tH\x0e\x88\x01\x01\x12\'\n\x1ainitMarginBeforeOutsideRTH\x18\x10 \x01(\x01H\x0f\x88\x01\x01\x12(\n\x1bmaintMarginBeforeOutsideRTH\x18\x11 \x01(\x01H\x10\x88\x01\x01\x12+\n\x1e\x65quityWithLoanBeforeOutsideRTH\x18\x12 \x01(\x01H\x11\x88\x01\x01\x12\'\n\x1ainitMarginChangeOutsideRTH\x18\x13 \x01(\x01H\x12\x88\x01\x01\x12(\n\x1bmaintMarginChangeOutsideRTH\x18\x14 \x01(\x01H\x13\x88\x01\x01\x12+\n\x1e\x65quityWithLoanChangeOutsideRTH\x18\x15 \x01(\x01H\x14\x88\x01\x01\x12&\n\x19initMarginAfterOutsideRTH\x18\x16 \x01(\x01H\x15\x88\x01\x01\x12\'\n\x1amaintMarginAfterOutsideRTH\x18\x17 \x01(\x01H\x16\x88\x01\x01\x12*\n\x1d\x65quityWithLoanAfterOutsideRTH\x18\x18 \x01(\x01H\x17\x88\x01\x01\x12\x1a\n\rsuggestedSize\x18\x19 \x01(\tH\x18\x88\x01\x01\x12\x19\n\x0crejectReason\x18\x1a \x01(\tH\x19\x88\x01\x01\x12\x33\n\x10orderAllocations\x18\x1b \x03(\x0b\x32\x19.protobuf.OrderAllocation\x12\x18\n\x0bwarningText\x18\x1c \x01(\tH\x1a\x88\x01\x01\x12\x1a\n\rcompletedTime\x18\x1d \x01(\tH\x1b\x88\x01\x01\x12\x1c\n\x0f\x63ompletedStatus\x18\x1e \x01(\tH\x1c\x88\x01\x01\x42\t\n\x07_statusB\x13\n\x11_initMarginBeforeB\x14\n\x12_maintMarginBeforeB\x17\n\x15_equityWithLoanBeforeB\x13\n\x11_initMarginChangeB\x14\n\x12_maintMarginChangeB\x17\n\x15_equityWithLoanChangeB\x12\n\x10_initMarginAfterB\x13\n\x11_maintMarginAfterB\x16\n\x14_equityWithLoanAfterB\x14\n\x12_commissionAndFeesB\x17\n\x15_minCommissionAndFeesB\x17\n\x15_maxCommissionAndFeesB\x1c\n\x1a_commissionAndFeesCurrencyB\x11\n\x0f_marginCurrencyB\x1d\n\x1b_initMarginBeforeOutsideRTHB\x1e\n\x1c_maintMarginBeforeOutsideRTHB!\n\x1f_equityWithLoanBeforeOutsideRTHB\x1d\n\x1b_initMarginChangeOutsideRTHB\x1e\n\x1c_maintMarginChangeOutsideRTHB!\n\x1f_equityWithLoanChangeOutsideRTHB\x1c\n\x1a_initMarginAfterOutsideRTHB\x1d\n\x1b_maintMarginAfterOutsideRTHB \n\x1e_equityWithLoanAfterOutsideRTHB\x10\n\x0e_suggestedSizeB\x0f\n\r_rejectReasonB\x0e\n\x0c_warningTextB\x10\n\x0e_completedTimeB\x12\n\x10_completedStatusB:\n\x16\x63om.ib.client.protobufB\x0fOrderStateProto\xaa\x02\x0eIBApi.protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'OrderState_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.ib.client.protobufB\017OrderStateProto\252\002\016IBApi.protobuf'
  _globals['_ORDERSTATE']._serialized_start=54
  _globals['_ORDERSTATE']._serialized_end=1835
# @@protoc_insertion_point(module_scope)
